/***************************************************************************/
/**                 BLINK-LED ALS NI-TASK (MODULAR-4/486)                 **/
/**                                                                       **/
/**  Dieses Programm lt entweder die On-Board- oder die Externe-LED     **/
/**  blinken. Es soll als Beispiel zur Erstellung von Echtzeitprogrammen  **/
/**  dienen. Weitere Hinweise entnehmen Sie bitte dem Benutzerhandbuch    **/
/**  bzw. dem Technischen-Handbuch                                        **/
/**                                                                       **/
/**  Installationshinweise:                                               **/
/**                                                                       **/
/**  Programmnummer  : 300h                                               **/
/**  Tasknummer      : beliebig                                           **/
/**  Interruptquelle : 1 (wird nicht verwendet, da NI-Task)               **/
/**  Datenbereich    : 0                                                  **/
/**  Flags           : 0980h (nicht aktivieren, Auto-Init aufrufen        **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**                     Fr Borland C++ (Version 3.1)                     **/
/**                                                                       **/
/**                             Version 1.F.                              **/
/**                                                                       **/
/**                   Erstellungsdatum : 24.09.92                         **/
/**                    letzte nderung : 06.06.2000 (HK)                  **/
/**                             PDT auf neustem Stand                     **/
/**                             Status : freigegeben                      **/
/**                              Autor : M.Heisig                         **/
/**                                                                       **/
/**            (c) 1992 - 1996 SORCUS Computer GmbH Heidelberg            **/
/**                                                                       **/
/***************************************************************************/

#pragma option -r-           /* Keine Registervariablen!! */

#include "ml8rtbib.h"        /* Header fr die Echtzeitbibliothek einbinden */
/* #include "ml8makro.h" */  /* Makro-Header fr die wichtigsten Prozeduren einbinden (Turbo-Assembler erforderlich!) */
#include "dos.h"
#include "stdlib.h"

			     /* ---- Allgemeine Angaben ---- */
#define PROGRAMM_NUMBER 0x300             /* Programmnummer = 300h */
#define VERSION         '1'               /* Version des Programms */
#define REVISION        'F'               /* Revision des Programms */

			     /* ---- Die verschiedenen Statusmglichkeiten des Programms ---- */
#define READY           0                 /* Bereit */
#define RUNNING         2                 /* Programm luft */
#define ERROR           3                 /* Falsche LED angewhlt */
#define STOPPED         4                 /* Programmlauf angehalten */

			     /* ---- Zustnde der LED ---- */
#define  ON             1
#define  OFF            0

			     /* ---- Benutzte LED ---- */
#define INTERN          1                 /* Interne LED */
#define EXTER           2                 /* Externe LED */


			     /* ---- Aufbau der Programm-Descriptor-Tabelle (PDT) ---- */
struct pdt_type
{
   ML8_PDT_HEAD pdt_head;                    /* PDT-Header */

   void* PFAR   main_proc;                   /* Adresse der Haupt-Prozedur (Task) */
   void* PFAR   auto_init;                   /* Adresse der Auto-Init Prozedur */
   void* PFAR   start;                       /* Adresse der Start-Prozedur */
   void* PFAR   stop;                        /* Adresse der Stop-Prozedur */
} pdt;

			     /* ---- Aufbau des Parameterbereichs ---- */
struct parameter_type
{
   tdt_type     tdt;                         /* Platz fr die Task-Descriptor-Tabelle */
   byte         status;                      /* Status des Programms (READY etc.) */
   ushort       blink_rate;                  /* Blinkrate */
   byte         led_status;                  /* Zustand der LED */
   byte         led;                         /* Welche LED 1=intern, 2=extern */
} parameter;


ushort          pause;           /* Ein Zhler, der die Durchlufe des Programms zhlt */


void PFAR auto_init(void);                     /* Prototypen der benutzten Prozeduren */
void PFAR start (void);
void PFAR stop (void);
void PFAR main_task(void);

void PFAR auto_init(void)         /* ---- AUTO-INITIALISIERUNG ---- */
{
   ml8rt_entry();                         /* Register retten und vorbereiten */
   parameter.status = READY;              /* Status des Programms auf "BEREIT" */
   parameter.blink_rate = 10000;          /* Blinkrate auf einen Defaultwert einstellen */
   parameter.led_status = OFF;            /* Status der LED auf "AUS" */
   parameter.led = INTERN;                /* Standartmig die interne LED benutzen */
   ml8rt_external_led_off();              /* Die externe LED ausschalten */
   ml8rt_local_led_off();                 /* Die On-Board-LED ausschalten */
   pause = 0;                             /* Durchlaufzhler auf Null */
   ml8rt_exit();                          /* Register wieder restaurieren */
};


void PFAR start(void)             /* ---- Start-Prozedur ---- */
{
   ml8rt_entry();
   parameter.status = RUNNING;            /* Status auf "Programm luft" */
   if ((parameter.led != INTERN) && (parameter.led != EXTER))
   {                                      /* Als erstes die angewhlte LED berprfen */
      parameter.status = ERROR;           /* Wenn sie nicht stimmt, eine Fehlermeldung im Status-Parameter bergeben */
   }
   else
   {
      parameter.status = RUNNING;         /* Falls alles OK, Task aktivieren (starten) */
      ml8rt_wakeup_task (parameter.tdt.task);
   }
   ml8rt_exit();
}


void PFAR stop(void)              /* ---- Stop-Prozedur ---- */
{
   ml8rt_entry();                        /* Programm deaktivieren (stoppen) */
   ml8rt_sleep_task (parameter.tdt.task);
   parameter.status = STOPPED;
   ml8rt_exit();
}


void PFAR main_task()             /* ---- HAUPTPROZEDUR DER TASK ---- */
{
   ml8rt_entry();
   if (++pause >= parameter.blink_rate)
   {                                   /* Falls die LED eingeschaltet ist, ... */
      if (parameter.led_status == ON)
      {                                /* ... LED ausschalten und ... */
	 if (parameter.led == EXTER)
	    ml8rt_external_led_off();
	 else
	    ml8rt_local_led_off();
	 parameter.led_status = OFF;   /* ... Zustand merken */
      }
      else                             /* Falls die LED ausgeschaltet ist, ... */
      {
	 if (parameter.led == EXTER)
	    ml8rt_external_led_on();   /* ... LED einschalten ... */
	 else
	    ml8rt_local_led_on();
	 parameter.led_status = ON;    /* ... Zustand merken */
      }
	 pause = 0;                    /* Durchlaufzhler wieder auf Null setzen */
   }
   ml8rt_exit();
}

void main ()                 /* ---- PREPARE ---- */
{
   pdt.pdt_head.ucType          = 1;                     /* PDT-Typ einstellen (im Moment wird nur Typ 1 untersttzt) */
   pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD);  /* Lnge des PDT-Vorspanns (im Moment immer = 48) */
   pdt.pdt_head.usGlobalProc    = 4;                     /* Anzahl der Prozeduren */
   pdt.pdt_head.usProgNo        = PROGRAMM_NUMBER;       /* Programm-Nummer */
   pdt.pdt_head.cVersion        = VERSION;               /* Programm-Version */
   pdt.pdt_head.cRevision       = REVISION;              /* Programm-Revision */
   pdt.pdt_head.ucCPU           = _486SX;                /* CPU-Type */
   pdt.pdt_head.ucCoProc        = _NOCOPROZ;             /* Co-Prozessor-Typ */
   pdt.pdt_head.ucLanguage      = _CPP31;                /* Programmiersprache = C++ */
   pdt.pdt_head.ucProgType      = _USER_PRG;             /* Programm-Typ: Anwenderprogramm */
   pdt.pdt_head.usFlags         = 0;                     /* Flags */
   pdt.pdt_head.usFlags         = _NI_TASK + _PDT_INFO_ENABLE + _LOCAL_DATA + _LOCAL_PARAMETER + _FIXED_DATASIZE;
   pdt.pdt_head.usInt           = 0;                     /* Interrupt-Nummer fr die das Programm gedacht ist (nicht relevant, da NI-Task) */
   pdt.pdt_head.ulDataAdr       = 0;                     /* Adresse des Datenbereichs (hier wird kein Datenbereich verwendet) */
   pdt.pdt_head.ulDataSize      = 0;                     /* Gre des Datenbereichs */
   pdt.pdt_head.ulDataSizeMin   = 0;                     /* Minimale Datenbereichsgre */
   pdt.pdt_head.ulDataSizeMax   = 0;                     /* Maximale Datenbereichsgre */
					                                          /* Anfangsadresse des Parameterbereichs errechnen */
   ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
   pdt.pdt_head.ulGlobalParAdr  = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
                                                         /* Anzahl der Parameter eintragen */
   pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);
   pdt.pdt_head.ulHyperAdr      = 0;                     /* Adresse des Hypertextes (wird nicht verwendet) */
   pdt.main_proc                = main_task;             /* Adresse der Hauptprozedur */
   pdt.auto_init                = auto_init;             /* Adresse der Auto_Init Prozedur */
   pdt.start                    = start;                 /* Adresse der Start-Prozedur */
   pdt.stop                     = stop;                  /* Adresse der Stop-Prozedur */
   ml8rt_set_pdt_adr (&pdt);                             /* Adresse der PDT im Rckgabe-Record dem Betriebssystem mitteilen */
}
