/***************************************************************************
/**                 BLINK-LED ALS DI-TASK (MODULAR-4/486)
/**
/**  Dieses Programm lt entweder die On-Board- oder die Externe-LED
/**  blinken. Es soll als Beispiel zur Erstellung von Echtzeitprogrammen
/**  dienen. Weitere Hinweise entnehmen Sie bitte dem Benutzerhandbuch
/**  bzw. dem Technischen-Handbuch.
/**
/**  Installationshinweise:
/**
/**  Programmnummer  : 301h
/**  Tasknummer      : beliebig
/**  Interruptquelle : 91h (Timer-A, Slave-Controller)
/**  Datenbereich    : 0
/**  Flags           : 0982h (Task nicht aktivieren, Auto-Init aufrufen)
/**
/**
/**                       Borland C++ (Version 3.1)
/**
/**                             Version 1.E.
/**
/**                   Erstellungsdatum : 28.09.92
/**                    letzte nderung : 06.06.2000 (HK)
/**                             PDT auf neustem Stand
/**                             Status : freigegeben
/**                              Autor : M.H.
/**
/**          (c) 1992..1996 SORCUS Computer GmbH Heidelberg
/**
/***************************************************************************/


#include "ml8rtbib.h"        				/* Echtzeitbibliothek einbinden */
/* #include "ml8makro.h" */  				/* Makro-Definitionen; TASM notwendig!*/
#include "stdlib.h"
#include "dos.h"

/* ---- Allgemeine Angaben ---- */
#define PROGRAMM_NUMBER 0x301          /* Programmnummer = 301h */
#define VERSION         '1'            /* Version des Programms */
#define REVISION        'E'            /* Revision des Programms */

/* ---- Die verschiedenen Statusmglichkeiten des Programms ---- */
#define READY           0              /* Bereit */
#define RUNNING         2              /* Programm luft */
#define ERROR           3              /* Falsche LED angewhlt */
#define STOPPED         4              /* Programm wurde angehalten */

/* ---- Zustnde der LED ---- */
#define  ON             1
#define  OFF            0

/* ---- Benutzte LED ---- */
#define INTERN          1              /* Interne LED */
#define EXTER           2              /* Externe LED */

/* ---- Timer - Initialisierungen---- */
/* Timer A 16-bit-> (65535/ 2), da Periodendauer T = t(LED_ON) + t(LED_OFF) */
#define TIMER_VAL  30000UL
/* Timer A Basistakt Standard 2.5MHz (evt. 10 MHz) */
#define TIMER_FREQ 2500000UL


/* ---- Aufbau der Programm-Descriptor-Tabelle (PDT) ---- */
struct pdt_type
{
	ML8_PDT_HEAD pdt_head;       /* PDT-Header */

	void* PFAR main_proc;        /* Adresse der Haupt-Prozedur */
	void* PFAR auto_init;        /* Adresse der Auto-Init Prozedur */
	void* PFAR start_proc;       /* Adresse der Start-Prozedur */
	void* PFAR stop_proc;        /* Adresse der Stop-Prozedur */
} pdt;

/* ---- Aufbau des Parameterbereichs ---- */
struct parameter_type
{
	tdt_type     tdt;          /* Platz fr Task-Descriptor-Tabelle */
	byte         status;       /* Status des Programms (READY etc.) */
	ushort       blink_rate;   /* Blinkrate */
	byte         led_status;   /* Zustand der LED */
	byte         led;          /* Welche LED 1=intern, 2=extern */
	byte         soft_cnt;     /* Soft-Counter noetig, um mit Timer */
													/* auch niedrige Freq. zu erzeugen!  */
	byte         soft_init;    /* Soft-Counter Initialisierungswert */
} parameter;

/* Prototypen der benutzten Prozeduren */
void PFAR auto_init(void);
void PFAR main_proc(void);
void PFAR start(void);
void PFAR stop(void);

/* ---- AUTO-INITIALISIERUNG ---- */
void PFAR auto_init(void)
{
	ml8rt_entry();                      /* Register retten und vorbereiten */
	parameter.status = READY;           /* Status des Programms auf "BEREIT" */
	parameter.blink_rate = 1;           /* Blinkrate Defaultwert von 1 Hz */
	parameter.led_status = OFF;         /* Status der LED auf "AUS" */
	parameter.led = INTERN;             /* Standardmig interne LED */
	ml8rt_external_led_off();           /* Die externe LED ausschalten */
	ml8rt_local_led_off();              /* Die On-Board-LED ausschalten */
	ml8rt_exit();                       /* Register wieder restaurieren */
}


void PFAR start(void)
{
	ushort timer;

	ml8rt_entry();

	/* Falls nicht die richtige LED angegeben wurde, Fehler zurckgeben */
	if ((parameter.led != INTERN) && (parameter.led != EXTER))
	{
	 parameter.status = ERROR;
	}
	else
	{
	 /* Softzaehler ausrechnen (muss ganzzahlig sein, keine Fliesskommaop.!):  */
	 parameter.soft_init = TIMER_FREQ / (2 * parameter.blink_rate * TIMER_VAL);

	 /* falls Softzaehler auf 0 "abgerundet" wird, setze Zaehler auf 1 */
	 if (parameter.soft_init == 0) parameter.soft_init = 1;
	 parameter.soft_cnt = parameter.soft_init;

	 /* Timerwert ausrechnen: Fehler durch Softteiler (ganzzahlig!) wird durch
		 Anpassung des maximalen Timerwerts kompensiert! */
	 timer = TIMER_FREQ / (2 * parameter.blink_rate * parameter.soft_init);

	 /* Timer auf den berechneten Wert setzen und starten */
	 ml8rt_set_timer (TIMER_A, timer, INT_MODE);
	 parameter.status = RUNNING;
	 ml8rt_unmask_int (IRQ_TIMER_A);    /* Timer-Interrupt freigeben! */
	}
	ml8rt_exit();
}


void PFAR stop(void)
{
	ml8rt_entry();
	ml8rt_mask_int (IRQ_TIMER_A);       /* Timer-Interrupt sperren */
	parameter.status = STOPPED;         /* Status auf "abgebrochen" */
	ml8rt_exit();
}

/* ---- HAUPTPROZEDUR DER TASK ---- */
void PFAR main_proc(void)
{
	ml8rt_entry();
	parameter.soft_cnt--;               /* Soft-Counter inkrementieren */
	if (parameter.soft_cnt == 0)        /* Soft-Counter schon auf Null? */
	{                                   /* Wenn ja, dann LED umschalten */
	 if (parameter.led_status == ON)    /* Falls die LED eingeschaltet ist, */
	  {
		if (parameter.led == EXTER)      /* ... LED ausschalten ... */
		ml8rt_external_led_off();
		else
		ml8rt_local_led_off();
		parameter.led_status = OFF;      /* ... Zustand merken */
	  }
	  else                              /* Falls die LED ausgeschaltet ist, */
	  {
		if (parameter.led == EXTER)
			ml8rt_external_led_on();      /* ... LED einschalten ... */
		else
			ml8rt_local_led_on();
			parameter.led_status = ON;    /* ... Zustand merken */
	  }                                 /* Software-Zaehler rcksetzen */
	 parameter.soft_cnt = parameter.soft_init;
	}

	ml8rt_end_of_int(IRQ_TIMER_A);      /* EOI an Interrupt Contr. senden */
	ml8rt_exit_interrupt();
}


void main (void)                 		/* ---- PREPARE ---- */
{
	pdt.pdt_head.ucType          = 1;                    /* PDT-Typ einstellen, Standard=1 */
	pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD); /* Lnge des PDT-Vorspanns */
	pdt.pdt_head.usGlobalProc    = 4;                    /* Anzahl der Prozeduren */
	pdt.pdt_head.usProgNo        = PROGRAMM_NUMBER;      /* Programm-Nummer */
	pdt.pdt_head.cVersion        = VERSION;              /* Programm-Version */
	pdt.pdt_head.cRevision       = REVISION;             /* Programm-Revision */
	pdt.pdt_head.ucCPU           = _486DX;               /* CPU-Type */
	pdt.pdt_head.ucCoProc        = _NOCOPROZ;            /* Co-Prozessor-Typ */
	pdt.pdt_head.ucLanguage      = _CPP31;               /* Programmiersprache = C++ */
	pdt.pdt_head.ucProgType      = _USER_PRG;            /* Programm-Typ: Anwenderprogramm */
	pdt.pdt_head.usFlags         = 0;                    /* Flags */
	pdt.pdt_head.usFlags         = _DI_TASK + _PDT_INFO_ENABLE +_LOCAL_PARAMETER;
	pdt.pdt_head.usInt           = IRQ_TIMER_A;          /* Interrupt-Nummer fr Programm */
	pdt.pdt_head.ulDataAdr       = 0;                    /* Adresse des Datenbereichs (hier 0)*/
	pdt.pdt_head.ulDataSize      = 0;                    /* Gre des Datenbereichs */
	pdt.pdt_head.ulDataSizeMin   = 0;                    /* Minimale Datenbereichsgre */
	pdt.pdt_head.ulDataSizeMax   = 0;                    /* Maximale Datenbereichsgre */
	                                                     /* Anfangsadresse des Parameterbereichs errechnen */
	ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
	pdt.pdt_head.ulGlobalParAdr   = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
	                                                     /* Anzahl der Parameter eintragen */
	pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);
	pdt.pdt_head.ulHyperAdr      = 0;                    /* Adresse des Hypertextes */
	pdt.main_proc                = main_proc;            /* Adresse der Hauptprozedur */
	pdt.auto_init                = auto_init;            /* Adresse der Auto_Init Prozedur */
	pdt.start_proc               = start;                /* Adresse der Start Prozedur */
	pdt.stop_proc                = stop;                 /* Adresse der Stop Prozedur */
	ml8rt_set_pdt_adr (&pdt);                            /* Adresse der PDT im Rckgabe-Record dem Betriebssystem mitteilen */
}
