/***************************************************************************/
/**                 BLINK-LED ALS DI-TASK (MODULAR-4/486)                 **/
/**                                                                       **/
/**  Dieses Programm lt entweder die On-Board- oder die Externe-LED     **/
/**  blinken. Es soll als Beispiel zur Erstellung von Echtzeitprogrammen  **/
/**  dienen. Es ist zu groen Teil identisch mit dem Programm M8P0301.C   **/
/**  Es ist ein Durchlaufzhler in den Parametern hinzugefgt worden, der **/
/**  die Anzahl der Blinksequenzen mitzhlt. Dieser Parameter soll dann   **/
/**  von einer zweiten Task (M8P0303.C) berwacht werden. Ab einer        **/
/**  festgelegten Zahl der Umschaltungen soll die zweite Task dann diese  **/
/**  Task stoppen.                                                        **/
/**  Die Anzahl der Umschaltungen kann ber eine Taskfunktion ermittelt   **/
/**  werden.                                                              **/
/**  Als besonderes Feature ist dieses Programm unter den Timern A bis C  **/
/**  installierbar.                                                       **/
/**  Weitere Hinweise entnehmen Sie bitte dem Benutzerhandbuch.           **/
/**                                                                       **/
/**  Installationshinweise:                                               **/
/**                                                                       **/
/**  Programmnummer  : 302h                                               **/
/**  Tasknummer      : beliebig                                           **/
/**  Interruptquelle : 91h (Timer-A), 92h (Timer-B), 93h (Timer-C)        **/
/**  Datenbereich    : 0                                                  **/
/**  Flags           : 0x098A (Task nicht aktivieren, Auto-Init aufrufen, **/
/**                    Interrupt-Nummer wird beim Installieren festgelegt)**/
/**                                                                       **/
/**                       Borland C++ (Version 3.1)                       **/
/**                                                                       **/
/**                             Version 1.H.                              **/
/**                                                                       **/
/**                   Erstellungsdatum : 28.09.92                         **/
/**                    letzte nderung : 19.05.00 (HK)                    **/
/**                    ml8rt_end_of_int statt outportb...                 **/
/**                    Pointerzugriffe ber Bibliothek                    **/
/**                       PDT auf neuestem Stand                          **/
/**                             Status : freigegeben                      **/
/**                              Autor : M.H.                             **/
/**                                                                       **/
/**      (c) 1992 - 2000 SORCUS Computer GmbH Heidelberg                  **/
/**                                                                       **/
/***************************************************************************/

#pragma option -r-           /* Bei Verwendung einer Task-Funktion drfen keine (!) Register-Variablen verwendet werden! */


#include "ml8rtbib.h"        /* Echtzeitbibliothek einbinden */
/* #include "ml8makro.h" */  /* Makro-Datei bentigt Turbo-Assembler! */
#include "stdlib.h"
#include "dos.h"

			     /* ---- Allgemeine Angaben ---- */
#define PROGRAMM_NUMBER 0x302             /* Programmnummer = 302h */
#define VERSION         '1'               /* Version des Programms */
#define REVISION        'H'               /* Revision des Programms */

			     /* ---- Die verschiedenen Statusmglichkeiten des Programms ---- */
#define READY           0                 /* Bereit */
#define RUNNING         2                 /* Programm luft */
#define LED_ERROR       3                 /* Falsche LED angewhlt */
#define STOPPED         4                 /* Programm wurde angehalten */
#define SOURCE_ERROR    5                 /* Programm unter einem falschen Interrupt installiert */

			     /* ---- Zustnde der LED ---- */
#define  ON             1
#define  OFF            0

			     /* ---- Timer starten oder stoppen ---- */
#define  START          1
#define  STOP           0

			     /* ---- Benutzte LED ---- */
#define INTERN          1                 /* Interne LED */
#define EXTER           2                 /* Externe LED */


			     /* ---- Aufbau der Programm-Descriptor-Tabelle (PDT) ---- */
struct pdt_type
{
   ML8_PDT_HEAD pdt_head;                    /* PDT-Header */
   void* PFAR   main_proc;                   /* Adresse der Haupt-Prozedur (Interrupt-Task) */
   void* PFAR   auto_init;                   /* Adresse der Auto-Init Prozedur */
   void* PFAR   start_proc;                  /* Adresse der Start-Prozedur */
   void* PFAR   stop_proc;                   /* Adresse der Stop-Prozedur */
   void* PFAR   get_switches;                /* Adresse der Taskfunktion fr die Anzahl der Umschaltungen */
} pdt;

			     /* ---- Aufbau des Parameterbereichs ---- */
struct parameter_type
{
   tdt_type     tdt;                         /* Platz fr die Task-Descriptor-Tabelle */
   byte         status;                      /* Status des Programms (READY etc.) */
   ushort       blink_rate;                  /* Blinkrate */
   byte         led_status;                  /* Zustand der LED */
   byte         led;                         /* Welche LED 1=intern, 2=extern */
   byte         soft_cnt;                    /* Soft-Counter. Erweiterung des Hardware-Counters */
   byte         soft_init;                   /* Soft-Counter Initialisierungswert */
   ushort       switch_cnt;                  /* Anzahl der Blinksequenzen */
} parameter;


void PFAR auto_init(void);                     /* Prototypen der benutzten Prozeduren */
void PFAR main_task(void);
void PFAR start(void);
void PFAR stop(void);
void PFAR get_switches(ushort task, ushort insize,
		ushort *indata, ushort outsize, ushort *outdata);

void PFAR control_timer(ushort mode);


void PFAR auto_init(void)             /* ---- AUTO-INITIALISIERUNG ---- */
{
   ml8rt_entry();                         /* Register retten und vorbereiten */
   parameter.status = READY;              /* Status des Programms auf "BEREIT" */
   parameter.blink_rate = 1;              /* Blinkrate auf einen Defaultwert von 1 Hz einstellen */
   parameter.led_status = OFF;            /* Status der LED auf "AUS" */
   parameter.led = INTERN;                /* Standartmig die interne LED benutzen */
   parameter.switch_cnt = 0;              /* Anzahl der LED-Blink-Sequenzen auf 0 */
   ml8rt_external_led_off();              /* Die externe LED ausschalten */
   ml8rt_local_led_off();                 /* Die On-Board-LED ausschalten */
   ml8rt_exit();                          /* Register wieder restaurieren */
}


ushort PFAR programm_control (ushort mode, ushort timer)
{
int                     timer_in_use;
ushort            res;

   timer_in_use = parameter.tdt.inter - ((int)IRQ_TIMER_A);
   switch (timer_in_use) {
    case TIMER_A : ;                      /* Nur wenn einer der angegebenen Timer als Interrupt-Quelle angegeben wurde, ... */
    case TIMER_B : ;                      /* ... den Timer starten oder stoppen. */
    case TIMER_C : {
      if (mode==START)
      {                                   /* Falls das Programm gestartet werden soll, ... */
	 ml8rt_set_timer (timer_in_use, timer, INT_MODE);
	 ml8rt_unmask_int (parameter.tdt.inter);
	 res = RUNNING;
      }
      else
      {                                   /* Falls das Programm gestopped werden soll, ... */
	 ml8rt_mask_int (parameter.tdt.inter);
	 res = STOPPED;
      }
    } break;
    default : res = SOURCE_ERROR;         /* Falls das Programm unter einen anderen Interrupt installiert wurde, ... */
   }
   return (res);
}


void PFAR start(void)
{
long                    n;
ushort            timer;
ldiv_t                  div_erg;

   ml8rt_entry();
   if ((parameter.led != INTERN) && (parameter.led != EXTER))
   {                                      /* Falls nicht die richtige LED angegeben wurde, Fehler zurckgeben */
      parameter.status = LED_ERROR;
   }
   else
   {                                      /* Zuerst die angegebene Blink-Rate von Hz in Timer und Soft-Teiler umrechnen */
      div_erg = ldiv (2500000L, parameter.blink_rate);
      n = div_erg.quot;
      div_erg = ldiv (n, 60000L);
      parameter.soft_init = (char) div_erg.quot;
      parameter.soft_cnt = parameter.soft_init;
      timer = (int) (2*n/(parameter.soft_init+1));
      parameter.switch_cnt = 0;           /* Blink-Sequenzzhler zurcksetzen */
					  /* Programm starten */
      parameter.status = programm_control (START, timer);
   }
   ml8rt_exit();
}


void PFAR stop(void)
{
   ml8rt_entry();
					  /* Programmlauf abbrechen */
   parameter.status = programm_control (STOP, 0);
   parameter.switch_cnt = 0;
   ml8rt_exit();
}

			/* ---- Diese Funktion liefert die Anzahl der schon gemachten Umschaltungen ---- */
void PFAR get_switches(ushort task, ushort insize,
		ushort *indata, ushort outsize, ushort *outdata)
{
ushort            err;

   ml8rt_entry_function();
   if (outsize == sizeof(*outdata))      /* Der erwartete Datentyp muss bereinstimmen! (ushort = 2 Byte) */
   {
      *outdata = parameter.switch_cnt;   /* Die Anzahl der Umschaltungen bergeben */
      err = 0;
   }
   else
   {
      outsize = 0;                       /* Falls der Datentyp nicht stimmt -> Fehlermeldung */
      err = 0x01E0;                      /* Das Low-Byte der Fehlermeldung mu immer $E0 sein, damit die Fehlermeldung */
   }                                    /* auf dem PC korrekt ankommt! (E0 = Fehler in Funktion) */
   ml8rt_exit_function(0, outsize, err);
}


void PFAR main_task(void)   /* ---- HAUPTPROZEDUR DER TASK ---- */
{
   ml8rt_entry();

   if (parameter.soft_cnt-- == 0)           /* Soft-Counter schon auf Null? */
   {                                        /* Wenn ja, dann LED umschalten */
      if (parameter.led_status == ON)       /* Falls die LED eingeschaltet ist, ... */
      {
	 ++parameter.switch_cnt;            /* Blink-Sequenz-Zhler um eins erhhen */

	 if (parameter.led == EXTER)        /* ... LED ausschalten ... */
	    ml8rt_external_led_off();
	 else
	    ml8rt_local_led_off();
	 parameter.led_status = OFF;        /* ... Zustand merken */
      }
      else                                  /* Falls die LED ausgeschaltet ist, ... */
      {
	 if (parameter.led == EXTER)
	    ml8rt_external_led_on();        /* ... LED einschalten ... */
	 else
	    ml8rt_local_led_on();
	 parameter.led_status = ON;         /* ... Zustand merken */
      }                                     /* Software-Teiler wieder auf den Anfangswert setzen */
      parameter.soft_cnt = parameter.soft_init;
   }

   ml8rt_end_of_int(parameter.tdt.inter);           /* EOI an den Interrupte-Controller senden */
   ml8rt_exit_interrupt();
}

void main (void)                 /* ---- PREPARE ---- */
{
   pdt.pdt_head.ucType          = 1;                    /* PDT-Typ einstellen (im Moment wird nur Typ 1 untersttzt) */
   pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD); /* Lnge des PDT-Vorspanns (im Moment immer = 48) */
   pdt.pdt_head.usGlobalProc    = 5;                    /* Anzahl der Prozeduren (in diesem Beispiel "Auto_Init", "Main_Task", "Start", "Stop" ) */
   pdt.pdt_head.usProgNo        = PROGRAMM_NUMBER;      /* Programm-Nummer */
   pdt.pdt_head.cVersion        = VERSION;              /* Programm-Version */
   pdt.pdt_head.cRevision       = REVISION;             /* Programm-Revision */
   pdt.pdt_head.ucCPU           = _486SX;               /* CPU-Type */
   pdt.pdt_head.ucCoProc        = _NOCOPROZ;            /* Co-Prozessor-Typ */
   pdt.pdt_head.ucLanguage      = _CPP31;               /* Programmiersprache = C++ */
   pdt.pdt_head.ucProgType      = _USER_PRG;            /* Programm-Typ: Anwenderprogramm */
   pdt.pdt_head.usFlags         = _DI_TASK + _LOCAL_DATA + _LOCAL_PARAMETER + _FIXED_DATASIZE;
   pdt.pdt_head.usInt           = 0;                    /* Interrupt-Nummer wird bei der Installierung festgelegt. */
   pdt.pdt_head.ulDataAdr       = 0;                    /* Adresse des Datenbereichs (hier wird kein Datenbereich verwendet) */
   pdt.pdt_head.ulDataSize      = 0;                    /* Gre des Datenbereichs */
   pdt.pdt_head.ulDataSizeMin   = 0;                    /* Minimale Datenbereichsgre */
   pdt.pdt_head.ulDataSizeMax   = 0;                    /* Maximale Datenbereichsgre */
					  /* Anfangsadresse des Parameterbereichs errechnen */
   ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
   pdt.pdt_head.ulGlobalParAdr  = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
					  /* Anzahl der Parameter eintragen */
   pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);
   pdt.pdt_head.ulHyperAdr      = 0;                    /* Adresse des Hypertextes (wird nicht verwendet) */
   pdt.main_proc                = main_task;            /* Adresse der Hauptprozedur */
   pdt.auto_init                = auto_init;            /* Adresse der Auto_Init Prozedur */
   pdt.start_proc               = start;                /* Adresse der Start Prozedur */
   pdt.stop_proc                = stop;                 /* Adresse der Stop Prozedur */
   pdt.get_switches             = get_switches;         /* Adresse der Taskfunktion */
   ml8rt_set_pdt_adr (&pdt);                            /* Adresse der PDT im Rckgabe-Record dem Betriebssystem mitteilen */
}
