/***************************************************************************/
/**   berwachungs NI-Task (MODULAR-4/486) fr das Programm INTTASK1.PAS  **/
/**                            (Blink-LED).                               **/
/**                                                                       **/
/**  Dieses Programm berwacht das Blink-LED Programm. Ab einer Anzahl    **/
/**  von Durchlufen wird das Blinken durch Aufruf der Stop-Prozedur      **/
/**  (Prozedur #3 von M8P0302.C) gestoppt.                                **/
/**  Es soll als Beispiel zur Intertaskkomunikation dienen.               **/
/**  Weitere Hinweise entnehmen Sie bitte dem Benutzerhandbuch bzw. dem   **/
/**  Technischen-Handbuch.                                                **/
/**                                                                       **/
/**  Installationshinweise:                                               **/
/**                                                                       **/
/**  Programmnummer  : 303h                                               **/
/**  Tasknummer      : beliebig                                           **/
/**  Interruptquelle : 1 (wird nicht verwendet, da NI-Task)               **/
/**  Datenbereich    : 0                                                  **/
/**  Flags           : 0x0980 (nicht aktiviert, Auto-Init aufrufen)       **/
/**                                                                       **/
/**                                                                       **/
/**                    Fr Borland C++ (Version 3.1)                      **/
/**                                                                       **/
/**                             Version 1.G.                              **/
/**                                                                       **/
/**                   Erstellungsdatum : 26.01.93                         **/
/**                    letzte nderung : 06.06.2000 (HK)                  **/
/**                             PDT auf neustem Stand                     **/
/**                             Status : freigegeben                      **/
/**                              Autor : M.Heisig                         **/
/**                                                                       **/
/**        (c) 1992 - 1996 SORCUS Computer GmbH Heidelberg                **/
/**                                                                       **/
/***************************************************************************/

#pragma option -r-           /* Keine Registervariablen!! */

#include "ml8rtbib.h"        /* Header fr die Echtzeitbibliothek einbinden */
/* #include "ml8makro.h" */  /* Makro-Datei bentigt Turbo-Assembler! */
#include "dos.h"

			     /* ---- Allgemeine Angaben ---- */
#define PROGRAMM_NUMBER 0x303             /* Programmnummer = 303h */
#define VERSION         '1'               /* Version des Programms */
#define REVISION        'G'               /* Revision des Programms */

			     /* ---- Die verschiedenen Statusmglichkeiten des Programms ---- */
#define READY           0                 /* Bereit */
#define RUNNING         2                 /* Programm luft */
#define ERROR           3                 /* Fehler (z.B. Blink-Task nicht installiert) */
#define STOPPED         4                 /* Programmlauf wurde abgebrochen */

			     /* ---- Informationen ber die Task-1 (blink-LED) ---- */
#define SWITCH_FUNC     4                 /* Taskfunktion liefert die Anzahl der Blinksequenzen */
#define STOP_PROC_NR    3                 /* Stop-Prozedur-Nummer der Blink-Task */

			     /* ---- Aufbau der Programm-Descriptor-Tabelle (PDT) ---- */
struct pdt_type
{
   ML8_PDT_HEAD pdt_head;                    /* PDT-Header */

   void* PFAR   main_proc;                   /* Adresse der Haupt-Prozedur (Task) */
   void* PFAR   auto_init;                   /* Adresse der Auto-Init Prozedur */
   void* PFAR   start;	                     /* Adresse der Start-Prozedur */
   void* PFAR   stop;                        /* Adresse der Stop-Prozedur */
} pdt;

			     /* ---- Aufbau des Parameterbereichs ---- */
struct parameter_type
{
   tdt_type     tdt;                         /* Platz fr die Task-Descriptor-Tabelle */
   byte         status;                      /* Status des Programms (READY etc.) */
   ushort       watch_task;                  /* Task-Nummer der zu berwachenden Task (Blink-LED) */
   ushort       switches;                    /* Anzahl der Umschaltungen, nachdem gestoppt werden soll. */
} parameter;


void PFAR auto_init(void);                     /* Prototypen der benutzten Prozeduren */
void PFAR start(void);
void PFAR stop(void);
void PFAR main_task(void);

void PFAR auto_init(void)         /* ---- AUTO-INITIALISIERUNG ---- */
{
   ml8rt_entry();                         /* Register retten und vorbereiten */
   parameter.status = READY;              /* Status des Programms auf "BEREIT" */
   parameter.watch_task = 0;
   parameter.switches = 10;
   ml8rt_exit();                          /* Register wieder restaurieren */
}


void PFAR start(void)
{
   ml8rt_entry();
   parameter.status = RUNNING;            /* Status auf Programm luft */
   ml8rt_wakeup_task (parameter.tdt.task);/* Task aktivieren, bzw. starten */
   ml8rt_exit();
}


void PFAR stop(void)
{
   ml8rt_entry();
   ml8rt_sleep_task (parameter.tdt.task); /* Task deaktivieren (stoppen) */
   parameter.status = STOPPED;
   ml8rt_exit();
}


void PFAR main_task(void)             /* ---- HAUPTPROZEDUR DER NI-TASK ---- */
{
ushort    switch_cnt;
ushort    outsize;
ushort    indata;
ushort    insize;
ushort    error;

   ml8rt_entry();
					  /* Diese Funktion liefert die Anzahl der schon gemachten Umschaltungen (siehe M8P0302). */
					  /* Es werden nur Daten von der Funktion zurck erwartet! */
   ml8rt_call_func(parameter.watch_task, SWITCH_FUNC,
	  &insize, &indata, sizeof(switch_cnt), &outsize, &switch_cnt, &error);

   if ((error != 0) || (outsize != sizeof(switch_cnt)))
   {                                      /* Falls ein Fehler auftritt, ... */
      ml8rt_sleep_task (parameter.tdt.task); /* ... Task deaktivieren (stoppen) */
      parameter.status = ERROR;
	}
	else
   {                                      /* Jetzt die Anzahl der Umschaltungen berprfen. */
      if (switch_cnt >= parameter.switches)
      {                                   /* Falls die Anzahl der gewnschten Blink-Sequenzen erreicht ist, ... */
	 ml8rt_call_proc (parameter.watch_task, STOP_PROC_NR);
      }                                   /* Blink-Task anhalten */
   }                                      /* Hinweis: Die berwachungstask ist weiter aktiv! */

   ml8rt_exit();
}


void main (void)                 /* ---- PREPARE ---- */
{
   pdt.pdt_head.ucType          = 1;                    /* PDT-Typ einstellen (im Moment wird nur Typ 1 untersttzt) */
   pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD); /* Lnge des PDT-Vorspanns (im Moment immer = 48) */
   pdt.pdt_head.usGlobalProc    = 4;                    /* Anzahl der Prozeduren */
   pdt.pdt_head.usProgNo        = PROGRAMM_NUMBER;      /* Programm-Nummer */
   pdt.pdt_head.cVersion        = VERSION;              /* Programm-Version */
   pdt.pdt_head.cRevision       = REVISION;             /* Programm-Revision */
   pdt.pdt_head.ucCPU           = _486SX;               /* CPU-Type */
   pdt.pdt_head.ucCoProc        = _NOCOPROZ;            /* Co-Prozessor-Typ */
   pdt.pdt_head.ucLanguage      = _CPP31;               /* Programmiersprache = C++ */
   pdt.pdt_head.ucProgType      = _USER_PRG;            /* Programm-Typ: Anwenderprogramm */
   pdt.pdt_head.usFlags         = 0;                    /* Flags */
   pdt.pdt_head.usFlags         = _NI_TASK + _PDT_INFO_ENABLE + _LOCAL_DATA + _LOCAL_PARAMETER + _FIXED_DATASIZE;
   pdt.pdt_head.usInt           = 0;                    /* Interrupt-Nummer fr die das Programm gedacht ist (nicht relevant, da NI-Task) */
   pdt.pdt_head.ulDataAdr       = 0;                    /* Adresse des Datenbereichs (hier wird kein Datenbereich verwendet) */
   pdt.pdt_head.ulDataSize      = 0;                    /* Gre des Datenbereichs */
   pdt.pdt_head.ulDataSizeMin   = 0;                    /* Minimale Datenbereichsgre */
   pdt.pdt_head.ulDataSizeMax   = 0;                    /* Maximale Datenbereichsgre */
					                                         /* Anfangsadresse des Parameterbereichs errechnen */
   ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
   pdt.pdt_head.ulGlobalParAdr  = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
					                                         /* Anzahl der Parameter eintragen */
   pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);
   pdt.pdt_head.ulHyperAdr      = 0;                    /* Adresse des Hypertextes (wird nicht verwendet) */
   pdt.main_proc                = main_task;            /* Adresse der Hauptprozedur */
   pdt.auto_init                = auto_init;            /* Adresse der Auto_Init Prozedur */
   pdt.start                    = start;                /* Adresse der Start-Prozedur */
   pdt.stop                     = stop;                 /* Adresse der Stop-Prozedur */
   ml8rt_set_pdt_adr (&pdt);                            /* Adresse der PDT im Rckgabe-Record dem Betriebssystem mitteilen */
}
