/***************************************************************************/
/**      Analogdaten-Erfassungsprogramm mit M-AD16-3 (MODULAR-4/486)      **/
/**                                                                       **/
/**  Dieses Programm erfat mit einer einstellbaren Abtastrate bis zu     **/
/**  16 analoge Eingnge mit dem Modul M-AD16-3. Die Parameter mssen     **/
/**  vor dem Start des Programms entsprechend eingestellt werden.         **/
/**  Die Gre des Datenbereichs kann per Makrobefehl eingestellt werden. **/
/**                                                                       **/
/**  Installationshinweise:                                               **/
/**                                                                       **/
/**  Programmnummer  : 304h						  **/
/**  Tasknummer      : beliebig                                           **/
/**  Interruptquelle : 91h (Timer-A, Slave-Controller)                    **/
/**  Datenbereich    : per Makrobefehl einstellbar                        **/
/**  Flags           : 1982h (Task aktivieren, Auto-Init aufrufen)        **/
/**                                                                       **/
/**                                                                       **/
/**                       Borland C++ (Version 3.1)                       **/
/**                                                                       **/
/**                             Version 1.D.                              **/
/**                                                                       **/
/**                   Erstellungsdatum : 21.04.93                         **/
/**                    letzte nderung : 19.05.00 (HK)                    **/
/**                    ml8rt_end_of_int statt outportb...                 **/
/**                    Pointerzugriffe ber Bibliothek                    **/
/**                        PDT auf neustem Stand                          **/
/**                             Status : freigegeben                      **/
/**                              Autor : M.H.                             **/
/**                                                                       **/
/**          (c) 1993 SORCUS Computer GmbH Heidelberg                     **/
/**                                                                       **/
/***************************************************************************/


#include "ml8rtbib.h"        /* Echtzeitbibliothek einbinden */
//#include "ml8makro.h"      /* Makro-Definitionen einiger ML8RTBIB Routinen einbinden */
#include "dos.h"             /* outport, outportb etc. als Makros */

const _8087 = 3;             /* ---- 80387 Code erzwingen (speziell die SIN, ... Fktnen) ---- */

			     /* ---- Allgemeine Angaben ---- */
#define PROGRAMM_NUMBER 0x304             /* Programmnummer = 304h */
#define VERSION         '1'               /* Version des Programms */
#define REVISION        'D'               /* Revision des Programms */

			     /* ---- Die verschiedenen Statusmglichkeiten des Programms ---- */
#define READY           0                 /* Bereit */
#define RUNNING         2                 /* Messung luft */
#define FINISHED        3                 /* Messung beendet */
#define STOPPED         4                 /* Programm wurde abgebrochen */
#define ERROR           0x80              /* Fehler aufgetreten */
#define DATA_ERROR      0x81              /* Fehler: Datenbereich zu klein */
#define MODUL_ERROR     0x82              /* Fehler: Falscher Modulsteckplatz angewhlt */
#define CHANNEL_ERROR   0x83              /* Fehler: Falsche Kanalzahl */


			     /* ---- Aufbau der Programm-Descriptor-Tabelle (PDT) ---- */
struct pdt_type
{
   ML8_PDT_HEAD pdt_head;                    /* PDT-Header */
   void* PFAR   main_proc;                   /* Adresse der Haupt-Prozedur (Interrupt-Task) */
   void* PFAR   auto_init;                   /* Adresse der Auto-Init Prozedur */
   void* PFAR   start_proc;                  /* Adresse der Start-Prozedur */
   void* PFAR   stop_proc;                   /* Adresse der Stop-Prozedur */
} pdt;


			     /* ---- Aufbau des Parameterbereichs ---- */
struct parameter_type
{
   tdt_type     tdt;                         /* Platz fr die Task-Descriptor-Tabelle */
   byte         status;                      /* Status des Programms (READY etc.) */
   ushort       abtastrate;                  /* Abtastrate * 0.4 us = Zeit zwischen 2 Abtastwerten */
   byte         modul;                       /* Modulsteckplatz des M-AD16-3 */
   byte         anzkan;                      /* Anzahl der zu messenden Kanle */
   ushort       anzmess;                     /* Anzahl der Messungen */
} parameter;


ushort          cntmess;          /* Zhler fr Messungen */
ushort          mba;              /* Modulbasisadresse */
ushort          io_kanal;         /* I/O-Adresse fr die Kanalwahl */
ushort          io_status;        /* I/O-Adresse fr Status des AD-Wandlers */
ushort          io_msb_data;      /* I/O-Adresse zum Lesen der oberen 8-Bit */
ushort          io_lsb_data;      /* I/O-Adresse zum Lesen der unteren 4 Bit */


void PFAR auto_init(void);                     /* Prototypen der benutzten Prozeduren */
void PFAR main_task(void);
void PFAR start(void);
void PFAR stop(void);

void PFAR auto_init(void)             /* ---- AUTO-INITIALISIERUNG ---- */
{
   ml8rt_entry();                         /* Register retten und vorbereiten */

   ml8rt_local_led_off();                 /* On-Board LED ausschalten */
   parameter.status = READY;              /* Status des Programms auf "BEREIT" */
   parameter.abtastrate = 2500;           /* 2500 * 0,4 us = 1 ms oder 1 kHz */
   parameter.modul = 1;                   /* Modulsteckplatz = 1 */
   parameter.anzkan = 1;                  /* Ein Kanal */
   parameter.anzmess = 1000;              /* Tausend Messpunkte */

   ml8rt_exit();                          /* Register wieder restaurieren */
}


void PFAR start(void)
{
ulong     datalength;

   ml8rt_entry();

   /* ---------------- Parameterprfungen ----------------- */
   datalength = parameter.tdt.datalast - parameter.tdt.datafirst;
   if (datalength < (parameter.anzkan * 2 * parameter.anzmess))
      parameter.status = DATA_ERROR;      /* Datenbereich zu klein! */

   if ((parameter.modul<1) || (parameter.modul>4))
      parameter.status = MODUL_ERROR;     /* Falscher Modulsteckplatz angewhlt */

   if ((parameter.anzkan<1) || (parameter.anzkan>16))
      parameter.status = CHANNEL_ERROR;   /* Unzulssige Kanalzahl */

   if ((parameter.status & ERROR) == 0)
   {
      ml8rt_local_led_on();               /* Eingeschaltete LED zeigt an, da die Messung luft. */
      ml8rt_reset_w_pointer(parameter.tdt.task);  /* Zeiger auf den Datenbereichsanfang */
      mba = 0x100 * parameter.modul + 0x300;  /* Modulbasisadresse berechnen */
      io_status = mba + 0x09;             /* I/O-Adressen fr die Modulzugriffe ermitteln */
      io_lsb_data = mba;                  /* (siehe Modul-Beschreibung) */
      io_msb_data = mba + 0x0a;
      io_kanal = mba + 0x01;
      cntmess = 0;
      outportb(io_kanal, 0);              /* Den ersten zu messenden Kanal einstellen */
      ml8rt_set_timer(TIMER_A, parameter.abtastrate, INT_MODE);
      parameter.status = RUNNING;         /* Timer-A initialisieren */
      ml8rt_unmask_int(IRQ_TIMER_A);      /* Durch das Demaskieren des Timers wird die Messung gestartet. */
   }

   ml8rt_exit();
}


void PFAR stop(void)
{
   ml8rt_entry();
   ml8rt_local_led_off();                 /* LED ausschalten. (Messung beendet) */
   ml8rt_mask_int (IRQ_TIMER_A);          /* Timer-Interrupt sperren */
   parameter.status = STOPPED;            /* Status auf "abgebrochen" */
   ml8rt_exit();
}


void PFAR main_task(void)          /* ---- HAUPTPROZEDUR DER TASK ---- */
{
byte           kanal;
byte           b;
ushort         w;

   ml8rt_entry();

   inportb(io_lsb_data);                  /* AD-Wandlung des ersten Kanals starten */
   kanal = 0;                             /* (Kanal-0 !!) */
   do {
      outportb(io_kanal, ++kanal);        /* neuer Kanal kann schon mal angewhlt werden */
      while ((inportb(io_status) & 0x01) == 1); /* Solange warten, bis Status OK ist. */

      w = inport(io_msb_data) & 0xFF00;   /* High-Byte (MSB) der Wandlung lesen */
      w = ((w | inportb(io_lsb_data)) >> 4);  /* Low-Byte lesen, und das Ergebnis rechtsbndig schieben. */
      ml8rt_write_data_word(parameter.tdt.task,w); /* Speichern */
					  /* Anschlieend wird der Pointer um zwei Byte erhht */
   } while (kanal<parameter.anzkan);      /* Solange messen, bis alle Kanle gesampled wurden */

   if (++cntmess > parameter.anzmess)     /* Wenn alle Messungen gemacht wurden, Programmlauf stoppen */
   {
      ml8rt_local_led_off();              /* LED ausschalten, Messung beendet */
      ml8rt_mask_int(IRQ_TIMER_A);
      parameter.status = FINISHED;
   }

   outportb(io_kanal, 0);                 /* Kanal-0 fr die nchste Messung schon mal anwhlen */

   ml8rt_end_of_int(IRQ_TIMER_A);         /* EOI an den Interrupt-Controller senden */
   ml8rt_exit_interrupt();
}



void main (void)                 /* ---- PREPARE ---- */
{
   pdt.pdt_head.ucType          = 1;                      /* PDT-Typ einstellen (im Moment wird nur Typ 1 untersttzt) */
   pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD);   /* Lnge des PDT-Vorspanns (im Moment immer = 48) */
   pdt.pdt_head.usGlobalProc    = 4;                      /* Anzahl der Prozeduren (in diesem Beispiel "Auto_Init", "Main_Task", "Start", "Stop" ) */
   pdt.pdt_head.usProgNo        = PROGRAMM_NUMBER;        /* Programm-Nummer */
   pdt.pdt_head.cVersion        = VERSION;                /* Programm-Version */
   pdt.pdt_head.cRevision       = REVISION;               /* Programm-Revision */
   pdt.pdt_head.ucCPU           = _486DX;                 /* CPU-Type */
   pdt.pdt_head.ucCoProc        = _NOCOPROZ;              /* Co-Prozessor-Typ */
   pdt.pdt_head.ucLanguage      = _CPP31;                 /* Programmiersprache = C++ */
   pdt.pdt_head.ucProgType      = _USER_PRG;              /* Programm-Typ: Anwenderprogramm */
   pdt.pdt_head.usFlags         = _DI_TASK + _PDT_INFO_ENABLE + _LOCAL_PARAMETER;
   pdt.pdt_head.usInt           = IRQ_TIMER_A;            /* Interrupt-Nummer fr die das Programm gedacht ist */
   pdt.pdt_head.ulDataAdr       = 0;                      /* Adresse des Datenbereichs (hier wird kein Datenbereich verwendet) */
   pdt.pdt_head.ulDataSize      = 100000;                 /* Datenbereichsgren */
   pdt.pdt_head.ulDataSizeMin   = 32;                     /* Minimale Datenbereichsgre */
   pdt.pdt_head.ulDataSizeMax   = 800000;                 /* Maximale Datenbereichsgre */
					  /* Anfangsadresse des Parameterbereichs errechnen */
   ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
   pdt.pdt_head.ulGlobalParAdr  = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
					  /* Anzahl der Parameter eintragen */
   pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);
   pdt.pdt_head.ulHyperAdr      = 0;                      /* Adresse des Hypertextes (wird nicht verwendet) */
   pdt.main_proc                = main_task;              /* Adresse der Hauptprozedur */
   pdt.auto_init                = auto_init;              /* Adresse der Auto_Init Prozedur */
   pdt.start_proc               = start;                  /* Adresse der Start Prozedur */
   pdt.stop_proc                = stop;                   /* Adresse der Stop Prozedur */
   ml8rt_set_pdt_adr (&pdt);                              /* Adresse der PDT im Rckgabe-Record dem Betriebssystem mitteilen */
}
