/*************************************************************************
*      Analogdaten-Erfassungsprogramm mit M-SH12-8 (MODULAR-4/486)
*
*  Dieses Programm erfasst mit einer einstellbaren Abtastrate bis zu
*  8 analoge Eingaenge mit dem Modul M-SH12-8. Die Parameter des
*  Programms muessen vor dem Start des Programms entsprechend
*  eingestellt werden.
*  Die Groesse des Datenbereichs kann per Makrobefehl eingestellt werden.
*
*  Installationshinweise:
*
*  Programmnummer  : 305h
*  Tasknummer      : beliebig
*  Interruptquelle : 91h (Timer-A, Slave-Controller)
*  Datenbereich    : per Makrobefehl einstellbar
*  Flags           : 1982h (Task aktivieren, Auto-Init aufrufen)
*
*
*                       Borland C++ (Version 3.1)
*                                                                       
*                             Version 1.D.
*
*                   Erstellungsdatum : 19.07.93
*                    letzte nderung : 06.06.2000(HK)
*                             Status : freigegeben
*                              Autor : G. Rudolf
*                                      MIH
*
*          (c) 1993 - 1996 SORCUS Computer GmbH Heidelberg
*
**************************************************************************/

#pragma option -r-                     /* Keine Registervariablen!! */

#include "dos.h"                       /* outport, outportb etc. als Makros*/
#include "ml8rtbib.h"                  /* Echtzeitbibliothek einbinden */
//#include "ml8makro.h"                /* Makro-Definitionen einiger */
				       /*  ML8RTBIB Routinen einbinden */

			     /* ---- Allgemeine Angaben ---- */
#define PROGRAMM_NUMBER 0x305          /* Programmnummer = 305h */
#define VERSION         '1'            /* Version des Programms */
#define REVISION        'D'            /* Revision des Programms */

			     /* ---- Mgliche Werte des Parameters Status ---- */
#define READY           0              /* Bereit */
#define RUNNING         2              /* Messung laeuft */
#define FINISHED        3              /* Messung beendet */
#define STOPPED         4              /* Programm wurde abgebrochen */
#define ERROR           0x80           /* Fehler aufgetreten */
#define DATA_ERROR      0x81           /* Datenbereich zu klein */
#define MODUL_ERROR     0x82           /* Falscher Modulsteckplatz*/
#define CHANNEL_ERROR   0x83           /* Falsche Kanalzahl */

#define MSH12_MODE      3              /* Abtasttrigger von Timer A */
#define MSH12_MODE_1    2              /* 1. Byte fuer Modeeinstellung */
#define MSH12_MODE_2    3              /* 2. Byte fuer Modeeinstellung */
#define MSH12_INTERRUPT 0              /* kein Interrupt ausgeloest */

			     /* ---- Aufbau der Programm-Descriptor-Tabelle (PDT) ---- */
struct pdt_type
{
   ML8_PDT_HEAD pdt_head;     /* PDT-Header */
   void* PFAR main_proc;      /* Adresse der Haupt-Prozedur */
   void* PFAR auto_init;      /* Adresse der Auto-Init Prozedur */
   void* PFAR start_proc;     /* Adresse der Start-Prozedur */
   void* PFAR stop_proc;      /* Adresse der Stop-Prozedur */
} pdt;

			     /* ---- Aufbau des Parameterbereichs ---- */
struct parameter_type
{
   tdt_type     tdt;          /* Task-Descriptor-Tabelle */
   byte         status;       /* Status des Programms (READY etc.) */
   ushort       abtastrate;   /* Abtastrate * 0.4 us = Zeit zwischen 2 Abtastwerten */
   byte         modul;        /* Modulsteckplatz */
   byte         anzkan;       /* Anzahl der zu messenden Kanaele */
   ushort       anzmess;      /* Anzahl der Messungen */
} parameter;


ushort          cntmess;       /* Zaehler fuer Messungen */
ushort          mba;           /* Modulbasisadresse */
ushort          io_kanal;      /* I/O-Adr. fuer die Kanalwahl */
ushort          io_ad_status;  /* I/O-Adr. fuer Status des Wandlers */
ushort          io_data;      	/* I/O-Adr. zum Lesen der 16 Bit */
ushort          io_start_ad;   /* I/O-Adr. zum Starten der Wandlung */
byte            kanal;		   /* Aktuelle Kanalnummer */
byte            b;		         /* Byte */
ushort          w;		         /* Word */


void PFAR auto_init(void);                  /* Prototypen  */
void PFAR main_proc(void);
void PFAR start(void);
void PFAR stop(void);

void PFAR auto_init(void)             /* ---- AUTO-INITIALISIERUNG ---- */
{
   ml8rt_entry();                      /* Register retten und vorbereiten */

   ml8rt_local_led_off();              /* On-Board LED ausschalten */
   parameter.status = READY;           /* Status des Programms auf "BEREIT" */
   parameter.abtastrate = 2500;        /* 2500 * 0,4 us = 1 ms oder 1 kHz */
   parameter.modul = 1;                /* Modulsteckplatz = 1 */
   parameter.anzkan = 1;               /* Ein Kanal */
   parameter.anzmess = 1000;           /* Tausend Messpunkte */

   ml8rt_exit();                       /* Register wieder restaurieren */
}


void PFAR start(void)
{
   ulong  datalength;

   ml8rt_entry();

   /* ---------------- Parameterpruefungen ----------------- */
   datalength = parameter.tdt.datalast - parameter.tdt.datafirst;
   if (datalength < (parameter.anzkan * 2 * parameter.anzmess))
      parameter.status = DATA_ERROR;   /* Datenbereich zu klein! */

   if ((parameter.modul<1) || (parameter.modul>10))
      parameter.status = MODUL_ERROR;  /* Falscher Modulsteckplatz */

   if ((parameter.anzkan<1) || (parameter.anzkan>8))
      parameter.status = CHANNEL_ERROR;/* Unzulaessige Kanalzahl */

   if ((parameter.status & ERROR) == 0)
   {
      ml8rt_local_led_on();            /* LED eingeschalten */

      /* Schreibzeiger fuer Datenbereich der Task zuruecksetzen */
      ml8rt_reset_w_pointer(parameter.tdt.task);

      if( parameter.modul <= 4 )       /* Modulbasisadresse berechnen */
	     mba = 0x100 * parameter.modul + 0x300;	      /* Auf Basiskarte */
      else
	     mba = 0x20 * (parameter.modul-4) + 0x400;	   /* Auf Extender */

      io_ad_status = mba + 0x02;       /* I/O-Adressen fuer Modulzugriffe */
      io_start_ad = mba;               /* (siehe Modul-Beschreibung) */
      io_data = mba + 0x04;
      io_kanal = mba + 0x02;
      cntmess = 0;

      /*---------------------- Initialisieren des Moduls -------------------*/
      /* Mode einstellen (s. Handbuch)*/
      outportb(mba + 0x06, 4);
      outportb(mba + 0x06, 0);
      outportb(mba + 0x06, MSH12_MODE_1);
      outportb(mba, 0);
      outportb(mba + 0x06, 0);
      outportb(mba + 0x04, MSH12_MODE_2);

      /*Interruptleitung waehlen (s. Handbuch S. 31f)*/
      outportb(mba + 0x06, 4);
      outportb(mba + 0x06, MSH12_INTERRUPT);

      outportb(io_kanal, 0);           /* Abtasttrigger aktivieren
					  und Kanal 0 einstellen */

      /* Timer-A initialisieren, evtl. gespeicherten Timerinterrupt loeschen
	      und durch das Demaskieren des Timers die Messung starten */
      ml8rt_set_timer(TIMER_A, parameter.abtastrate, INT_MODE);
      parameter.status = RUNNING;
      ml8rt_clear_int(IRQ_TIMER_A);
      ml8rt_unmask_int(IRQ_TIMER_A);
   }
   ml8rt_exit();
}


void PFAR stop(void)
{
   ml8rt_entry();
   ml8rt_local_led_off();              /* LED ausschalten (Messung beendet)*/
   ml8rt_mask_int (IRQ_TIMER_A);       /* Timer-Interrupt sperren */
   parameter.status = STOPPED;         /* Status auf "abgebrochen" */
   ml8rt_exit();
}


void PFAR main_proc(void)          /* ---- HAUPTPROZEDUR DER TASK ---- */
{
   byte    kanal;
   byte    b;
   ushort  w;
   long    abord_time;

   ml8rt_entry();

   inportb(io_start_ad);               /* Wandlung des ersten Kanals starten*/
   kanal = 0;                          /* (Kanal-0 !!) */
   do {
      abord_time = 0;
      while( (inport(io_ad_status) & 0x08000) &&
		  (abord_time < 100000) )
      {
	 abord_time++;			         /* warten, bis Status OK ist */
      }

      /* Messwert lesen und High Nibble ausblenden
	 und durch Dummyzugriff naechste Wandlung starten */
      w = inport(io_data) & 0x0FFF;
      inport(io_data);

      /* Ergebnis im Datenbereich der Task ablegen */
      ml8rt_write_data_word(parameter.tdt.task, w);

      kanal++;				               /* Kanal inkrementieren */
   } while (kanal<parameter.anzkan);   /* alle Kanaele messen */

   /* wenn alle Messungen gemacht wurden, Programm stoppen */
   if (++cntmess > parameter.anzmess)
   {
      ml8rt_local_led_off();           /* LED ausschalten, Messung beendet */
      ml8rt_mask_int(IRQ_TIMER_A);
      parameter.status = FINISHED;
   };

   /* Warten bis status <> busy, da der Kanal automatisch weitergeschaltet
      wird, wenn der AD-Wandler fertig ist und damit der einzustellende
      Kanal 0 wieder veraendert wuerde */
   abord_time = 0;
   while( (inport(io_ad_status) & 0x08000) &&
	   (abord_time < 100000) )
   {
      abord_time++;
   }
   outportb(io_kanal, 0);              /* Kanal-0 vorwaehlen */

   ml8rt_end_of_int(IRQ_TIMER_A);      /* Interrupt beenden */
   ml8rt_exit_interrupt();
}



void main (void)                 /* ---- PREPARE ---- */
{
   pdt.pdt_head.ucType          = 1;                     /* PDT-Typ */
   pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD);  /* Laenge des PDT-Vorspanns */
   pdt.pdt_head.usGlobalProc    = 4;                     /* Anzahl der Prozeduren */
   pdt.pdt_head.usProgNo        = PROGRAMM_NUMBER;       /* Programm-Nummer */
   pdt.pdt_head.cVersion        = VERSION;               /* Programm-Version */
   pdt.pdt_head.cRevision       = REVISION;              /* Programm-Revision */
   pdt.pdt_head.ucCPU           = _486DX;                /* CPU-Typ */
   pdt.pdt_head.ucCoProc        = _NOCOPROZ;             /* Co-Prozessor-Typ */
   pdt.pdt_head.ucLanguage      = _CPP31;                /* Programmiersprache */
   pdt.pdt_head.ucProgType      = _USER_PRG;             /* Programm-Typ */
   pdt.pdt_head.usFlags         = _DI_TASK               /* Flags */
	                               + _PDT_INFO_ENABLE + _LOCAL_PARAMETER;
   pdt.pdt_head.usInt           = IRQ_TIMER_A;           /* Interrupt-Nummer  */
   pdt.pdt_head.ulDataAdr       = 0;                     /* Adresse des Datenbereichs
                                                            (wird vom Betriebssystem festgelegt */
   pdt.pdt_head.ulDataSize      = 100000;                /* Datenbereichsgroessen */
   pdt.pdt_head.ulDataSizeMin   = 32;                    /* Minimale Datenbereichsgroesse */
   pdt.pdt_head.ulDataSizeMax   = 800000;                /* Maximale Datenbereichsgroesse */

   /* Anfangsadresse des Parameterbereichs und Anzahl Parameter setzen */
   ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
   pdt.pdt_head.ulGlobalParAdr  = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
   pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);

   pdt.pdt_head.ulHyperAdr      = 0;                     /* Adresse des Hypertextes */
   pdt.main_proc                = main_proc;             /* Adresse der Hauptprozedur */
   pdt.auto_init                = auto_init;             /* Adresse der Auto_Init Prozedur */
   pdt.start_proc               = start;                 /* Adresse der Start Prozedur */
   pdt.stop_proc                = stop;                  /* Adresse der Stop Prozedur */
   ml8rt_set_pdt_adr (&pdt);                             /* Adresse der PDT (fuer OsX) */
}
