/***************************************************************************
/**              Funktionsgenerator mit M-DA4-2 (MODULAR-4/486)
/**
/**  Dieses Programm gibt eine angegebene Anzahl von Stuetzstellen
/**  ueber das Analog-Modul M-DA4-2 aus. Die Daten muessen vor dem Starten
/**  in den Datenbereich der Task geschrieben werden.
/**
/**
/**  Installationshinweise:
/**
/**  Programmnummer  : 306h
/**  Tasknummer      : beliebig
/**  Interruptquelle : 92h (Timer-B, Slave-Controller)
/**  Datenbereich    : per Makrobefehl einstellbar
/**  Flags           : 1982h (Task aktivieren, Auto-Init aufrufen)
/**
/**
/**                       Borland C++ (Version 3.1)
/**                                                                                                    Version 1.C.                              **/
/**
/**                   Erstellungsdatum : 21.04.93
/**                    letzte nderung : 06.06.2000 (HK)
/**                             PDT auf neustem Stand
/**                             Status : freigegeben
/**                              Autor : M.H. & A.H.
/**
/**          (c) 1993 - 1996 SORCUS Computer GmbH Heidelberg
/**
/***************************************************************************/

#pragma option -r-                     /* Keine Registervariablen!!        */

#include "ml8rtbib.h"                  /* Echtzeitbibliothek einbinden     */
//#include "ml8makro.h"                /* Makro-Def. einiger ML8RTBIB Fkts.*/
#include "dos.h"                       /* outport, outportb etc als Makros */

				       /* Allgemeine Angaben               */
#define PROGRAMM_NUMBER 0x306          /* Programmnummer = 306h            */
#define VERSION         '1'            /* Version des Programms            */
#define REVISION        'E'            /* Revision des Programms           */

				       /* Statusmoeglichkeiten des Progr.  */
#define READY           0              /* Bereit                           */
#define RUNNING         2              /* Funktionsgenerator laeuft        */
#define FINISHED        3              /* Sweep beendet                    */
#define STOPPED         4              /* Programm wurde abgebrochen       */
#define ERROR           0x80           /* Fehler aufgetreten               */
#define DATA_ERROR      0x81           /* Fehler: Datenbereich zu klein    */
#define MODUL_ERROR     0x82           /* Fehler: Falscher Modulsteckplatz */
#define CHANNEL_ERROR   0x83           /* Fehler: Falsche Kanalzahl        */

				       /* Die verschiedenen Betriebsmodi   */
#define CONTI           0              /* Kontinuierliche Ausgabe          */
#define SWEEP           1              /* Einen Sweep ausgeben             */

				       /* Aufbau Prg.-Descriptor-Tab.(PDT) */
struct pdt_type
{
 ML8_PDT_HEAD pdt_head;       /* PDT-Header                       */

 void* PFAR   main_proc;      /* Adresse der Haupt-Prozedur       */
 void* PFAR   auto_init;      /* Adresse der Auto-Init Prozedur   */
 void* PFAR   start_proc;     /* Adresse der Start-Prozedur       */
 void* PFAR   stop_proc;      /* Adresse der Stop-Prozedur        */
} pdt;


				       /* Aufbau des Parameterbereichs     */
struct parameter_type
{
 tdt_type    tdt;             /* Platz fuer Task-Descriptor-Tab.  */
 byte        status;          /* Status d. Programms (READY,...)  */
 ushort      samplerate;      /* Ausgaberate * 0.4 us =           */
			      /* Zeit zwischen 2 Ausgabewerten    */
 byte        mode;            /* Betriebsart: 0=Kontinuierlich,   */
			      /*              1=Single-Sweep      */
 byte        modul;           /* Modulsteckplatz                  */
 byte        firstchan;       /* Erster auszugeb. Kanal (0 bis 3) */
 byte        lastchan;        /* Letzter auszugeb. Kanal (0 bis 3)*/
 ulong       number;          /* Anzahl der Stuetzstellen/Kanal   */
			      /* je Sweep                         */
} parameter;


ulong        samples;         /* Anzahl der ausgegebenen Werte    */
ushort       mba;             /* Modulbasisadresse                */
ushort       io_lsb_data[4];  /* IO-Adresse des Lo-Bytes (4 Kan.) */
ushort       io_msb_data[4];  /* IO-Adresse des Hi-Bytes (4 Kan.) */
ushort       io_output;       /* IO-Adresse zum Aktivieren der    */
			      /* eingetragenen DA-Werte           */


void PFAR auto_init(void);                  /* Prototypen der Prozeduren        */
void PFAR main_task(void);
void PFAR start(void);
void PFAR stop(void);

void PFAR default_function(void);


void PFAR default_function(void)            /* Prozedur erzeugt eine Rechteck-  */
					    /* funktion auf Kanal-0 und Kanal-1 */
					    /* im Datenbereich der Task         */
{
 ushort task;

 task = parameter.tdt.task;            /* Tasknummer, unter der das Prgr.  */
				       /* installiert wurde, ermitteln     */
				       /* (steht in der TDT)               */
 ml8rt_reset_w_pointer(task);          /* Datenbereichs-Write-Pointer auf  */
				       /* Anfang des Datenbereichs setzen  */

 parameter.firstchan = 0;              /* Erster Kanal = Kanal-0           */
 parameter.lastchan = 1;               /* Letzter Kanal = Kanal-1          */
				       /* (insgesamt 2 Kanaele)            */
 ml8rt_write_data_word(task, 0);       /* Erster Datenwert (=Stuetzstelle) */
				       /* fuer Kanal-0                     */
 ml8rt_write_data_word(task, 0);       /* Erster Datenwert      "          */
				       /* fuer Kanal-1                     */
 ml8rt_write_data_word(task, 0xFFF);   /* Zweiter Datenwert     "          */
				       /* fuer Kanal-0                     */
				       /* Zweiter Datenwert     "          */
 ml8rt_write_data_word(task, 0xFFF);   /* fuer Kanal-1                     */

 parameter.number = 2;                 /* Insgesamt 2 Stuetzstellen/ Kanal */
				       /* (hier eine Rechteckfunktion)     */
}


void PFAR auto_init(void)                   /* AUTO-INITIALISIERUNG             */
{
 ml8rt_entry();                        /* Register retten und vorbereiten  */

 ml8rt_local_led_off();                /* On-Board LED ausschalten         */
 parameter.status = READY;             /* Status des Programms "BEREIT"    */
 parameter.samplerate = 2500;          /* 2500 * 0,4 us = 1 ms oder 1 kHz  */
 parameter.modul = 1;                  /* Modulsteckplatz = 1              */
 parameter.mode = CONTI;               /* Betriebsart auf kontinuierlich   */
 parameter.firstchan = 0;              /* Erster auszugebender Kanal       */
 parameter.lastchan = 0;               /* Letzter auszugebender Kanal      */
 parameter.number = 0;                 /* Stuetzstellen je Sweep, je Kanal */

 default_function();                   /* Eine Testfunktion im Datenber.   */
				       /* eintragen                        */

 ml8rt_exit();                         /* Register wieder restaurieren     */
}


void PFAR start(void)
{
 ulong datalength;
 byte channel;

 ml8rt_entry();

 if(parameter.lastchan < parameter.firstchan)
   parameter.status = CHANNEL_ERROR;   /* Letzter Kanal groesser 1. Kanal  */

 if((parameter.firstchan>3) || (parameter.lastchan>3))
   parameter.status = CHANNEL_ERROR;   /* Eine der Kanalangaben ist > 3    */
				       /* oder kleiner 0                   */

 datalength = parameter.tdt.datalast - parameter.tdt.datafirst;
 if(datalength < ((parameter.lastchan - parameter.firstchan + 1) * 2 * parameter.number))
   parameter.status = DATA_ERROR;      /* Datenbereich zu klein!           */

 if((parameter.modul<1) || (parameter.modul>4))
   parameter.status = MODUL_ERROR;     /* Falscher Modulsteckplatz         */

 if((parameter.status & ERROR) == 0)
   {                                   /* Wenn alles OK ist, dann ...      */
				       /* Zeiger auf Datenbereichsanfang   */
    ml8rt_reset_r_pointer(parameter.tdt.task);
    samples = 0;                       /* Anzahl der ausgegeb. Werte = 0   */

				       /* Modulbasisadresse berechnen      */
    mba = 0x100 * parameter.modul + 0x300;
    io_lsb_data[0] = mba;              /* (I/O-Adr. siehe Modul-Beschr.)   */
    io_lsb_data[1] = mba + 2;          /* Ausgabewert Low-Byte             */
    io_lsb_data[2] = mba + 4;
    io_lsb_data[3] = mba + 6;
    io_msb_data[0] = mba + 1;          /* Ausgabewert High-Byte            */
    io_msb_data[1] = mba + 3;
    io_msb_data[2] = mba + 5;
    io_msb_data[3] = mba + 7;
    io_output = mba + 8;               /* Eingetragene Werte aktivieren    */

    if(parameter.number > 0)
      {
       for(channel = parameter.firstchan; channel <= parameter.lastchan; channel++)
	 {                             /* Die ersten Werte in die Register */
				       /* des Moduls eintragen             */
	  outportb(io_lsb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
	  outportb(io_msb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
	 }
       ml8rt_local_led_on();           /* Eingeschaltete LED zeigt an,     */
				       /* dass die Ausgabe laeuft          */
				       /* Timer-B initialisieren           */
       ml8rt_set_timer(TIMER_B, parameter.samplerate, INT_MODE);
       parameter.status = RUNNING;
       ml8rt_unmask_int(IRQ_TIMER_B);  /* Durch Demaskieren des Timers     */
				       /* wird die Ausgabe gestartet       */
      }
   }

 ml8rt_exit();
}


void PFAR stop(void)
{
 ml8rt_entry();
 ml8rt_local_led_off();                /* LED aus, Ausgabe beendet         */
 ml8rt_mask_int (IRQ_TIMER_B);         /* Timer-Interrupt sperren          */
 parameter.status = STOPPED;           /* Status auf "abgebrochen"         */
 ml8rt_exit();
}


void PFAR main_task(void)                   /* HAUPTPROZEDUR DER TASK           */
{
 byte channel;

 ml8rt_entry();

 outportb(io_output, 0x00);            /* Die neuen Werte ausgeben         */

 if(++samples >= parameter.number)     /* Wurden alle Werte eines Sweeps   */
				       /* ausgegeben?                      */
   {                                   /* Wenn ja, dann ...                */
    switch(parameter.mode)
      {
       case CONTI :                    /* Bei kontinuierlicher Ausgabe,    */
				       /* von vorne anfangen               */
	    ml8rt_reset_r_pointer(parameter.tdt.task);
				       /* Zeiger auf Datenbereichsanfang   */
	    samples = 0;               /* Anzahl der Ausgaben auf Null     */
	    for(channel = parameter.firstchan; channel <= parameter.lastchan; channel++)
	      {                        /* Die naechsten Werte eintragen    */
	       outportb(io_lsb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
	       outportb(io_msb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
	      }
	    break;

       default :
	    ml8rt_local_led_off();     /* LED aus, Ausgabe beendet         */
				       /* Interrupt maskieren              */
	    ml8rt_mask_int(IRQ_TIMER_B);
	    parameter.status = FINISHED;
	    break;
      }
   }
 else
   {
    for(channel = parameter.firstchan; channel <= parameter.lastchan; channel++)
      {                                /* Ausgabe-Werte in die Register    */
				       /* des Moduls eintragen             */
       outportb(io_lsb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
       outportb(io_msb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
      }
   }


 ml8rt_end_of_int(IRQ_TIMER_B);
 ml8rt_exit_interrupt();
}



void main (void)                       /* PREPARE                          */
{
 pdt.pdt_head.ucType          = 1;                     /* PDT-Typ einstellen (im Moment    */
				                                           /* wird nur Typ 1 unterstuetzt)     */
 pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD);  /* Laenge des PDT-Vorspanns (im     */
				                                           /* Moment immer = 48)               */
 pdt.pdt_head.usGlobalProc    = 4;                     /* Anzahl der Prozeduren (in diesem */
				                                           /* Beispiel "Auto_Init",            */
				                                           /*          "Main_Task",            */
				                                           /*          "Start",                */
				                                           /*          "Stop" )                */
 pdt.pdt_head.usProgNo        = PROGRAMM_NUMBER;       /* Programm-Nummer                  */
 pdt.pdt_head.cVersion        = VERSION;               /* Programm-Version                 */
 pdt.pdt_head.cRevision       = REVISION;              /* Programm-Revision                */
 pdt.pdt_head.ucCPU           = _486DX;                /* CPU-Type                         */
 pdt.pdt_head.ucCoProc        = _NOCOPROZ;             /* Co-Prozessor-Typ                 */
 pdt.pdt_head.ucLanguage      = _CPP31;                /* Programmiersprache = C++         */
 pdt.pdt_head.ucProgType      = _USER_PRG;             /* Programm-Typ: Anwenderprogramm   */
 pdt.pdt_head.usFlags         = 0;                     /* Flags                            */
 pdt.pdt_head.usFlags         = _DI_TASK + _PDT_INFO_ENABLE + _LOCAL_PARAMETER;
 pdt.pdt_head.usInt           = IRQ_TIMER_B;           /* Interrupt-Nummer des Programms   */
 pdt.pdt_head.ulDataAdr       = 0;                     /* Adresse des Datenbereichs        */
 pdt.pdt_head.ulDataSize      = 100000;                /* Datenbereichsgroessen            */
 pdt.pdt_head.ulDataSizeMin   = 32;                    /* Minimale Datenbereichsgroesse    */
 pdt.pdt_head.ulDataSizeMax   = 800000;                /* Maximale Datenbereichsgroesse    */
				                                           /* Anfangsadresse des Parameter-    */
				                                           /* bereichs errechnen               */
 ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
 pdt.pdt_head.ulGlobalParAdr  = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
				                                           /* Anzahl der Parameter eintragen   */
 pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);
 pdt.pdt_head.ulHyperAdr      = 0;                     /* Adresse des Hypertextes (wird    */
				                                           /* nicht verwendet)                 */
 pdt.main_proc                = main_task;             /* Adresse der Hauptprozedur        */
 pdt.auto_init                = auto_init;             /* Adresse der Auto_Init Prozedur   */
 pdt.start_proc               = start;                 /* Adresse der Start Prozedur       */
 pdt.stop_proc                = stop;                  /* Adresse der Stop Prozedur        */
 ml8rt_set_pdt_adr(&pdt);                              /* PDT-Adresse dem OsX mitteilen    */
}
