/***************************************************************************/
/**                    Funktionsgenerator (MODULAR-4/486)          	  **/
/**                                                                       **/
/**  Dieses Programm gibt eine angegebene Anzahl von Sttzstellen         **/
/**  ber ein Analog-Modul (siehe parameter.modul_type)                   **/
/**	  M-DA4-2, M-DA16-2 oder M-5B-1/U aus                             **/
/**  Die Daten mssen vor dem Starten in den Datenbereich der Task        **/
/**  geschrieben werden.                     				  **/
/**                                                                       **/
/**                                                                       **/
/**  Installationshinweise:                                               **/
/**                                                                       **/
/**  Programmnummer  : 30Bh						  **/
/**  Tasknummer      : beliebig                                           **/
/**  Interruptquelle : 92h (Timer-B, Slave-Controller)                    **/
/**  Datenbereich    : per Makrobefehl einstellbar                        **/
/**  Flags           : 1982h (Task aktivieren, Auto-Init aufrufen)        **/
/**                                                                       **/
/**                                                                       **/
/**                       Borland C++ (Version 3.1)                       **/
/**                                                                       **/
/**                             Version 1.D.                              **/
/**                                                                       **/
/**                   Erstellungsdatum : 30.09.93                         **/
/**                    letzte nderung : 19.05.00 (HK)                    **/
/**                    ml8rt_end_of_int statt outportb...                 **/
/**                    Pointerzugriffe ber Bibliothek                    **/
/**                         PDT auf neustem Stand                         **/
/**                             Status : freigegeben                      **/
/**                              Autor : G.R.                             **/
/**                                                                       **/
/**          (c) 1993 SORCUS Computer GmbH Heidelberg                     **/
/**                                                                       **/
/***************************************************************************/

#include "ml8rtbib.h"        /* Echtzeitbibliothek einbinden */
//#include "ml8makro.h"        /* Makro-Definitionen einiger ML8RTBIB Routinen einbinden (optional) */
#include "dos.h"             /* outport, outportb etc als Makros */

const _8087 = 3;             /* ---- 80387 Code erzwingen (speziell die SIN, ... Fktnen) ---- */

			     /* ---- Allgemeine Angaben ---- */
#define PROGRAMM_NUMBER 0x30B             /* Programmnummer = 30Bh */
#define VERSION         '1'               /* Version des Programms */
#define REVISION        'D'               /* Revision des Programms */

			     /* ---- Die verschiedenen Statusmglichkeiten des Programms ---- */
#define READY           0                 /* Bereit */
#define RUNNING         2                 /* Funktionsgenerator luft */
#define FINISHED        3                 /* Sweep beendet */
#define STOPPED         4                 /* Programm wurde abgebrochen */
#define ERROR           0x80              /* Fehler aufgetreten */
#define DATA_ERROR      0x81              /* Fehler: Datenbereich zu klein */
#define MODUL_ERROR     0x82              /* Fehler: Falscher Modulsteckplatz angewhlt */
#define CHANNEL_ERROR   0x83              /* Fehler: Falsche Kanalzahl */
#define BAD_MODUL_TYPE  0x84              /* Fehler: Undefinierbares Modul */
#define BAD_PARAMETER   0x85              /* Fehler: Parameteranzahl zu klein */

				  /* ---- Die verschiedenen Betriebsmodi ---- */
#define CONTI           0                 /* Kontinuierliche Ausgabe */
#define SWEEP           1                 /* Einen Sweep ausgeben */

				  /* ---- Die verschiedenen Modultypen ---- */
#define M_DA4_2_TYPE	9		  /* M-DA4-2, 4 Kanle, 12 Bit */
#define M_DA16_2_TYPE	29		  /* M-DA16-2, 2 Kanle, 16 Bit */
#define M_5B_1U_TYPE	20		  /* M-5B_1U, 1 Kanal, 12 Bit */

				  /* ---- Zustzliche Definitionen ---- */
#define WAIT_TIME 	10		  /* Wartezeit fr Datenbergabe an Modul, M-DA16-2*/

struct pdt_type
{
   ML8_PDT_HEAD pdt_head;                    /* PDT-Header */
   void* PFAR   main_proc;                   /* Adresse der Haupt-Prozedur (Interrupt-Task) */
   void* PFAR   auto_init;                   /* Adresse der Auto-Init Prozedur */
   void* PFAR	 start_proc;                  /* Adresse der Start-Prozedur */
   void* PFAR 	 stop_proc;                   /* Adresse der Stop-Prozedur */
} pdt;


			     /* ---- Aufbau des Parameterbereichs ---- */
struct parameter_type
{
   tdt_type    tdt;              /* Platz fr die Task-Descriptor-Tabelle */
   byte        modul_type;	 /* Modultyp, M-DA4-2, M-DA16-2, M-5B-1U */
   byte        status;           /* Status des Programms (READY etc.) */
   ushort      samplerate;       /* Ausgaberate * 0.4 us = Zeit zwischen 2 Ausgabewerten */
   byte        mode;             /* Betriebsart: 0=Kontinuierlich, 1=Single-Sweep */
   byte        modul;            /* Modulsteckplatz */
   byte        firstchan;        /* Erster auszugebender Kanal (0 bis 1) */
   byte        lastchan;         /* Letzter auszugebender Kanal (0 bis 1, jedoch grer/gleich "firstchan") */
   ulong       number;           /* Anzahl der Sttzstellen je Kanal je Sweep */
} parameter;


ulong          samples;         /* Anzahl der schon gemachten Ausgaben */
ushort         mba;             /* Modulbasisadresse */
ushort         io_lsb_data[4];  /* IO-Adresse des Low-Bytes (4 DA-Kanle) */
ushort         io_msb_data[4];  /* IO-Adresse des High-Bytes (4 DA-Kanle) */
ushort         io_output;       /* IO-Adresse zum Aktivieren der eingetragenen DA-Werte */
ushort         io_power_down;   /* IO-Adresse zum Veringern der Stromaufnahme */


void PFAR auto_init(void);                     /* Prototypen der benutzten Prozeduren */
void PFAR main_task(void);
void PFAR start(void);
void PFAR stop(void);

void PFAR default_function(void);

void PFAR (*modul_task)(void);
void PFAR m_da4_2_task(void);
void PFAR m_da16_2_task(void);
void PFAR m_5b_1u_task(void);


void PFAR default_function(void)               /* Diese Prozedur erzeugt eine Rechteckfunktion auf Kanal-0 und Kanal-1 im Datenbereich der Task. */
{
   ushort task;

   task = parameter.tdt.task;             /* Tasknummer, unter der das Programm installiert wurde, ermitteln (steht in der TDT) */
   ml8rt_reset_w_pointer(task);           /* Datenbereichs-Write-Pointer auf den Anfang des Datenbereichs setzen. */

   switch( parameter.modul_type ) {
      case M_DA4_2_TYPE:
	 parameter.firstchan = 0;               /* Erster Kanal = Kanal-0 */
	 parameter.lastchan = 1;                /* Letzter Kanal = Kanal-1 (insgesamt also 2 Kanle) */
	 ml8rt_write_data_word(task, 0);        /* Erster Datenwert (=Sttzstelle) fr Kanal-0 */
	 ml8rt_write_data_word(task, 0);        /* Erster Datenwert      "          "  Kanal-1 */
	 ml8rt_write_data_word(task, 0xFFF);    /* Zweiter Datenwert     "          "  Kanal-0 */
	 ml8rt_write_data_word(task, 0xFFF);    /*   "        "          "          "  Kanal-1 */
	 parameter.number = 2;                  /* Insgesamt 2 Sttzstellen je Kanal (in diesem Beispiel eine Rechteckfunktion) */
	 break;

      case M_DA16_2_TYPE:
	 parameter.firstchan = 0;               /* Erster Kanal = Kanal-0 */
	 parameter.lastchan  = 1;               /* Letzter Kanal = Kanal-1 (insgesamt also 2 Kanle) */
	 ml8rt_write_data_word(task, 0xFF7F);   /* Erster Datenwert (=Sttzstelle) fr Kanal-0 */
	 ml8rt_write_data_word(task, 0xFF7F);   /* Erster Datenwert      "          "  Kanal-1 */
	 ml8rt_write_data_word(task, 0x0080);   /* Zweiter Datenwert     "          "  Kanal-0 */
	 ml8rt_write_data_word(task, 0x0080);   /*   "        "          "          "  Kanal-1 */
	 parameter.number = 2;                  /* Insgesamt 2 Sttzstellen je Kanal (in diesem Beispiel eine Rechteckfunktion) */
	 break;

      case M_5B_1U_TYPE:
	 parameter.firstchan = 0;               /* Erster Kanal = Kanal-0 */
	 parameter.lastchan  = 0;               /* Letzter Kanal = Kanal-1 (insgesamt also 1 Kanal) */
	 ml8rt_write_data_word(task, 0x0000);   /* Erster Datenwert (=Sttzstelle) fr Kanal-0 */
	 ml8rt_write_data_word(task, 0xFF0F);   /* Zweiter Datenwert      "          "  Kanal-1 */
	 parameter.number = 2;                  /* Insgesamt 2 Sttzstellen je Kanal (in diesem Beispiel eine Rechteckfunktion) */
	 break;

      default:
	 parameter.status = BAD_MODUL_TYPE;
	 break;
   }
}


void PFAR auto_init(void)             /* ---- AUTO-INITIALISIERUNG ---- */
{
   ml8rt_entry();                         /* Register retten und vorbereiten */

   ml8rt_local_led_off();                 /* On-Board LED ausschalten */
   parameter.modul_type	= M_DA4_2_TYPE;	  /* Default Wert fr Typ = M-DA4-2 */
   parameter.status 	= READY;          /* Status des Programms auf "BEREIT" */
   parameter.samplerate = 2500;           /* 2500 * 0,4 us = 1 ms oder 1 kHz */
   parameter.modul 	= 1;              /* Modulsteckplatz = 1 */
   parameter.mode 	= CONTI;          /* Betriebsart auf kontinuierlich */
   parameter.firstchan 	= 0;              /* Erster auszugebender Kanal */
   parameter.lastchan 	= 0;              /* Letzter auszugebender Kanal */
   parameter.number 	= 0;              /* Sttzstellen je Sweep, je Kanal */

   ml8rt_exit();                          /* Register wieder restaurieren */
}


void PFAR start(void)
{
   ulong     	datalength;
   byte         channel;
   short	wait;

   ml8rt_entry();

   default_function();                    /* Eine Testfunktion im Datenbereich eintragen */

   if (parameter.lastchan < parameter.firstchan)
      parameter.status = CHANNEL_ERROR;   /* Letzter Kanal grer erster Kanal */

   if( ((parameter.lastchan > 3) && (parameter.modul_type == M_DA4_2_TYPE)) ||
       ((parameter.lastchan > 1) && (parameter.modul_type == M_DA16_2_TYPE)) ||
       ((parameter.lastchan > 0) && (parameter.modul_type == M_5B_1U_TYPE)) )
      parameter.status = CHANNEL_ERROR;   /* Eine der Kanalangaben ist grer 1 oder kleiner 0 */

   datalength = parameter.tdt.datalast - parameter.tdt.datafirst;
   if (datalength < ((parameter.lastchan - parameter.firstchan + 1) * 2 * parameter.number))
      parameter.status = DATA_ERROR;      /* Datenbereich zu klein! */

   if ((parameter.modul<1) || (parameter.modul>10))
      parameter.status = MODUL_ERROR;     /* Falscher Modulsteckplatz angewhlt */

   if ((parameter.status & ERROR) == 0)
   {                                      /* Wenn alles OK ist, dann ... */
      ml8rt_reset_r_pointer(parameter.tdt.task);  /* Zeiger auf den Datenbereichsanfang */
      samples = 0;                        /* Anzahl der gemachten Ausgabe auf Null setzen */

      if( parameter.modul <= 4 )          /* Modulbasisadresse berechnen */
	 mba = 0x100 * parameter.modul + 0x300;		/* Auf Basiskarte */
      else
	 mba = 0x20 * (parameter.modul-4) + 0x400;	/* Auf Extender */

      switch( parameter.modul_type ) {
	 case M_DA4_2_TYPE:
	    modul_task     = *m_da4_2_task;    	/* Adresse der M-DA4-2 Task */
	    io_lsb_data[0] = mba;               /* (I/O-Adressen siehe Modul-Beschreibung) */
	    io_lsb_data[1] = mba + 2;           /* Ausgabewert Low-Byte */
	    io_lsb_data[2] = mba + 4;
	    io_lsb_data[3] = mba + 6;
	    io_msb_data[0] = mba + 1;           /* Ausgabewert High-Byte */
	    io_msb_data[1] = mba + 3;
	    io_msb_data[2] = mba + 5;
	    io_msb_data[3] = mba + 7;
	    io_output      = mba + 8;           /* Die eingetragenen Werte bergeben */
	    if (parameter.number > 0)
	    {
	       for (channel = parameter.firstchan; channel <= parameter.lastchan; channel++)
	       {                                /* Die ersten Werte ausgeben */
		  outportb(io_lsb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
		  outportb(io_msb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
	       }
	    }
	    else {
	       parameter.status = BAD_PARAMETER;
	    }
	    break;

	 case M_DA16_2_TYPE:
	    modul_task     = *m_da16_2_task;    /* Adresse der M-DA16-2 Task */
	    io_lsb_data[0] = mba + 2;           /* (I/O-Adressen siehe Modul-Beschreibung) */
	    io_lsb_data[1] = mba + 6;           /* Ausgabewert Low-Byte */
	    io_msb_data[0] = mba + 1;           /* Ausgabewert High-Byte */
	    io_msb_data[1] = mba + 5;
	    io_power_down  = mba + 3;           /* Stromaufnahme des M-DA16-2 gering halten */
	    if (parameter.number > 0)
	    {
	       for (channel = parameter.firstchan; channel <= parameter.lastchan; channel++)
	       {                                /* Die ersten Werte ausgeben */
		  outportb(io_msb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
		  for(wait=0;wait<WAIT_TIME;wait++);
		  outportb(io_power_down, 0xFF );
		  for(wait=0;wait<WAIT_TIME;wait++);
		  outportb(io_lsb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
		  for(wait=0;wait<WAIT_TIME;wait++);
		  outportb(io_power_down, 0xFF );
		}
	    }
	    else {
	       parameter.status = BAD_PARAMETER;
	    }
	    break;

	 case M_5B_1U_TYPE:
	    modul_task     = *m_5b_1u_task;    	/* Adresse der M-5B-1U Task */
	    io_lsb_data[0] = mba + 2;           /* Ausgabewert Low-Byte */
	    io_msb_data[0] = mba + 1;           /* Ausgabewert High-Byte */
	    io_output      = mba + 6;           /* Die eingetragenen Werte bergeben */
	    if (parameter.number > 0)
	    {
	       outportb(io_lsb_data[0], ml8rt_read_data_byte(parameter.tdt.task));
	       outportb(io_msb_data[0], ml8rt_read_data_byte(parameter.tdt.task));
	    }
	    else {
	       parameter.status = BAD_PARAMETER;
	    }
	    break;

	 default:
	    parameter.status = BAD_MODUL_TYPE;
	    break;
      }

      if ((parameter.status & ERROR) == 0) {
	 ml8rt_local_led_on();            /* Eingeschaltete LED zeigt an, da die Ausgabe luft. */
	 ml8rt_set_timer(TIMER_B, parameter.samplerate, INT_MODE);
	 parameter.status = RUNNING;      /* Timer-A initialisieren */
	 ml8rt_unmask_int(IRQ_TIMER_B);   /* Durch das Demaskieren des Timers wird die Ausgabe gestartet. */
      }
   }

   ml8rt_exit();
}


void PFAR stop(void)
{
   ml8rt_entry();
   ml8rt_local_led_off();                 /* LED ausschalten. (Ausgabe beendet) */
   ml8rt_mask_int (IRQ_TIMER_B);          /* Timer-Interrupt sperren */
   parameter.status = STOPPED;            /* Status auf "abgebrochen" */
   ml8rt_exit();
}


void PFAR main_task(void)          /* ---- HAUPTPROZEDUR DER TASK ---- */
{
   ml8rt_entry();
   modul_task();
   ml8rt_exit_interrupt();
}


void PFAR m_da16_2_task(void)     /* ---- Ausgaberoutine M-DA16-2 ---- */
{
   byte	channel;
   short wait;

   if (++samples >= parameter.number)     /* Wurden alle Werte eines Sweeps ausgegeben? */
   {                                      /* Wenn ja, dann ... */
      switch (parameter.mode) {
	 case CONTI :                     /* Bei kontinuierlicher Ausgabe, wieder von vorne anfangen */
	    ml8rt_reset_r_pointer(parameter.tdt.task);  /* Zeiger auf den Datenbereichsanfang */
	    samples = 0;                  /* Anzahl der Ausgaben wieder auf Null setzen */
	    for (channel = parameter.firstchan; channel <= parameter.lastchan; channel++)
	    {                             /* 1. Werte ausgeben */
	       outportb(io_msb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
	       for(wait=0;wait<WAIT_TIME;wait++);
	       outportb(io_power_down, 0xFF );
	       for(wait=0;wait<WAIT_TIME;wait++);
	       outportb(io_lsb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
	       for(wait=0;wait<WAIT_TIME;wait++);
	       outportb(io_power_down, 0xFF );
	    }
	    break;

	 default :
	    ml8rt_local_led_off();        /* LED ausschalten, Messung beendet */
	    ml8rt_mask_int(IRQ_TIMER_B);  /* Interrupt maskieren */
	    parameter.status = FINISHED;
	    break;
      }
   }
   else
   {
      for (channel = parameter.firstchan; channel <= parameter.lastchan; channel++)
      {                                   /* Die Ausgabe der Daten */
	 outportb(io_msb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
	 for(wait=0;wait<WAIT_TIME;wait++);
	 outportb(io_power_down, 0xFF );
	 for(wait=0;wait<WAIT_TIME;wait++);
	 outportb(io_lsb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
	 for(wait=0;wait<WAIT_TIME;wait++);
	 outportb(io_power_down, 0xFF );
      }
   }

   ml8rt_end_of_int(IRQ_TIMER_B);       /* EOI an den Interrupt-Controller senden */
}


void PFAR m_da4_2_task(void)      /* ---- Ausgaberoutine M-DA4-2 ---- */
{
   byte channel;

   inport(io_output);                     /* Die neuen Werte ausgeben. */
   if (++samples >= parameter.number)     /* Wurden alle Werte eines Sweeps ausgegeben? */
   {                                      /* Wenn ja, dann ... */
      switch (parameter.mode) {
	 case CONTI :                     /* Bei kontinuierlicher Ausgabe, wieder von vorne anfangen */
	    ml8rt_reset_r_pointer(parameter.tdt.task);  /* Zeiger auf den Datenbereichsanfang */
	    samples = 0;                  /* Anzahl der Ausgaben wieder auf Null setzen */
	    for (channel = parameter.firstchan; channel <= parameter.lastchan; channel++)
	    {                             /* Die nchsten Werte schon mal wieder eintragen */
	       outportb(io_lsb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
	       outportb(io_msb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
	    }
	    break;

	 default :
	    ml8rt_local_led_off();        /* LED ausschalten, Messung beendet */
	    ml8rt_mask_int(IRQ_TIMER_B);  /* Interrupt maskieren */
	    parameter.status = FINISHED;
	    break;
      }
   }
   else
   {
      for (channel = parameter.firstchan; channel <= parameter.lastchan; channel++)
      {                                   /* Die Ausgabe-Werte in die Register des Moduls eintragen */
	 outportb(io_lsb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
	 outportb(io_msb_data[channel], ml8rt_read_data_byte(parameter.tdt.task));
      }
   }

   ml8rt_end_of_int(IRQ_TIMER_B);       /* EOI an den Interrupt-Controller senden */
}


void PFAR m_5b_1u_task(void)      /* ---- Ausgaberoutine M-5B-1U ---- */
{
   byte channel;

   outportb(io_output, 0x00);             /* Die neuen Werte ausgeben. */
   if (++samples >= parameter.number)     /* Wurden alle Werte eines Sweeps ausgegeben? */
   {                                      /* Wenn ja, dann ... */
      switch (parameter.mode) {
	 case CONTI :                     /* Bei kontinuierlicher Ausgabe, wieder von vorne anfangen */
	    ml8rt_reset_r_pointer(parameter.tdt.task);  /* Zeiger auf den Datenbereichsanfang */
	    samples = 0;                  /* Anzahl der Ausgaben wieder auf Null setzen */
	    outportb(io_lsb_data[0], ml8rt_read_data_byte(parameter.tdt.task));
	    outportb(io_msb_data[0], ml8rt_read_data_byte(parameter.tdt.task));
	    break;

	 default :
	    ml8rt_local_led_off();        /* LED ausschalten, Messung beendet */
	    ml8rt_mask_int(IRQ_TIMER_B);  /* Interrupt maskieren */
	    parameter.status = FINISHED;
	    break;
      }
   }
   else
   {
      outportb(io_lsb_data[0], ml8rt_read_data_byte(parameter.tdt.task));
      outportb(io_msb_data[0], ml8rt_read_data_byte(parameter.tdt.task));
   }

   ml8rt_end_of_int(IRQ_TIMER_B);       /* EOI an den Interrupt-Controller senden */
}



void main (void)                 /* ---- PREPARE ---- */
{
   pdt.pdt_head.ucType 	        = 1;                    /* PDT-Typ einstellen (im Moment wird nur Typ 1 untersttzt) */
   pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD); /* Lnge des PDT-Vorspanns (im Moment immer = 48) */
   pdt.pdt_head.usGlobalProc    = 4;                    /* Anzahl der Prozeduren (in diesem Beispiel "Auto_Init", "Main_Task", "Start", "Stop" ) */
   pdt.pdt_head.usProgNo        = PROGRAMM_NUMBER;      /* Programm-Nummer */
   pdt.pdt_head.cVersion        = VERSION;              /* Programm-Version */
   pdt.pdt_head.cRevision       = REVISION;             /* Programm-Revision */
   pdt.pdt_head.ucCPU           = _486DX;               /* CPU-Type */
   pdt.pdt_head.ucCoProc        = _NOCOPROZ;            /* Co-Prozessor-Typ */
   pdt.pdt_head.ucLanguage      = _CPP31;               /* Programmiersprache = C++ */
   pdt.pdt_head.ucProgType      = _USER_PRG;            /* Programm-Typ: Anwenderprogramm */
   pdt.pdt_head.usFlags         = _DI_TASK +            /* Flags */
		                            _PDT_INFO_ENABLE + _LOCAL_PARAMETER;
   pdt.pdt_head.usInt           = IRQ_TIMER_B;          /* Interrupt-Nummer fr die das Programm gedacht ist */
   pdt.pdt_head.ulDataAdr       = 0;                    /* Adresse des Datenbereichs (hier wird kein Datenbereich verwendet) */
   pdt.pdt_head.ulDataSize      = 100000;               /* Datenbereichsgren */
   pdt.pdt_head.ulDataSizeMin   = 32;                   /* Minimale Datenbereichsgre */
   pdt.pdt_head.ulDataSizeMax   = 800000;               /* Maximale Datenbereichsgre */
					  /* Anfangsadresse des Parameterbereichs errechnen */
   ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
   pdt.pdt_head.ulGlobalParAdr  = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
					  /* Anzahl der Parameter eintragen */
   pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);
   pdt.pdt_head.ulHyperAdr      = 0;                    /* Adresse des Hypertextes (wird nicht verwendet) */
   pdt.main_proc                = main_task;            /* Adresse der Hauptprozedur */
   pdt.auto_init                = auto_init;            /* Adresse der Auto_Init Prozedur */
   pdt.start_proc               = start;                /* Adresse der Start Prozedur */
   pdt.stop_proc                = stop;                 /* Adresse der Stop Prozedur */
   ml8rt_set_pdt_adr (&pdt);                            /* Adresse der PDT im Rckgabe-Record dem Betriebssystem mitteilen */
}
