/***************************************************************************/
/**      Analogdaten-Erfassungsprogramm mit M-5B-1/U (MODULAR-4/486)      **/
/**                                                                       **/
/**  Dieses Programm erfat mit einer einstellbaren Abtastrate bis zu     **/
/**  4 analoge Eingnge mit dem Modul M-5B-1/U. Die Parameter des 	  **/
/**  Programms mssen vor dem Start des Programms entsprechend		  **/
/**  eingestellt werden.         					  **/
/**  Die Gre des Datenbereichs kann per Makrobefehl eingestellt werden. **/
/**                                                                       **/
/**  Installationshinweise:                                               **/
/**                                                                       **/
/**  Programmnummer  : 30Ch						  **/
/**  Tasknummer      : beliebig                                           **/
/**  Interruptquelle : 91h (Timer-A, Slave-Controller)                    **/
/**  Datenbereich    : per Makrobefehl einstellbar                        **/
/**  Flags           : 1982h (Task aktivieren, Auto-Init aufrufen)        **/
/**                                                                       **/
/**                                                                       **/
/**                       Borland C++ (Version 3.1)                       **/
/**                                                                       **/
/**                             Version 1.C.                              **/
/**                                                                       **/
/**                   Erstellungsdatum : 30.09.93                         **/
/**                    letzte nderung : 19.05.00 (HK)                    **/
/**                    ml8rt_end_of_int statt outportb...                 **/
/**                    Pointerzugriffe ber Bibliothek                    **/
/**                         PDT auf neustem Stand                         **/
/**                             Status : freigegeben                      **/
/**                              Autor : G. Rudolf                        **/
/**                                                                       **/
/**          (c) 1993 SORCUS Computer GmbH Heidelberg                     **/
/**                                                                       **/
/***************************************************************************/


#include "dos.h"             /* outport, outportb etc. als Makros */
#include "ml8rtbib.h"        /* Echtzeitbibliothek einbinden */
//#include "ml8makro.h"        /* Makro-Definitionen einiger ML8RTBIB Routinen einbinden */
#pragma inline

const _8087 = 3;             /* ---- 80387 Code erzwingen (speziell fr SIN, ... Fktnen) ---- */

			     /* ---- Allgemeine Angaben ---- */
#define PROGRAMM_NUMBER 0x30C             /* Programmnummer = 30Ch */
#define VERSION         '1'               /* Version des Programms */
#define REVISION        'C'               /* Revision des Programms */

			     /* ---- Mgliche Werte des Parameters Status ---- */
#define READY           0                 /* Bereit */
#define RUNNING         2                 /* Messung luft */
#define FINISHED        3                 /* Messung beendet */
#define STOPPED         4                 /* Programm wurde abgebrochen */
#define ERROR           0x80              /* Fehler aufgetreten */
#define DATA_ERROR      0x81              /* Fehler: Datenbereich zu klein */
#define MODUL_ERROR     0x82              /* Fehler: Falscher Modulsteckplatz angewhlt */
#define CHANNEL_ERROR   0x83              /* Fehler: Falsche Kanalzahl */

			     /* ---- Aufbau der Programm-Descriptor-Tabelle (PDT) ---- */
struct pdt_type
{
   ML8_PDT_HEAD pdt_head;                    /* PDT-Header */
   void* PFAR   main_proc;                   /* Adresse der Haupt-Prozedur (Interrupt-Task) */
   void* PFAR   auto_init;                   /* Adresse der Auto-Init Prozedur */
   void* PFAR   start_proc;                  /* Adresse der Start-Prozedur */
   void* PFAR   stop_proc;                   /* Adresse der Stop-Prozedur */
} pdt;

			     /* ---- Aufbau des Parameterbereichs ---- */
struct parameter_type
{
   tdt_type     tdt;                         /* Platz fr die Task-Descriptor-Tabelle */
   byte         status;                      /* Status des Programms (READY etc.) */
   ushort       abtastrate;                  /* Abtastrate * 0.4 us = Zeit zwischen 2 Abtastwerten */
   byte         modul;                       /* Modulsteckplatz des M-AD16-3 */
   byte         anzkan;                      /* Anzahl der zu messenden Kanle */
   ushort       anzmess;                     /* Anzahl der Messungen */
} parameter;


ushort          cntmess;          /* Zhler fr Messungen */
ushort          mba;              /* Modulbasisadresse */
ushort          io_kanal;         /* I/O-Adresse fr die Kanalwahl */
ushort          io_ad_status;     /* I/O-Adresse fr Status des AD-Wandlers */
ushort          io_data;      	  /* I/O-Adresse zum Lesen der 16 Bit */
ushort          io_start_ad;      /* I/O-Adresse zum Starten der Wandlung */
byte            kanal;		  /* Aktuelle Kanalnummer */
byte            b;		  /* Byte */
ushort          w;		  /* Word */


void PFAR auto_init(void);                     /* Prototypen der benutzten Funktionen */
void PFAR main_task(void);
void PFAR start(void);
void PFAR stop(void);


void PFAR auto_init(void)             /* ---- AUTO-INITIALISIERUNG ---- */
{
   ml8rt_entry();                         /* Register retten und vorbereiten */

   ml8rt_local_led_off();                 /* On-Board LED ausschalten */
   parameter.status = READY;              /* Status des Programms auf "BEREIT" */
   parameter.abtastrate = 2500;           /* 2500 * 0,4 us = 1 ms oder 1 kHz */
   parameter.modul = 1;                   /* Modulsteckplatz = 1 */
   parameter.anzkan = 1;                  /* Ein Kanal */
   parameter.anzmess = 1000;              /* Tausend Messpunkte */

   ml8rt_exit();                          /* Register wieder restaurieren */
}


void PFAR start(void)
{
   ulong datalength;

   ml8rt_entry();

   /* ---------------- Parameterprfungen ----------------- */
   datalength = parameter.tdt.datalast - parameter.tdt.datafirst;
   if (datalength < (parameter.anzkan * 2 * parameter.anzmess))
      parameter.status = DATA_ERROR;      /* Datenbereich zu klein! */

   if ((parameter.modul<1) || (parameter.modul>10))
      parameter.status = MODUL_ERROR;     /* Falscher Modulsteckplatz angewhlt */

   if ((parameter.anzkan<1) || (parameter.anzkan>4))
      parameter.status = CHANNEL_ERROR;   /* Unzulssige Kanalzahl */

   if ((parameter.status & ERROR) == 0)
   {
      ml8rt_local_led_on();               /* Eingeschaltete LED zeigt an, da die Messung luft. */
      ml8rt_reset_w_pointer(parameter.tdt.task);  /* Zeiger auf den Datenbereichsanfang */

      if( parameter.modul <= 4 )          /* Modulbasisadresse berechnen */
	 mba = 0x100 * parameter.modul + 0x300;		/* Auf Basiskarte */
      else
	 mba = 0x20 * (parameter.modul-4) + 0x400;	/* Auf Extender */

      io_ad_status = mba + 18; 		   /* I/O-Adressen fr die Modulzugriffe ermitteln */
      io_start_ad  = mba + 20;             /* (siehe Modul-Beschreibung) */
      io_data      = mba + 26;
      io_kanal     = mba + 16;
      cntmess      = 0;

      /*---------------------- Initialisieren des Moduls -------------------*/
      /* (s. Handbuch S. 39) */
      outportb(mba + 22, 0x00);		  /* Setze RE = 1 */

      outportb(io_kanal, 0);              /* Kanal IN-0 einstellen */

      ml8rt_set_timer(TIMER_A, parameter.abtastrate, INT_MODE);
      parameter.status = RUNNING;         /* Timer-A initialisieren */
      ml8rt_clear_int(IRQ_TIMER_A);       /*evtl. gespeicherten Timerinterrupt lschen*/
      ml8rt_unmask_int(IRQ_TIMER_A);      /* Durch das Demaskieren des Timers wird die Messung gestartet. */
   }

   ml8rt_exit();
}


void PFAR stop(void)
{
   ml8rt_entry();
   ml8rt_local_led_off();                 /* LED ausschalten. (Messung beendet) */
   ml8rt_mask_int (IRQ_TIMER_A);          /* Timer-Interrupt sperren */
   parameter.status = STOPPED;            /* Status auf "abgebrochen" */
   ml8rt_exit();
}


void PFAR main_task(void)          /* ---- HAUPTPROZEDUR DER TASK ---- */
{
   byte           kanal;
   byte           kanal_adr;
   byte           b;
   ushort         w;
   long		  abord_time;

   ml8rt_entry();

   inportb(io_start_ad);                  /* AD-Wandlung des ersten Kanals starten */
   kanal = 0;                             /* (Kanal-0 !!) */
   kanal_adr = 0x40;
   outportb(io_kanal, kanal_adr);  	  /* Nchsten Kanal einstellen */
   do {
      abord_time = 0;
      while( (inportb(io_ad_status) & 0x01) &&
	     (abord_time < 100000) )
	 abord_time++;			  /* Solange warten, bis Status OK ist. */

      w = inport(io_data) >> 4;       	  /* Wandlung lesen und um 4 Bits nach rechts schieben*/

      inportb(io_start_ad);               /* AD-Wandlung des ersten Kanals starten */
      kanal++;				  /* Kanal weiterzhlen */
      kanal_adr += 0x40;
      outportb(io_kanal, kanal_adr);	  /* Nchsten Kanal einstellen */
      ml8rt_write_data_word(parameter.tdt.task,w); /* Ergebnis abspeichern */
					  /* Anschlieend wird der Pointer um zwei Byte erhht */
   } while (kanal<parameter.anzkan);      /* Solange messen, bis alle Kanle gesampled wurden */

   if (++cntmess > parameter.anzmess)     /* Wenn alle Messungen gemacht wurden, Programmlauf stoppen */
   {
      ml8rt_local_led_off();              /* LED ausschalten, Messung beendet */
      ml8rt_mask_int(IRQ_TIMER_A);
      parameter.status = FINISHED;
   }

   outportb(io_kanal, 0);                 /* Kanal-0 fr die nchste Messung schon mal anwhlen */

   ml8rt_end_of_int(IRQ_TIMER_A);       /* EOI an den Interrupt-Controller senden */
   ml8rt_exit_interrupt();
}




void main (void)                 /* ---- PREPARE ---- */
{
   pdt.pdt_head.ucType          = 1;                    /* PDT-Typ einstellen (im Moment wird nur Typ 1 untersttzt) */
   pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD); /* Lnge des PDT-Vorspanns (im Moment immer = 48) */
   pdt.pdt_head.usGlobalProc    = 4;                    /* Anzahl der Prozeduren (in diesem Beispiel "Auto_Init", "Main_Task", "Start", "Stop" ) */
   pdt.pdt_head.usProgNo        = PROGRAMM_NUMBER;      /* Programm-Nummer */
   pdt.pdt_head.cVersion        = VERSION;              /* Programm-Version */
   pdt.pdt_head.cRevision       = REVISION;             /* Programm-Revision */
   pdt.pdt_head.ucCPU           = _486DX;               /* CPU-Type */
   pdt.pdt_head.ucCoProc        = _NOCOPROZ;            /* Co-Prozessor-Typ */
   pdt.pdt_head.ucLanguage      = _CPP31;               /* Programmiersprache = C++ */
   pdt.pdt_head.ucProgType      = _USER_PRG;            /* Programm-Typ: Anwenderprogramm */
   pdt.pdt_head.usFlags         = _DI_TASK              /* Flags */
	                               + _PDT_INFO_ENABLE + _LOCAL_PARAMETER;
   pdt.pdt_head.usInt           = IRQ_TIMER_A;          /* Interrupt-Nummer fr die das Programm gedacht ist */
   pdt.pdt_head.ulDataAdr       = 0;                    /* Adresse des Datenbereichs (hier wird kein Datenbereich verwendet) */
   pdt.pdt_head.ulDataSize      = 100000;               /* Datenbereichsgren */
   pdt.pdt_head.ulDataSizeMin   = 32;                   /* Minimale Datenbereichsgre */
   pdt.pdt_head.ulDataSizeMax   = 800000;               /* Maximale Datenbereichsgre */
					  /* Anfangsadresse des Parameterbereichs errechnen */
   ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
   pdt.pdt_head.ulGlobalParAdr  = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
					  /* Anzahl der Parameter eintragen */
   pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);
   pdt.pdt_head.ulHyperAdr      = 0;                    /* Adresse des Hypertextes (wird nicht verwendet) */
   pdt.main_proc                = main_task;            /* Adresse der Hauptprozedur */
   pdt.auto_init                = auto_init;            /* Adresse der Auto_Init Prozedur */
   pdt.start_proc               = start;                /* Adresse der Start Prozedur */
   pdt.stop_proc                = stop;                 /* Adresse der Stop Prozedur */
   ml8rt_set_pdt_adr (&pdt);                            /* Adresse der PDT im Rckgabe-Record dem Betriebssystem mitteilen */
}
