/****************************************************************************/
/**           Beispiel fr die Verwendung eines Funktionsaufrufs           **/
/**                                                                        **/
/** Das Programm enthlt nur eine wichtige Prozedur. Sie dient zur Bildung **/
/** des Mittelwertes aus bis zu 256 Bytes, die vom PC oder von einer       **/
/** anderen Task per Funktionsaufruf bergeben werden.                     **/
/** Sehen Sie sich dazu auch das PC-Programm "MITT0311.PAS" an.            **/
/**                                                                        **/
/** Das Beispiel dient nur zur Veranschaulichung des Funktionsaufrufs und  **/
/** hat darber hinaus keine praktische Bedeutung.                         **/
/**                                                                        **/
/**  Installationshinweise:                                                **/
/**                                                                        **/
/**  Programmnummer  : 311h						   									**/
/**  Tasknummer      : beliebig                                            **/
/**  Interruptquelle : 0 (wird nicht verwendet, da NI-Task)                **/
/**  Datenbereich    : 0                                                   **/
/**  Flags           : 0x0180 (nicht aktiviert, kein Auto-Init)            **/
/**                                                                        **/
/**                                                                        **/
/**                                                                        **/
/**                             Version 1.D                                **/
/**                                                                        **/
/**                   Erstellungsdatum : 8.6.94                            **/
/**                    letzte nderung : 06.06.2000 (HK)                   **/
/**                             PDT auf neustem Stand                      **/
/**                             Status : freigegeben                       **/
/**                              Autor : M.H.                              **/
/**                                                                        **/
/**               (c) 1994..1996 SORCUS Computer GmbH Heidelberg           **/
/**                                                                        **/
/****************************************************************************/

#include "ml8rtbib.h"        /* Header fr die Echtzeitbibliothek einbinden */
//#include "ml8makro.h"        /* Makro-Header fr die wichtigsten Prozeduren einbinden */
#include "dos.h"
#include "stdlib.h"

				  /* ---- Allgemeine Angaben ---- */
#define PROGRAMM_NUMBER 0x311             /* Programmnummer = 311h */
#define VERSION         '1'               /* Version des Programms */
#define REVISION        'D'               /* Revision des Programms */



				  /* ---- Aufbau der Programm-Descriptor-Tabelle (PDT) ---- */
struct pdt_type
{
	ML8_PDT_HEAD pdt_head;               /* PDT-Header */

	void* PFAR   main_proc;              /* Adresse der Haupt-Prozedur (Task) */
	void* PFAR   auto_init;              /* Adresse der Auto-Init Prozedur */
	                                     /* Funktion zur Mittelwertbildung */
	void PFAR   (*mittel)(ushort task, ushort insize, void *inptr, ushort outsize,
			       long *outptr);
} pdt;


				  /* ---- Aufbau des Parameterbereichs ---- */
struct parameter_type
{
  tdt_type           tdt;          /* Platz fr die Task-Descriptor-Tabelle */
} parameter;                       /* Ansonsten werden keine weiteren Parameter bentigt. */


void PFAR auto_init(void);                  /* Prototypen der benutzten Prozeduren */
void PFAR main_task(void);
void PFAR mittel(ushort task, ushort insize, void *inptr,
		 ushort outsize, long *outptr);

/* Die Prozeduren auto_init und main_proc werden in diesem Beispiel nicht */
/* benutzt, sie werden nur deklariert, damit das System nicht abstrzt,   */
/* falls Sie versehentlich (z.B. beim Installieren) aufgerufen werden     */

void PFAR auto_init(void)         /* ---- AUTO-INITIALISIERUNG ---- */
{
	ml8rt_entry();                         /* Register retten und vorbereiten */
	ml8rt_exit();                          /* Register wieder restaurieren */
}


void PFAR mittel(ushort task, ushort insize, void *inptr,
		 ushort outsize, long *outptr)
{
long            *m;               /* Zeiger auf Long-Integer */
ushort          i;                /* Laufvariable */
ushort          error;            /* Fehlercode */
byte            *dataarr;         /* Pointer auf ein Byte */
ldiv_t          mittel;           /* Mittelwert */

  ml8rt_entry_function();                 /* Retten der Register, Setzen der Variablen des Funktionsaufrufs */
  m = outptr;                             /* m zeigt jetzt auf die Daten, die von der Funktion zurckgeliefert werden (Mittelwert) */
  *m = 0;                                 /* Mittelwert initialisieren */
  if (insize>0)                           /* wurde mindestens ein Byte bergeben? */
  {
	  dataarr = inptr;                     /* Zeiger auf Anfang der bergebenen Daten setzen */

	  for (i=0; i<insize; i++)
		  *m = *m + dataarr[i];             /* bergebene Bytes summieren */
	  mittel = ldiv (*m, insize);          /* Mittelwert bilden */
	  *m = mittel.quot;                    /* Ergebnis der Mittelwertbildung wieder in *m bertragen (m zeigt auf Ausgangsbereich!) */

	  error = 0;                           /* Kein Fehler */
  }
  else
	  error = 1;                           /* Fehler: keine Daten bergeben */

  ml8rt_exit_function(0,1,error);         /* Funktionsaufruf beenden:              */
					  /* Bedeutung der Parameter:              */
					  /* 0 : Alle bernommenen Bytes genutzt   */
					  /* 1 : Ein Byte wird zurckgegeben       */
					  /* error : Fehlermeldung                 */
}



void PFAR main_proc()             /* ---- HAUPTPROZEDUR DER TASK ---- */
{
	ml8rt_entry();
	ml8rt_exit();
}




void main ()                 /* ---- PREPARE ---- */
{
	pdt.pdt_head.ucType          = 1;                    /* PDT-Typ einstellen (im Moment wird nur Typ 1 untersttzt) */
	pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD); /* Lnge des PDT-Vorspanns (im Moment immer = 48) */
	pdt.pdt_head.usGlobalProc    = 3;                    /* Anzahl der Prozeduren */
	pdt.pdt_head.usProgNo        = PROGRAMM_NUMBER;      /* Programm-Nummer */
	pdt.pdt_head.cVersion        = VERSION;              /* Programm-Version */
	pdt.pdt_head.cRevision       = REVISION;             /* Programm-Revision */
	pdt.pdt_head.ucCPU           = _486DX;               /* CPU-Type */
	pdt.pdt_head.ucCoProc        = _487DX;               /* Co-Prozessor-Typ */
	pdt.pdt_head.ucLanguage      = _CPP31;               /* Programmiersprache = C++ */
	pdt.pdt_head.ucProgType      = _USER_PRG;            /* Programm-Typ: Anwenderprogramm */
	pdt.pdt_head.usFlags         = 0;                    /* Flags */
	pdt.pdt_head.usFlags         = _NI_TASK + _PDT_INFO_ENABLE + _LOCAL_PARAMETER;
	pdt.pdt_head.usInt           = 0;                    /* Interrupt-Nummer fr die das Programm gedacht ist (nicht relevant, da NI-Task) */
	pdt.pdt_head.ulDataAdr       = 0;                    /* Adresse des Datenbereichs (hier wird kein Datenbereich verwendet) */
	pdt.pdt_head.ulDataSize      = 0;                    /* Gre des Datenbereichs */
	pdt.pdt_head.ulDataSizeMin   = 0;                    /* Minimale Datenbereichsgre */
	pdt.pdt_head.ulDataSizeMax   = 0;                    /* Maximale Datenbereichsgre */
					                                         /* Anfangsadresse des Parameterbereichs errechnen */
	ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
	pdt.pdt_head.ulGlobalParAdr   = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
					                                         /* Anzahl der Parameter eshortragen */
	pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);
	pdt.pdt_head.ulHyperAdr      = 0;                    /* Adresse des Hypertextes (wird nicht verwendet) */
	pdt.main_proc                = main_proc;            /* Adresse der Hauptprozedur */
	pdt.auto_init                = auto_init;            /* Adresse der Auto_Init Prozedur */
	pdt.mittel                   = mittel;               /* Adresse der Mittelwertberechnung */
	ml8rt_set_pdt_adr (&pdt);                            /* Adresse der PDT im Rckgabe-Record dem Betriebssystem mitteilen */
}
