/***************************************************************************/
/**      Analogdaten-Erfassungsprogramm mit M-AD12-16 (MODULAR-4/486)     **/
/**                                                                       **/
/**  Dieses Programm erfat mit einer einstellbaren Abtastrate bis zu     **/
/**  16 analoge Eingnge mit dem Modul M-AD12-16. Die Parameter mssen    **/
/**  vor dem Start des Programms entsprechend eingestellt werden.         **/
/**  Die Gre des Datenbereichs kann per Makrobefehl eingestellt werden. **/
/**  Das Programm verwendet den Auto-Inkrement-Mode des M-AD12-16 Moduls. **/
/**  Das Programm M8P0313 ist identisch zu diesem Programm, jedoch benutzt**/
/**  es den Autostart-Mode des Moduls.                                    **/
/**                                                                       **/
/**  Installationshinweise:                                               **/
/**                                                                       **/
/**  Programmnummer  : 312h						  **/
/**  Tasknummer      : beliebig                                           **/
/**  Interruptquelle : 91h (Timer-A, Slave-Controller)                    **/
/**  Datenbereich    : per Makrobefehl einstellbar                        **/
/**  Flags           : 1982h (Task aktivieren, Auto-Init aufrufen)        **/
/**                                                                       **/
/**                                                                       **/
/**                       Borland C++ (Version 3.1)                       **/
/**                                                                       **/
/**                             Version 1.D.                              **/
/**                                                                       **/
/**                   Erstellungsdatum : 15.06.94                         **/
/**                    letzte nderung : 19.05.00 (HK)                    **/
/**                    ml8rt_end_of_int statt outportb...                 **/
/**                    Pointerzugriffe ber Bibliothek                    **/
/**                        PDT auf neustem Stand                          **/
/**                             Status : freigegeben                      **/
/**                              Autor : M.Heisig                         **/
/**                                                                       **/
/**          (c) 1994 - 1996 SORCUS Computer GmbH Heidelberg              **/
/**                                                                       **/
/***************************************************************************/

#pragma option -r-           /* Keine Registervariablen!! */

#include "ml8rtbib.h"        /* Echtzeitbibliothek einbinden */
/* #include "ml8makro.h" */        /* Makro-Definitionen einiger ML8RTBIB Routinen einbinden */
			     /* (Der Turbo-Assembler ist fr die Verwendung von "ml8makro.h" erforderlich!) */
#include "dos.h"             /* outport, outportb etc. als Makros */


			     /* ---- Allgemeine Angaben ---- */
#define PROGRAMM_NUMBER 0x312             /* Programmnummer = 312h */
#define VERSION         '1'               /* Version des Programms */
#define REVISION        'D'               /* Revision des Programms */

			     /* ---- Die verschiedenen Statusmglichkeiten des Programms ---- */
#define READY           0                 /* Bereit */
#define RUNNING         2                 /* Messung luft */
#define FINISHED        3                 /* Messung beendet */
#define STOPPED         4                 /* Programm wurde abgebrochen */
#define ERROR           0x80              /* Fehler aufgetreten */
#define DATA_ERROR      0x81              /* Fehler: Datenbereich zu klein */
#define MODUL_ERROR     0x82              /* Fehler: Falscher Modulsteckplatz angewhlt */
#define CHANNEL_ERROR   0x83              /* Fehler: Falsche Kanalzahl */

			     /* ---- Modulspezifische Informationen ---- */
#define KONFIG          0x72              /* Mode: M-AD12-16, Autoinkrement, 16-Bit Mode */
#define IOKONFIG        0x0B              /* Offset zur Modul-Basisadresse fr die Konfiguration */
#define IOSETTLE        0x07              /* Offset zur Modul-Basisadresse fr die Settle-Time */
#define IOGAIN          0x05              /* Offset zur Modul-Basisadresse fr die Gain-Korrektur */
#define IOOFFS          0x06              /* Offset zur Modul-Basisadresse fr die Offset-Korrektur */
#define IOREAD          0x08              /* Offset zur Modul-Basisadresse zum lesen des Ergebnisses und zur Einstellung des Modes */


			     /* ---- Aufbau der Programm-Descriptor-Tabelle (PDT) ---- */
struct pdt_type
{
   ML8_PDT_HEAD pdt_head;                    /* PDT-Header */

   void* PFAR   main_proc;                   /* Adresse der Haupt-Prozedur (Interrupt-Task) */
   void* PFAR   auto_init;                   /* Adresse der Auto-Init Prozedur */
   void* PFAR   start_proc;                  /* Adresse der Start-Prozedur */
   void* PFAR   stop_proc;                   /* Adresse der Stop-Prozedur */
} pdt;


			     /* ---- Aufbau des Parameterbereichs ---- */
struct parameter_type
{
   tdt_type     tdt;                         /* Platz fr die Task-Descriptor-Tabelle */
   byte         status;                      /* Status des Programms (READY etc.) */
   ushort       abtastrate;                  /* Abtastrate * 0.4 us = Zeit zwischen 2 Abtastwerten */
   byte         modul;                       /* Modulsteckplatz des M-AD16-3 */
   byte         anzkan;                      /* Anzahl der zu messenden Kanle */
   ushort       anzmess;                     /* Anzahl der Messungen */
   byte         settle;                      /* Wert fr Settle-Timer */
   byte         gain;                        /* Korrekturwert fr Gain */
   byte         offset;                      /* Korrekturwert fr Offset */
   ushort       bereich;                     /* Messbereich aller Kanle */
} parameter;


ushort          cntmess;          /* Zhler fr Messungen */
ushort          io_mode_data;     /* I/O-Adresse fr Messbereich, Endkanal und zum Daten lesen */

			     /* ---- Die verschiedenen Eingangsbereiche ---- */
			/* 0 = +/- 10 Volt
			   1 = +/- 5 Volt
			   2 = +/- 2,5 Volt
			   3 = +/- 1,25 Volt
			   4 = +/- 0,625 Volt
			   5 = +/- 312,5 mVolt
			   6 = 0 - 10 Volt
			   7 = 0 - 5 Volt
			   8 = 0 - 2,5 Volt
			   9 = 0 - 1,25 Volt
			   10 = 0 - 0,625 Volt
			   11 = -10 - 0 Volt
			   12 = -5 - 0 Volt
			   13 = -2,5 - 0 Volt
			   14 = -1,25 - 0 Volt
			   15 = -0,625 - 0 Volt */
ushort mode[16] = {0x0000, 0x0200, 0x1200, 0x2200, 0x3200, 0x4200,
			 0x0600, 0x1600, 0x2600, 0x3600, 0x4600,
			 0x0E00, 0x1E00, 0x2E00, 0x3E00, 0x4E00};



void PFAR auto_init(void);                     /* Prototypen der benutzten Prozeduren */
void PFAR main_task(void);
void PFAR start(void);
void PFAR stop(void);


/* Die Auto-Init-Prozedur wird direkt nach der Installierung des Programms
   einmal (!) aufgerufen. Es setzt hauptschlich die Parameter auf
   Default-Werte.
*/
void PFAR auto_init(void)             /* ---- AUTO-INITIALISIERUNG ---- */
{
   ml8rt_entry();                         /* Register retten und vorbereiten */
					  /* Defaultwerte eintragen */
   parameter.status = READY;              /* Status des Programms */
   parameter.abtastrate = 2500;           /* Abtastrate */
   parameter.modul = 1;                   /* Modulsteckplatz */
   parameter.anzkan = 1;                  /* Anzahl der Kanaele */
   parameter.anzmess = 1000;              /* Anzahl der Messpunkte */
   parameter.bereich = 0;                 /* Messbereich = 10V */
   parameter.gain = 0;                    /* Keine Korrektur */
   parameter.offset = 0;
   parameter.settle = 0x80;               /* Default-Settle-Time einstellen */

   ml8rt_exit();                          /* Register wieder restaurieren */
}


/* Diese Prozedur wertet die Parameter aus und startet die Messung.
*/
void PFAR start(void)
{
ulong           datalength;
ushort          mba;

   ml8rt_entry();

   /* ---------------- Parameterprfungen ----------------- */
   datalength = parameter.tdt.datalast - parameter.tdt.datafirst;
   if (datalength < (parameter.anzkan * 2 * parameter.anzmess))
      parameter.status = DATA_ERROR;      /* Datenbereich zu klein! */

   if ((parameter.modul<1) || (parameter.modul>4))
      parameter.status = MODUL_ERROR;     /* Falscher Modulsteckplatz angewhlt */

   if ((parameter.anzkan<1) || (parameter.anzkan>16))
      parameter.status = CHANNEL_ERROR;   /* Unzulssige Kanalzahl */

   if ((parameter.status & ERROR) == 0)
   {
      ml8rt_reset_w_pointer(parameter.tdt.task);  /* Zeiger auf den Datenbereichsanfang */

      mba = 0x100 * parameter.modul + 0x300;  /* Modulbasisadresse berechnen */
      io_mode_data = mba + IOREAD;        /* Adresse zum Bereich einstellen und zum lesen des Ergebnisses. */

      cntmess = 0;

      outportb(mba+IOKONFIG, KONFIG);     /* Modul-Konfiguration einstellen */
					  /* Settle-Time einstellen */
      outportb(mba+IOSETTLE, parameter.settle);
					  /* Gain-Korrektur einstellen */
      outportb(mba+IOGAIN, parameter.gain);
					  /* Offset-Korrektur einstellen */
      outportb(mba+IOOFFS, parameter.offset);
					  /* Den letzten zu messenden Kanal einstellen (es wird "Auto-Inkrement" verwendet!) */
      outport(io_mode_data, mode[parameter.bereich] + parameter.anzkan - 1);

      parameter.status = RUNNING;

      ml8rt_set_timer(TIMER_A, 0xFFFF, INT_MODE);
      ml8rt_clear_int (IRQ_TIMER_A);      /* Pending-Interrupts lschen */
      ml8rt_unmask_int(IRQ_TIMER_A);      /* Durch das Demaskieren des Timers wird die Messung gestartet. */

      cntmess = 0;
      ml8rt_set_timer(TIMER_A, parameter.abtastrate, INT_MODE);
      ml8rt_local_led_on();
   }

   ml8rt_exit();
}


/* Die Stop-Prozedur bricht eine gerade laufende Messung ab.
*/
void PFAR stop(void)
{
   ml8rt_entry();
   ml8rt_mask_int (IRQ_TIMER_A);          /* Timer-Interrupt sperren */
   parameter.status = STOPPED;            /* Status auf "abgebrochen" */
   ml8rt_exit();
}


/* Folgende Prozedur wird vom Betriebssystem mit jedem Interrupt vom Timer-A
   aufgerufen.
*/
void PFAR main_task(void)          /* ---- HAUPTPROZEDUR DER TASK ---- */
{
byte           kanal;
byte           b;
ushort         w;

   ml8rt_entry();

   kanal = 0;
   do {
      /* AD-Wert lesen und speichern. Gleichzeitig wird der nchste Kanal angewhlt! */
      ml8rt_write_data_word(parameter.tdt.task, inport (io_mode_data));
					  /* Der Datenpointer wird anschlieend um 1 (bzw. 2 Byte) erhht. */
      kanal++;
   } while (kanal<parameter.anzkan);      /* Solange messen, bis alle Kanle gesampled wurden */

   if (++cntmess > parameter.anzmess)     /* Wenn alle Messungen gemacht wurden, Programmlauf stoppen */
   {
      ml8rt_mask_int(IRQ_TIMER_A);
      parameter.status = FINISHED;
   }

   ml8rt_end_of_int(IRQ_TIMER_A);         /* EOI an den Interrupt-Controller senden */
   ml8rt_exit_interrupt();
}




void main (void)                 /* ---- PREPARE ---- */
{
   pdt.pdt_head.ucType          = 1;                    /* PDT-Typ einstellen (im Moment wird nur Typ 1 untersttzt) */
   pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD); /* Lnge des PDT-Vorspanns (im Moment immer = 48) */
   pdt.pdt_head.usGlobalProc    = 4;                    /* Anzahl der Prozeduren (in diesem Beispiel "Auto_Init", "Main_Task", "Start", "Stop" ) */
   pdt.pdt_head.usProgNo        = PROGRAMM_NUMBER;      /* Programm-Nummer */
   pdt.pdt_head.cVersion        = VERSION;              /* Programm-Version */
   pdt.pdt_head.cRevision       = REVISION;             /* Programm-Revision */
   pdt.pdt_head.ucCPU           = _486DX;               /* CPU-Type */
   pdt.pdt_head.ucCoProc        = _NOCOPROZ;            /* Co-Prozessor-Typ */
   pdt.pdt_head.ucLanguage      = _CPP31;               /* Programmiersprache = C++ */
   pdt.pdt_head.ucProgType      = _USER_PRG;            /* Programm-Typ: Anwenderprogramm */
   pdt.pdt_head.usFlags         = _DI_TASK              /* Flags */
	                               + _PDT_INFO_ENABLE + _LOCAL_PARAMETER;
   pdt.pdt_head.usInt           = IRQ_TIMER_A;          /* Interrupt-Nummer fr die das Programm gedacht ist */
   pdt.pdt_head.ulDataAdr       = 0;                    /* Adresse des Datenbereichs (hier wird kein Datenbereich verwendet) */
   pdt.pdt_head.ulDataSize      = 100000;               /* Datenbereichsgren */
   pdt.pdt_head.ulDataSizeMin   = 32;                   /* Minimale Datenbereichsgre */
   pdt.pdt_head.ulDataSizeMax   = 800000;               /* Maximale Datenbereichsgre */
					                                         /* Anfangsadresse des Parameterbereichs errechnen */
   ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
   pdt.pdt_head.ulGlobalParAdr  = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
					                                         /* Anzahl der Parameter eintragen */
   pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);
   pdt.pdt_head.ulHyperAdr      = 0;                    /* Adresse des Hypertextes (wird nicht verwendet) */
   pdt.main_proc                = main_task;            /* Adresse der Hauptprozedur */
   pdt.auto_init                = auto_init;            /* Adresse der Auto_Init Prozedur */
   pdt.start_proc               = start;                /* Adresse der Start Prozedur */
   pdt.stop_proc                = stop;                 /* Adresse der Stop Prozedur */
   ml8rt_set_pdt_adr (&pdt);                            /* Adresse der PDT im Rckgabe-Record dem Betriebssystem mitteilen */
}
