/***************************************************************************/
/**      Analogdaten-Erfassungsprogramm mit M-AD16-4 (MODULAR-4/486)      **/
/**                                                                       **/
/**  Dieses Programm erfasst mit einer einstellbaren Abtastrate einen     **/
/**  analogen Eingang mit dem Modul M-AD16-4. Die Parameter muessen       **/
/**  vor dem Start des Programms entsprechend eingestellt werden. Die     **/
/**  Groesse des Datenbereichs kann per Makrobefehl eingestellt werden.   **/
/**                                                                       **/
/**  Installationshinweise:                                               **/
/**                                                                       **/
/**  Programmnummer  : 322h = 802 					  **/
/**  Tasknummer      : beliebig                                           **/
/**  Interruptquelle : 91h (Timer-A, Slave-Controller)                    **/
/**  Datenbereich    : per Makrobefehl einstellbar                        **/
/**  Flags           : 1982h (Task aktivieren, Auto-Init aufrufen)        **/
/**                                                                       **/
/**                                                                       **/
/**                       Borland C++ (Version 3.1)                       **/
/**                                                                       **/
/**                             Version 1.C                               **/
/**                                                                       **/
/**                   Erstellungsdatum : 17.03.95                         **/
/**                    letzte nderung : 19.05.00  (HK)                   **/
/**                    ml8rt_end_of_int statt outportb...                 **/
/**                    Pointerzugriffe ber Bibliothek                    **/
/*+                          PDT auf neustem Stand                        **/
/**                             Status : freigegeben                      **/
/**                              Autor : Dipl. Ing. A. Horst              **/
/**                                                                       **/
/**          (c) 1995, 96 SORCUS Computer GmbH Heidelberg                 **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/** A/D-Wandler ADS7809:                                                  **/
/** Performance (MODULAR-4/486 DX2/66) :  100.00 kHz Abtastrate (1 Kanal) **/
/**                                       Settle Time = 0s               **/
/** Performance (MODULAR-4/486 DX4/120):  125.00 kHz Abtastrate (1 Kanal) **/
/**                                       Settle Time = 0s               **/
/**                                                                       **/
/** A/D-Wandler AD977:                                                    **/
/** Performance (MODULAR-4/486 DX2/66) :  150.00 kHz Abtastrate (1 Kanal) **/
/**                                       Settle Time = 0s               **/
/** Performance (MODULAR-4/486 DX4/120):  200.00 kHz Abtastrate (1 Kanal) **/
/**                                       Settle Time = 0s               **/
/***************************************************************************/

#include "ml8rtbib.h"

				   /* Makro-Definitionen einiger ML8RTBIB  */
				   /* Routinen einbinden                   */
#include "dos.h"                   /* outport, outportb etc. als Makros    */


				   /* Allgemeine Angaben                   */
#define PROGRAMM_NUMBER 0x322      /* Programmnummer = 322h                */
#define VERSION         '1'        /* Version des Programms                */
#define REVISION        'C'        /* Revision des Programms               */

				   /* Die verschiedenen Statusmoeglichk.   */
#define READY           0          /* Bereit                               */
#define RUNNING         2          /* Messung laeuft                       */
#define FINISHED        3          /* Messung beendet                      */
#define STOPPED         4          /* Programm wurde abgebrochen           */
#define ERROR           0x80       /* Fehler aufgetreten                   */
#define DATA_ERROR      0x81       /* Fehler: Datenbereich zu klein        */
#define MODUL_ERROR     0x82       /* Fehler: Falscher Modulsteckplatz     */
#define CHANNEL_ERROR   0x83       /* Fehler: Falsche Kanalzahl            */


				   /* Aufbau der Programm-Descr.-Tab.(PDT) */
struct pdt_type
{
 ML8_PDT_HEAD pdt_head;      /* PDT-Header                           */

 void* PFAR main_proc;       /* Adresse der Haupt-Prozedur (DI-Task) */
 void* PFAR auto_init;       /* Adresse der Auto-Init Prozedur       */
 void* PFAR start_proc;      /* Adresse der Start-Prozedur           */
 void* PFAR stop_proc;       /* Adresse der Stop-Prozedur            */
} pdt;


struct parameter_type              /* Aufbau des Parameterbereichs         */
{
 tdt_type      tdt;                /* Platz fuer die Task-Descriptor-Tab.  */
 byte 	       status;             /* Status des Programms (READY etc.)    */
 ulong         abtastrate;         /* Abtastrate                           */
 byte          modul;              /* Modulsteckplatz des M-AD16-4         */
 byte          kanal;              /* der zu messende Kanal                */
 ushort        anzmess;            /* Anzahl der Messungen                 */
 ushort        settle;             /* Settle Timer Wert (hier = 0)         */
} parameter;

				   /* globale Variablen                    */
ushort      cntmess;         /* Zaehler fuer Messungen               */
ushort      mba;             /* Modulbasisadresse                    */

void PFAR auto_init(void);              /* Prototypen der benutzten Prozeduren  */
void PFAR main_task(void);
void PFAR start(void);
void PFAR stop(void);


void PFAR auto_init(void)               /* AUTO-INITIALISIERUNG                 */
{
 ml8rt_entry();                    /* Register retten und vorbereiten      */

 ml8rt_local_led_off();            /* On-Board LED ausschalten             */
 parameter.status = READY;         /* Status des Programms auf "BEREIT"    */
 parameter.abtastrate =  2500L;    /* 2500 Hz                              */
 parameter.modul      =      1;    /* Modulsteckplatz = 1                  */
 parameter.kanal      =      0;    /* Kanalnummer                          */
 parameter.anzmess    =   1000;    /* Tausend Messpunkte                   */
 parameter.settle     = 0x0000;    /* Settle-Timer = 0 (gleicher Kanal)    */

 ml8rt_exit();                     /* Register wieder restaurieren         */
}


void PFAR start(void)
{
 ulong datalength;
 ushort timer;

 ml8rt_entry();

				   /* Modulbasisadresse berechnen          */
 if(parameter.modul > 4)
   mba = 0x020 * (parameter.modul - 4) + 0x400;
 else
   mba = 0x100 * (parameter.modul - 1) + 0x400;

 outportb(mba + 0x1C, 0x11);       /* Messmode einstellen (TA deaktiv.)    */

				   /* Kanal einstellen                     */
 outportb(mba + 0x10 + parameter.kanal, 0x00);

				   /* Settle Timer einstellen              */
 outport(mba + 0x1A, parameter.settle);

 datalength = parameter.tdt.datalast - parameter.tdt.datafirst;
 if(datalength < (2 * parameter.anzmess))
   parameter.status = DATA_ERROR;  /* Datenbereich zu klein!               */

 if((parameter.modul < 1) || (parameter.modul > 10))
   parameter.status = MODUL_ERROR; /* Falscher Modulsteckplatz angewaehlt  */

 if((parameter.status & ERROR) == 0)
   {
    ml8rt_entry();
    ml8rt_local_led_on();          /* Eingeschaltete LED zeigt an,         */
				   /* dass die Messung laeuft.             */
				   /* Zeiger auf den Datenbereichsanfang   */
    ml8rt_reset_w_pointer(parameter.tdt.task);

    cntmess = 0;                   /* Zaehler fuer Messwerte               */

    timer = ml8rt_convert_timer_data((unsigned short)(1000000L / parameter.abtastrate));
    ml8rt_set_timer(TIMER_A, timer, INT_MODE);

    parameter.status = RUNNING;    /* Timer-A initialisieren               */
    outportb(mba + 0x1C, 0x17);    /* Messmode einstellen (TA aktiv.)      */
				   /* 2er Komplement Format wg. bipolar !  */

				   /* Durch das Demaskieren des Timers     */
				   /* wird die Messung gestartet.          */
    ml8rt_unmask_int(IRQ_TIMER_A);
   }

 ml8rt_exit();
}


void PFAR stop(void)
{
 ml8rt_entry();
 ml8rt_local_led_off();            /* LED ausschalten. (Messung beendet)   */
 ml8rt_mask_int(IRQ_TIMER_A);      /* Timer-Interrupt sperren              */
 parameter.status = STOPPED;       /* Status auf "abgebrochen"             */
 ml8rt_exit();
}


void PFAR main_task(void)               /* HAUPTPROZEDUR DER TASK               */
{
 ml8rt_entry();

				   /* Status Wandlung lesen                */
 while((inportb(mba + 0x08) & 0x80) != 0x80);

 /* Ergebnis lesen und abspeichern       */
 ml8rt_write_data_word(parameter.tdt.task,inport(mba + 0x02));

				   /* EOI an den Slave-Controller senden   */
				   /* (weil Timer-A am Slave-Controller    */
				   /* angeschlossen ist                    */

 if(cntmess++ > parameter.anzmess) /* Wenn alle Messungen gemacht wurden : */
   {
    ml8rt_mask_int(IRQ_TIMER_A);
    ml8rt_local_led_off();         /* LED ausschalten, Messung beendet     */
    outportb(mba + 0x1D, 0x00);    /* Modul Reset                          */
    parameter.status = FINISHED;
   }

 ml8rt_end_of_int(IRQ_TIMER_A);    /* EOI an den Interrupt -Controller senden  */
 ml8rt_exit_interrupt();
}




void main(void)                    /* PREPARE                              */
{
 pdt.pdt_head.ucType          = 1;                    /* PDT-Typ einstellen (im Moment wird   */
				                                          /* nur Typ 1 unterstuetzt)              */
 pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD); /* Laenge des PDT-Vorspanns (im Moment  */
				                                          /* immer = 48)                          */
 pdt.pdt_head.usGlobalProc    = 4;                    /* Anzahl der Prozeduren (in diesem     */
				                                          /* Beispiel :                           */
				                                          /*            "Auto_Init", "Main_Task", */
				                                          /*            "Start", "Stop" )         */
				                                          /* Programm-Nummer                      */
 pdt.pdt_head.usProgNo        = PROGRAMM_NUMBER;
 pdt.pdt_head.cVersion        = VERSION;              /* Programm-Version                     */
 pdt.pdt_head.cRevision       = REVISION;             /* Programm-Revision                    */
 pdt.pdt_head.ucCPU           = _486DX;               /* CPU-Type                             */
 pdt.pdt_head.ucCoProc        = _487DX;               /* Co-Prozessor-Typ                     */
 pdt.pdt_head.ucLanguage      = _CPP31;               /* Programmiersprache = C++             */
 pdt.pdt_head.ucProgType      = _USER_PRG;            /* Programm-Typ: Anwenderprogramm       */
 pdt.pdt_head.usFlags         = _DI_TASK +            /* Flags                                */
		                          _PDT_INFO_ENABLE + _LOCAL_PARAMETER;
 pdt.pdt_head.usInt= IRQ_TIMER_A;                     /* Interrupt-Nummer fuer die das        */
				                                          /* Programm gedacht ist                 */
 pdt.pdt_head.ulDataAdr       = 0;                    /* Adresse des Datenbereichs            */
 pdt.pdt_head.ulDataSize      = 100000;               /* Datenbereichsgroessen                */
 pdt.pdt_head.ulDataSizeMin   = 32;                   /* Minimale Datenbereichsgroesse        */
 pdt.pdt_head.ulDataSizeMax   = 800000;               /* Maximale Datenbereichsgroesse        */
				                                          /* Anfangsadresse des Parameterbereichs */
 ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
 pdt.pdt_head.ulGlobalParAdr  = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
				                                          /* Anzahl der Parameter eintragen       */
 pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);
 pdt.pdt_head.ulHyperAdr      = 0;                    /* Adresse des Hypertextes (wird nicht  */
				                                          /* verwendet)                           */
 pdt.main_proc                = main_task;            /* Adresse der Hauptprozedur            */
 pdt.auto_init                = auto_init;            /* Adresse der Auto_Init Prozedur       */
 pdt.start_proc               = start;                /* Adresse der Start Prozedur           */
 pdt.stop_proc                = stop;                 /* Adresse der Stop Prozedur            */
 ml8rt_set_pdt_adr (&pdt);                            /* Adresse der PDT im Rueckgabe-Record  */
}
