/***************************************************************************/
/**      Analogdaten-Erfassungsprogramm mit M-AD16-4 (MODULAR-4/486)      **/
/**                                                                       **/
/**  Dieses Programm erfasst mit einer einstellbaren Abtastrate alle acht **/
/**  analogen Eingaenge des Moduls M-AD16-4. Die Parameter muessen        **/
/**  vor dem Start des Programms entsprechend eingestellt werden. Die     **/
/**  Groesse des Datenbereichs kann per Makrobefehl eingestellt werden.   **/
/**                                                                       **/
/**  Installationshinweise:                                               **/
/**                                                                       **/
/**  Programmnummer  : 323h						  **/
/**  Tasknummer      : beliebig                                           **/
/**  Interruptquelle : 91h (Timer-A, Slave-Controller)                    **/
/**  Datenbereich    : per Makrobefehl einstellbar                        **/
/**  Flags           : 1982h (Task aktivieren, Auto-Init aufrufen)        **/
/**                                                                       **/
/**                                                                       **/
/**                       Borland C++ (Version 3.1)                       **/
/**                                                                       **/
/**                             Version 1.C                               **/
/**                                                                       **/
/**                   Erstellungsdatum : 17.03.95                         **/
/**                    letzte nderung : 19.05.00 (HK)                    **/
/**                    ml8rt_end_of_int statt outportb...                 **/
/**                    Pointerzugriffe ber Bibliothek                    **/
/**                         PDT auf neustem Stand                         **/
/**                             Status : freigegeben                      **/
/**                              Autor : Dipl. Ing. A. Horst	           **/
/**                                                                       **/
/**          (c) 1995, 96 SORCUS Computer GmbH Heidelberg                 **/
/**                                                                       **/
/** A/D-Wandler ADS7809 bzw. AD977:                                       **/
/** Performance (MODULAR-4/486 DX4/120):  52.00 kHz Summenabtastrate      **/
/**                                       13.00 kHz je Kanal (4 Kanaele)  **/
/**                                       Settle Time = 18s              **/
/***************************************************************************/

#include "ml8rtbib.h"

				   /* Makro-Definitionen einiger ML8RTBIB  */
				   /* Routinen einbinden                   */
#include "dos.h"                   /* outport, outportb etc. als Makros    */


				   /* Allgemeine Angaben                   */
#define PROGRAMM_NUMBER 0x323      /* Programmnummer = 323h                */
#define VERSION         '1'        /* Version des Programms                */
#define REVISION        'C'        /* Revision des Programms               */

				   /* Die verschiedenen Statusmoeglichk.   */
#define READY           0          /* Bereit                               */
#define RUNNING         2          /* Messung laeuft                       */
#define FINISHED        3          /* Messung beendet                      */
#define STOPPED         4          /* Programm wurde abgebrochen           */
#define ERROR           0x80       /* Fehler aufgetreten                   */
#define DATA_ERROR      0x81       /* Fehler: Datenbereich zu klein        */
#define MODUL_ERROR     0x82       /* Fehler: Falscher Modulsteckplatz     */
#define CHANNEL_ERROR   0x83       /* Fehler: Falsche Kanalzahl            */


				   /* Aufbau der Programm-Descr.-Tab.(PDT) */
struct pdt_type
{
 ML8_PDT_HEAD pdt_head;      /* PDT-Header                           */

 void* PFAR main_proc;       /* Adresse der Haupt-Prozedur (DI-Task) */
 void* PFAR auto_init;       /* Adresse der Auto-Init Prozedur       */
 void* PFAR start_proc;      /* Adresse der Start-Prozedur           */
 void* PFAR stop_proc;       /* Adresse der Stop-Prozedur            */
} pdt;


struct parameter_type              /* Aufbau des Parameterbereichs         */
{
 tdt_type      tdt;                /* Platz fuer die Task-Descriptor-Tab.  */
 byte 	       status;             /* Status des Programms (READY etc.)    */
 ushort        abtastrate;         /* Abtastrate                           */
 byte 	       modul;              /* Modulsteckplatz des M-AD16-4         */
 byte 	       kanal;              /* der 1. zu messende Kanal             */
 byte          anz_kan;            /* Anzahl zu messender Kanaele          */
 ushort        anzmess;            /* Anzahl der Messungen je Kanal        */
 ushort        settle;             /* Settle Timer Wert (hier = 0x00b0)    */
} parameter;

				   /* globale Variablen                    */
ushort      mba;             /* Modulbasisadresse                    */
byte        akt_kanal;       /* zu messender Kanal                   */
byte        startkanal;      /* 1. zu messender Kanal                */
byte        endkanal;        /* letzter zu messender Kanal           */
ushort      gesamtanz;       /* Anzahl Messungen (Gesamt)            */

ushort      dummy;

void PFAR auto_init(void);              /* Prototypen der benutzten Prozeduren  */
void PFAR main_task(void);
void PFAR start(void);
void PFAR stop(void);


void PFAR auto_init(void)               /* AUTO-INITIALISIERUNG                 */
{
 ml8rt_entry();                    /* Register retten und vorbereiten      */

 ml8rt_local_led_off();            /* On-Board LED ausschalten             */
 parameter.status = READY;         /* Status des Programms auf "BEREIT"    */
 parameter.abtastrate =   2500;    /* 2500 * 1/Timertakt (hier 10MHz)      */
 parameter.modul      =      3;    /* Modulsteckplatz = 3                  */
 parameter.kanal      =      0;    /* Start-Kanalnummer                    */
 parameter.anz_kan    =      8;    /* Anzahl Kanaele                       */
 parameter.anzmess    =   1000;    /* Tausend Messpunkte                   */
 parameter.settle     = 0x00B0;    /* Settle-Timer = 18s (Timer 10 MHz !) */

 ml8rt_exit();                     /* Register wieder restaurieren         */
}


void PFAR start(void)
{
 ulong datalength;
 ushort timer;

 ml8rt_entry();

 gesamtanz  = parameter.anzmess;
 startkanal = parameter.kanal;     /* Startkanal                           */
 endkanal   = parameter.kanal + parameter.anz_kan;
 akt_kanal  = startkanal;          /* aktueller Kanal                      */
				   /* Modulbasisadresse berechnen          */
 mba = 0x100 * parameter.modul + 0x300;

				   /* Kanal einstellen                     */
 outportb(mba + 0x10 + startkanal, 0x00);
 outportb(mba + 0x1C, 0x13);       /* Messmode einstellen (TA deaktiv.)    */
				   /* 2er Komplement Format wg. bipolar !  */
				   /* Settle Timer einstellen              */
 outport(mba + 0x1A, parameter.settle);

 datalength = parameter.tdt.datalast - parameter.tdt.datafirst;
 if(datalength < (2 * gesamtanz))
   parameter.status = DATA_ERROR;  /* Datenbereich zu klein!               */

 if((parameter.modul < 1) || (parameter.modul > 4))
   parameter.status = MODUL_ERROR; /* Falscher Modulsteckplatz angewaehlt  */

 if((parameter.status & ERROR) == 0)
   {
    ml8rt_entry();
    ml8rt_local_led_on();          /* Eingeschaltete LED zeigt an,         */
				   /* dass die Messung laeuft.             */
				   /* Zeiger auf den Datenbereichsanfang   */
    ml8rt_reset_w_pointer(parameter.tdt.task);

    timer = ml8rt_convert_timer_data((unsigned short)(1000000L / parameter.abtastrate));
    ml8rt_set_timer(TIMER_A, timer, INT_MODE);

    parameter.status = RUNNING;    /* Timer-A initialisieren               */
    outportb(mba + 0x1C, 0x33);    /* Messmode einstellen: TA inaktiv      */
				   /* 2er Komplement Format wg. bipolar !  */

				   /* Durch das Demaskieren des Timers     */
				   /* wird die Messung gestartet.          */
    ml8rt_unmask_int(IRQ_TIMER_A);
    ml8rt_exit();
   }

 ml8rt_exit();
}


void PFAR stop(void)
{
 ml8rt_entry();
 ml8rt_local_led_off();            /* LED ausschalten. (Messung beendet)   */
 ml8rt_mask_int (IRQ_TIMER_A);     /* Timer-Interrupt sperren              */
 parameter.status = STOPPED;       /* Status auf "abgebrochen"             */
 ml8rt_exit();
}


void PFAR main_task(void)               /* HAUPTPROZEDUR DER TASK               */
{				   /* Int.-Service durch Timer-A gestartet */
 byte i;

 ml8rt_entry();

 for(i = startkanal; i < endkanal; i++)
   {

				   /* Status Settle-Timer lesen            */
    while((inportb(mba + 0x08) & 0x40) != 0x40);

    outportb(mba + 0x01, 0);       /* Wandlung starten  (Kanal i-1)        */
    outportb(mba + 0x08, i + 1);   /* Settle-Timer starten (Kanal i)       */

				   /* Status Wandlung lesen                */
    while((inportb(mba + 0x08) & 0x80) != 0x80);
				   /* Ergebnis Kanal i einlesen            */
    ml8rt_write_data_word(parameter.tdt.task,inport(mba + 0x02));
				   /* naechster Kanal...                   */
   }

 outportb(mba + 0x08, startkanal);

 if(!gesamtanz--)                  /* Wenn alle Messungen gemacht wurden : */
   {
    ml8rt_mask_int(IRQ_TIMER_A);
    ml8rt_local_led_off();         /* LED ausschalten, Messung beendet     */
    outportb(mba + 0x1D, 0x00);    /* Modul Reset                          */
    parameter.status = FINISHED;
   }

 ml8rt_end_of_int(IRQ_TIMER_A);    /* EOI an den Interrupt -Controller senden  */

 ml8rt_exit_interrupt();
}




void main(void)                    /* PREPARE                              */
{
 pdt.pdt_head.ucType          = 1;                    /* PDT-Typ einstellen (im Moment wird   */
				                                          /* nur Typ 1 unterstuetzt)              */
 pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD); /* Laenge des PDT-Vorspanns (im Moment  */
				                                          /* immer = 48)                          */
 pdt.pdt_head.usGlobalProc   = 4;                     /* Anzahl der Prozeduren (in diesem     */
				                                          /* Beispiel :                           */
				                                          /*            "Auto_Init", "Main_Task", */
				                                          /*            "Start", "Stop" )         */
 pdt.pdt_head.usProgNo        = PROGRAMM_NUMBER;		/* Programm-Nummer                      */
 pdt.pdt_head.cVersion        = VERSION;              /* Programm-Version                     */
 pdt.pdt_head.cRevision       = REVISION;             /* Programm-Revision                    */
 pdt.pdt_head.ucCPU           = _486DX;               /* CPU-Type                             */
 pdt.pdt_head.ucCoProc        = _487DX;               /* Co-Prozessor-Typ                     */
 pdt.pdt_head.ucLanguage      = _CPP31;               /* Programmiersprache = C++             */
 pdt.pdt_head.ucProgType      = _USER_PRG;            /* Programm-Typ: Anwenderprogramm       */
 pdt.pdt_head.usFlags         = _DI_TASK +            /* Flags                                */
		                          _PDT_INFO_ENABLE + _LOCAL_PARAMETER;

 pdt.pdt_head.usInt= IRQ_TIMER_A;                     /* Interrupt-Nummer fuer die das        */
				                                          /* Programm gedacht ist                 */
 pdt.pdt_head.ulDataAdr       = 0;                    /* Adresse des Datenbereichs (hier wird */
				                                          /* kein Datenbereich verwendet)         */
 pdt.pdt_head.ulDataSize      = 100000L;              /* Datenbereichsgroesse                 */
 pdt.pdt_head.ulDataSizeMin   = 32;                   /* Minimale Datenbereichsgroesse        */
 pdt.pdt_head.ulDataSizeMax   = 800000L;              /* Maximale Datenbereichsgroesse        */
				                                          /* Anfangsadresse des Parameterbereichs */
 ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
 pdt.pdt_head.ulGlobalParAdr  = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
				                                          /* Anzahl der Parameter eintragen       */
 pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);
 pdt.pdt_head.ulHyperAdr      = 0;                    /* Adresse des Hypertextes (wird nicht  */
				                                          /* verwendet)                           */
 pdt.main_proc                = main_task;            /* Adresse der Hauptprozedur            */
 pdt.auto_init                = auto_init;            /* Adresse der Auto_Init Prozedur       */
 pdt.start_proc               = start;                /* Adresse der Start Prozedur           */
 pdt.stop_proc                = stop;                 /* Adresse der Stop Prozedur            */
 ml8rt_set_pdt_adr (&pdt);                            /* Adresse der PDT im Rueckgabe-Record  */
				                                          /* dem Betriebssystem mitteilen         */
}
