/****************************************************************************
*                             M8P0327.H
*      Header filer to M8P0327.C, that demonstrates continuous
*      measurement and transfer of analog data on a MODULAR-4/486 board
*
*              version                       : 1.D.
*              creation                      : 27.03.1996
*              last change                   : 07.06.2000
*              author                        : H. Klinkenborg
*                                              A. Horst
*
*    (c) by SORCUS Computer GmbH Heidelberg 1996
****************************************************************************/

/****************************************************************************
*  Description: This program measures continuously analog input from up to
*        16 channels of the module M-AD12-16 and puts the data into a buffer
*        of the operating system OsX of the MODULAR-4/486 board.
*
*  Compiler: Borland C 3.1
*
*  hardware requirements: PC with monitor, MODULAR-4/486 board and SPB
*        module M-AD12-16
*
*  software requirements: This program requires the library ML8RTBIB.OBJ
*        and the header ML8RTBIB.H and the header M039_LIB.H. The operating
*        system OSX on the MODULAR-4/486 board must be ML8-3A.00x or greater
****************************************************************************/

/****************************************************************************
*  history:
*        1.A. 27.03.1996 10:40; HK
*        - creation of the file
*        1.B. 19.12.1996 15:30; AH
*        - only one task, sends buffered SRQ
*        1.C. 18.11.1998 15:30; HK (use ML8RTBIB.LIB)
*        1.D. 07.06.2000 11:17; HK (newest PDT-configuration)
****************************************************************************/

#ifndef byte
	#define byte        unsigned char
#endif
#ifndef ushort
	#define ushort      unsigned short
#endif
#ifndef ulong
	#define ulong       unsigned long
#endif

#define TRUE           1
#define FALSE          0

#define INSTALLED      0
#define PREPARED       1
#define RUNNING	       2
#define FINISHED       3
#define CANCELLED      4



struct pdt_type                        /* Program-Descriptor-Tab. (PDT)    */
    {
     ML8_PDT_HEAD pdt_head;         /* PDT-Header                       */

     void* PFAR main_proc;          /* address of main-proc. (Task)     */
     void* PFAR auto_init;          /* address of auto-init-proced.     */
     void* PFAR start;              /* address of start-procedure       */
     void* PFAR stop;               /* address of stop-procedure        */
     void* PFAR readbuf;            /* address of special function      */
     void* PFAR clearflag;          /* address of special procedure     */
    } pdt;

typedef struct
    {                                  /* parameters                       */
#ifdef RT
     tdt_type tdt;                     /* reservation for TDT              */
#endif				       /* used with realtime program       */           */

     byte     status;                  /* status of program:		   */
				       /* = 0: installed 		   */
				       /* = 1: prepared  		   */
				       /* = 2: running   		   */
				       /* = 3: finished  		   */
				       /* = 4: cancelled 		   */
     byte     trans_status;            /* bit 0 = 1: waiting for PC        */
				       /*     0 = 0: waiting for SRQ       */

     byte     micro_slot;              /* no. of slot for M-AD12-16        */
     byte     no_of_channels;          /* number of A/D-channels           */
     byte     settletime;              /* settletime in multip. of 62,5ns  */

     ulong    timer_tic;               /* timer tic for TI-tasks           */
     ushort   rate;                    /* sampling rate in multiples of    */
				       /* timer tics                       */
     ulong    count;                   /* number of cycles                 */
     byte     mode;                    /* = 1: duration = count            */
				       /* = 0: duration = infinite         */

     ushort   error;                   /* error code returned by buffer    */
				       /* functions                        */
     ulong    bufnr;                   /* number of buffer (= address)     */
     ulong    buffer_size;             /* size of system buffer            */
     ushort   srq_limit;               /* number of bytes until SRQ -> PC  */
     ushort   srq_word;                /* word to be sent to PC            */
     ushort   trans_size;              /* number of bytes to be transfer.  */
				       /* to the buffer                    */
    } par_type_M8P0327;

