/********************************************************************
*  This header file is included by the header files ML8RTBIB.H
*  and ML6RTBIB.H and ML7RTBIB.H.
*
*  Its purpose is to defines all constants for the libraries
*  ML8RTBIB and ML6RTBIB and ML7RTBIB.H.
*
*  Author         HK
*
*  Last change    22/09/98 ML7
*
*  Author         Dipl.-Ing. H.Doersam
*
*  Last change    09/04/98
*
*  Copyrights by SORCUS Computer GmbH 1998
*  All rights reserved.
********************************************************************/



#ifndef _INC_MLXRTDEF
#define _INC_MLXRTDEF



/********************************************************************
*  Version types
********************************************************************/

#define FREE_RELEASE                      'F'
#define BETA_VERSION                      'B'
#define EPROM_OSX                         'E'
#define RAM_OSX                           'R'



/********************************************************************
* Local I/O Addresses
********************************************************************/

#define PC_STAT                           0x28
#define PC_TRANS                          0x28
#define PC_REC                            0x26

#define SCC_CONTROL_A                     0x32
#define SCC_CONTROL_B                     0x30

#define EXTERNAL_LED_ON                   0x50
#define EXTERNAL_LED_OFF                  0x51
#define LOCAL_LED_ON                      0x5C
#define LOCAL_LED_OFF                     0x5D

#define MASTER_INT_CONTR                  0x00
#define SLAVE_INT_CONTR                   0x08

#define REAL_TIME_CLK                     0x70

#define TIMER_A_PORT                      0x10
#define TIMER_B_PORT                      0x11
#define TIMER_C_PORT                      0x12
#define TIMER_CTR                         0x13

#define IRQ_F_POS_EDGE                    0x4C
#define IRQ_F_NEG_EDGE                    0x4D
#define IRQ_E_POS_EDGE                    0x4E
#define IRQ_E_NEG_EDGE                    0x4F
#define IRQ_A_POS_EDGE                    0x68
#define IRQ_A_NEG_EDGE                    0x69
#define IRQ_B_POS_EDGE                    0x6A
#define IRQ_B_NEG_EDGE                    0x6B
#define IRQ_C_POS_EDGE                    0x6C
#define IRQ_C_NEG_EDGE                    0x6D
#define IRQ_D_POS_EDGE                    0x6E
#define IRQ_D_NEG_EDGE                    0x6F



/********************************************************************
*  Hardware Interrupts
********************************************************************/

#define MASTER_VEC                        0x78
#define SLAVE_VEC                         0x90

#define NMI                               0x02
#define IRQ_RBF                           0x78
#define IRQ_F                             0x79
#define IRQ_SLAVE                         0x7A
#define IRQ_A                             0x7B
#define IRQ_B                             0x7C
#define IRQ_C                             0x7D
#define IRQ_D                             0x7E
#define IRQ_E                             0x7F
#define IRQ_SCC                           0x90
#define IRQ_TIMER_A                       0x91
#define IRQ_TIMER_B                       0x92
#define IRQ_TIMER_C                       0x93
#define IRQ_RTC                           0x94
#define IRQ_EXT                           0x95
#define IRQ_COM_B                         0x96
#define IRQ_TBE                           0x97

// Edge of interrupt
#define POS_EDGE                          0
#define NEG_EDGE                          1


#ifdef __ML7__

// ML7 specific interrupts

#define IRQ_TEMP                          0x79
#define IRQ_FAN                           0x7F

#endif // __ML7__

#ifdef __ML6__

// ML6 specific interrupts

#define IRQ_TEMP                          0x79
#define IRQ_SCC3                          0x7C
#define IRQ_SCC2                          0x7D
#define IRQ_SCC1                          0x7E
#define IRQ_FAN                           0x7F

#endif // __ML6__



/********************************************************************
*  Modes for the Real-Time-Clock
********************************************************************/

#define RTC_1_64_SECOND                   0x44
#define RTC_1_SECOND                      0x54
#define RTC_1_MINUTE                      0x64
#define RTC_1_HOUR                        0x74



/********************************************************************
*  Timer
********************************************************************/

//	Timer selection
#define TIMER_A                           0
#define TIMER_B                           1
#define TIMER_C                           2

// Timer modes
#define INT_MODE                          2



/********************************************************************
*  Modes for mlxrt_get_task_number (nur ml8 ?)
********************************************************************/

#define HIGHEST                           0
#define LOWEST                            1



/********************************************************************
*  Control bytes for mlxrt_cache_control   (nur ml8 ?)
********************************************************************/

#define CACHE_OFF                         0
#define CACHE_ON                          1
#define CACHE_INVALID_ON                  3



/********************************************************************
*  Constants for the PDT
********************************************************************/

//	Length of the PDT header
#define _PDT_LENGTH                       48

//	Processor types
#define _86                               0
#define _186                              1
#define _286                              2
#define _486SX                            3
#define _486DX                            4
#define _586                              5

//	Coprocessor types for floating-point operations
#define _NOCOPROC                         0
#define _NOCOPROZ                         0
#define _87                               1
#define _287                              2
#define _487SX                            3
#define _487DX                            4
#define _587                              5
#define _EMULATION                        6

//	Used programming language
#define _ASM                              1
#define _TP50                             2
#define _TP55                             3
#define _TP60                             4
#define _CPP30                            5
#define _CPP31                            6
#define _TP70                             7
#define _CPP40                            8
#define _CPP45                            9
#define _CPP50                            10

//	Program type
#define _SYSTEM_PRG                       0
#define _USER_PRG                         1
#define _SOFT_INTERRUPT                   2
#define _HARD_INTERRUPT                   3
#define _TASK_MANAGER                     4
#define _ERROR_TRAPS                      5

//	Flags
#define _NI_TASK                          0x0000
#define _II_TASK                          0x0001
#define _DI_TASK                          0x0002
#define _TI_TASK                          0x0003
#define _PDT_INFO_ENABLE                  0x0008
#define _PROTECTED_MD                     0x0010
#define _NO_CACHE_USE                     0x0020
#define _HYPER_TEXT                       0x0080
#define _LOCAL_DATA                       0x0100
#define _FIXED_DATASIZE                   0x0200
#define _LOCAL_PARAMETER                  0x0400



/********************************************************************
*  SCC (only ML7 and ML8)
********************************************************************/

#ifdef __ML7__

// SCC channels
#define SCC_CHANNEL_A                     0
#define SCC_CHANNEL_B                     1

//	Stop-Bits
#define SCC_NO_STOPBIT                    0
#define SCC_1_STOPBIT                     1
#define SCC_1_5_STOPBIT                   2
#define SCC_2_STOPBIT                     3

//	Parity
#define SCC_NO_PARITY                     0
#define SCC_PARITY                        1
#define SCC_ODD_PARITY                    'O'
#define SCC_EVEN_PARITY                   'E'

//	SCC quartz
#define SCC_CLOCK_4_9                     0
#define SCC_CLOCK_7_3                     1

//	Errors of ml7rt_init_scc
#define CHANNEL_ERR                       0x01
#define PARITY_ERR                        0x02
#define STOPBIT_ERR                       0x03
#define DATABIT_ERR                       0x04
#define BAUDRATE_ERR                      0x05

//	Baudrate
#define SCC_300_BAUD                      300L
#define SCC_600_BAUD                      600L
#define SCC_1200_BAUD                     1200L
#define SCC_2400_BAUD                     2400L
#define SCC_4800_BAUD                     4800L
#define SCC_9600_BAUD                     9600L
#define SCC_19200_BAUD                    19200L
#define SCC_38400_BAUD                    38400L
#define SCC_115200_BAUD                   115200L

#endif // __ML7__

#ifdef __ML8__

// SCC channels
#define SCC_CHANNEL_A                     0
#define SCC_CHANNEL_B                     1

//	Stop-Bits
#define SCC_NO_STOPBIT                    0
#define SCC_1_STOPBIT                     1
#define SCC_1_5_STOPBIT                   2
#define SCC_2_STOPBIT                     3

//	Parity
#define SCC_NO_PARITY                     0
#define SCC_PARITY                        1
#define SCC_ODD_PARITY                    'O'
#define SCC_EVEN_PARITY                   'E'

//	SCC quartz
#define SCC_CLOCK_4_9                     0
#define SCC_CLOCK_7_3                     1

//	Errors of ml8rt_init_scc
#define CHANNEL_ERR                       0x01
#define PARITY_ERR                        0x02
#define STOPBIT_ERR                       0x03
#define DATABIT_ERR                       0x04
#define BAUDRATE_ERR                      0x05

//	Baudrate
#define SCC_300_BAUD                      300L
#define SCC_600_BAUD                      600L
#define SCC_1200_BAUD                     1200L
#define SCC_2400_BAUD                     2400L
#define SCC_4800_BAUD                     4800L
#define SCC_9600_BAUD                     9600L
#define SCC_19200_BAUD                    19200L
#define SCC_38400_BAUD                    38400L
#define SCC_115200_BAUD                   115200L

#endif // __ML8__


/********************************************************************
*  Constants for memory/buffer allocation
********************************************************************/

// Strategy
#define ALLOC_UP_ABS                      1
#define ALLOC_UP_MAX                      2
#define ALLOC_DOWN_ABS                    3
#define ALLOC_DOWN_MAX                    4

//	Alignment
#define BYTE_ALIGN                        0
#define WORD_ALIGN                        1
#define DWORD_ALIGN                       2
#define QWORD_ALIGN                       3



/********************************************************************
*  Constants for the priority of NI-tasks
********************************************************************/

#define LOW_PRIORITY_NI_TASK              1
#define HIGH_PRIORITY_NI_TASK             2
#define HIGHEST_PRIORITY_NI_TASK          3



/********************************************************************
*  Exceptions flags for the FPU
********************************************************************/

#define _FPU_NO_INTERRUPT                 0x0000
#define _FPU_INVALID_COMMAND              0x0001
#define _FPU_DENORMAL_OPERATOR            0x0002
#define _FPU_DIVISION_BY_ZERO             0x0004
#define _FPU_OVERFLOW                     0x0008
#define _FPU_UNDERFLOW                    0x0010
#define _FPU_ACCURACY_LOST                0x0020



/********************************************************************
*  String conventions
********************************************************************/

#define C_STRING                          0
#define PAS_STRING                        1



/********************************************************************
*  Enable/Disable error handling
********************************************************************/

#define DISABLE                           0
#define ENABLE                            1



/********************************************************************
*  Day of week
********************************************************************/

#define SUNDAY                            0
#define MONDAY                            1
#define TUESDAY                           2
#define WEDNESDAY                         3
#define THURSDAY                          4
#define FRIDAY                            5
#define SATURDAY                          6


/********************************************************************
*  Flash programming
********************************************************************/

// Info types for mlxrt_get_flash_info (nur ml8 ?)

#define FLASH_GET_CUSTOMER_ID         	   0x00
#define FLASH_GET_DEVICE_ID           	   0x01
#define FLASH_GET_SIZE_INFO           	   0x02
#define FLASH_GET_ORGANIZATION        	   0x03
#define FLASH_CHECK_WRITE_PROTECTED   	   0x04
#define FLASH_GET_ERASE_STATUS        	   0x05
#define FLASH_COPY_EPROM              	   0x06
#define FLASH_GET_SOFT_STATE          	   0x0A

// Modes for mlxrt_set_flash_mode

#define FLASH_ERASE_ENABLE            	   0x0B
#define FLASH_ERASE_DISABLE           	   0x0C
#define FLASH_PROGRAM_ENABLE          	   0x0D
#define FLASH_PROGRAM_DISABLE         	   0x0E



#endif // !_INC_MLXRTDEF


