/********************************************************************
*  This header file is included by the header files ML8RTBIB.H
*  and ML6RTBIB.H and ML7RTBIB.H.
*
*  Its purpose is to declare the data types for the libraries
*  ML8RTBIB and ML6RTBIB and ML7RTBIB.
*
*  Author         HK
*
*  Last change    22/09/98   ML7
*
*  Author         Dipl.-Ing. H.Doersam
*
*  Last change    09/04/98
*
*  Copyrights by SORCUS Computer GmbH 1998
*  All rights reserved.
********************************************************************/



#ifndef _INC_MLXRTTYP
#define _INC_MLXRTTYP



// Standard data types

#define VOID            void
#define UCHAR           unsigned char
#define CHAR            char
#define USHORT          unsigned short int
#define SHORT           short int
#define ULONG           unsigned long int
#define LONG            long int

#ifndef byte
#define byte            UCHAR
#endif  // !byte

#ifndef ushort
#define ushort          USHORT
#endif  // !ushort

#ifndef ulong
#define ulong           ULONG
#endif  // !ulong



// Buffer handle
typedef LONG HBUF8;
typedef LONG HBUF7;
typedef LONG HBUF6;


// MDD handle
typedef ULONG HMDD8;
typedef ULONG HMDD7;
typedef ULONG HMDD6;


// Procedure pointer to call a global task procedure directly
typedef VOID (PFAR *GLOBALPROC) (VOID);



// Header of the Program-Descriptor-Table (PDT)
typedef struct
{
   UCHAR    ucType;              // Type of PDT (= 1)
   UCHAR    ucSize;              // Size of the CDT header 
                                 // (= _PDT_LENGTH)
   USHORT   usGlobalProc;        // Number of global procedures
   USHORT   usProgNo;            // Program number
   CHAR     cVersion;            // Version of program 
                                 // (character like '1')
   CHAR     cRevision;           // Revision of program 
                                 // (character like 'A')
   UCHAR    ucCPU;               // CPU type
   UCHAR    ucCoProc;            // Coprocessor type
   UCHAR    ucLanguage;          // Used programming language
   UCHAR    ucProgType;          // Program type
   USHORT   usFlags;             // Flags
   USHORT   usInt;               // Served interrupt
   ULONG    ulDataAdr;           // Address (physical) of task data
   ULONG    ulDataSize;          // Size of task data
   ULONG    ulDataSizeMin;       // Minimum size of task data
   ULONG    ulDataSizeMax;       // Maximum size of task data
   ULONG    ulGlobalParAdr;      // Address (physical) of global
                                 // parameters
   USHORT   usGlobalParSize;     // Size of global parameters
   ULONG    ulHyperAdr;          // Address (physical) of hypertext
   USHORT   usRes1;              // Reserved
   ULONG    ulRes2;              // Reserved
} ML8_PDT_HEAD, ML7_PDT_HEAD, ML6_PDT_HEAD;



// Task-Descriptor-Table
typedef struct
{
   UCHAR    typ;                 // Type of TDT
   UCHAR    lengt;               // Size of TDT
   USHORT   task;                // Task number of program
   USHORT   flags;               // Flags
   USHORT   inter;               // DI- or II-Task: Interrupt
                                 // NI-Task: Number of activations
   USHORT   par;                 // Size of global parameters
   USHORT   proc;                // Number of task procedures or
                                 // functions
   ULONG    pdt;                 // Address of PDT
   ULONG    hyper;               // Address of hypertext
   ULONG    datafirst;           // Start address of task data
   ULONG    datalast;            // End address of task data + 1
   ULONG    readptr;             // Actual data read-pointer
   ULONG    writeptr;            // Actual data write-pointer
} tdt_type, TDT_TYPE;



// Error information
typedef struct
{
   USHORT   errorcode;           // Error code
   USHORT   subnum;              // Error localization
   USHORT   mark;                // Last mark set before error
   USHORT   erroverflow;         // Error overflow (= 1)
}  ML8RT_ERROR_INFO_TYPE,
   ML7RT_ERROR_INFO_TYPE,
   ML6RT_ERROR_INFO_TYPE;



// Error information from host driver
typedef struct
{
   USHORT   usHostNo;            // Number of host
   USHORT   usError;             // Error code
}  ML8_HOST_ERROR_INFO,
   ML7_HOST_ERROR_INFO,
   ML6_HOST_ERROR_INFO;



// Specification of host
typedef struct
{
   USHORT   usHostNo;            // Number of host
   USHORT   usMode;              // Mode
   USHORT   usTaskNo;            // Task number
   USHORT   usSrqFuncNo;         // Number of task function for SRQs
   USHORT   usErrorFuncNo;       // Number of task function for error
}  ML8_HOST,
   ML7_HOST,
   ML6_HOST;



// Error handler for host adapter
typedef struct
{
   USHORT   usTask;              // Task number
   USHORT   usFuncNr;            // Function number
}  ML8_HOST_ERROR_HANDLER,
   ML7_HOST_ERROR_HANDLER,
   ML6_HOST_ERROR_HANDLER;

typedef struct
{
   USHORT   usTask;              // Task number
   USHORT   usFuncNr;            // Function number
}  MLX_CB_TYPE;


// Easier access to the bytes of a DWORD
typedef union
{
	UCHAR			ucData[4];        // Byte-0 .. Byte-3
	USHORT		usData[2];        // Word-0, Word-1
	ULONG			ulData;           // Dword
} DWORD_TYPE;


// Register for a standard call of a OsX subroutine
typedef struct
{
   DWORD_TYPE  ulEAX;            // ul.EAX.ucData[0]  = AL
                                 // ul.EAX.ucData[1]  = AH
                                 // ul.EAX.usData[0]  = AX
                                 // ul.EAX.ulData     = EAX     
   
   DWORD_TYPE  ulEBX;            // ul.EBX.ucData[0]  = BL
                                 // ul.EBX.ucData[1]  = BH
                                 // ul.EBX.usData[0]  = BX
                                 // ul.EBX.ulData     = EBX     
   
   DWORD_TYPE  ulECX;            // ul.ECX.ucData[0]  = CL
                                 // ul.ECX.ucData[1]  = CH
                                 // ul.ECX.usData[0]  = CX
                                 // ul.ECX.ulData     = ECX     
   
   DWORD_TYPE  ulEDX;            // ul.EDX.ucData[0]  = DL
                                 // ul.EDX.ucData[1]  = DH
                                 // ul.EDX.usData[0]  = DX
                                 // ul.EDX.ulData     = EDX     
} OSXSTDCALL;



// MICRO TIME CODE
typedef struct
{
	byte		ucRes;				//	Reserved
	byte		ucHours;				//	Hours (0..23)
	byte		ucMinutes;			//	Minutes (0..59)
	byte		ucSec;				//	Seconds (0..59)
	ushort	usMilliSec;			//	Milliseconds (0..999)
	ushort	usMicroSec;			//	Microseconds (0..999)
} MICRO_TIME_CODE;



// MDD service data
typedef union
{
   UCHAR    ucData;              // UCHAR data
   USHORT   usData;              // USHORT data
   SHORT    sData;               // SHORT data
   ULONG    ulData;              // ULONG data
   VOID*    pData;               // Data POINTER
} MDDI_DATA;


// MDD data exchange structure
typedef struct
{
   HMDD8       hChannel;         // Handle of channel
   MDDI_DATA   rcData;           // Service data
   USHORT      usSize;           // Block length
   USHORT      usID;             // Service ID
   USHORT      usError;          // Error status
} MDDI_DATA_EX;



#endif // !_INC_MLXRTTYP



