{****************************************************************************}
{**                Blink-LED als DI-Task (MODULAR-4/486)                   **}
{**                                                                        **}
{**  Dieses Programm lt entweder die On-Board- oder die Externe-LED      **}
{**  blinken. Es soll als Beispiel zur Erstellung von Echtzeitprogrammen   **}
{**  dienen. Weitere Hinweise entnehmen Sie bitte dem Benutzerhandbuch     **}
{**  bzw. dem Technischen-Handbuch.                                        **}
{**                                                                        **}
{**  Installationshinweise:                                                **}
{**                                                                        **}
{**  Programmnummer  : 301h						   **}
{**  Tasknummer      : beliebig                                            **}
{**  Interruptquelle : 91h (Timer-A, Slave-Controller)                     **}
{**  Datenbereich    : 0                                                   **}
{**  Flags           : 0x0982 (Task nicht aktiviert, Auto-Init aufrufen)   **}
{**                                                                        **}
{**                                                                        **}
{**                       TURBO-PASCAL (Version 7.0)                       **}
{**                                                                        **}
{**                             Version 1.D.                               **}
{**                                                                        **}
{**                   Erstellungsdatum : 28.09.92                          **}
{**                    letzte nderung : 20.09.94                          **}
{**                             Status : freigegeben                       **}
{**                              Autor : M.H.                              **}
{**                                                                        **}
{**      (c) 1992, 1993, 1994 SORCUS Computer GmbH Heidelberg              **}
{**                                                                        **}
{****************************************************************************}
{$N+}                                     { Coprozessor ein }
{$E+}                                     { keine Emulation }
{$R-}                                     { kein Range-Check }
{$S-}                                     { kein Stack-Checking }
{$I-}                                     { kein IO-Checking }
{$V-}                                     { Keine strikte Stringbergabe }
{$L+}                                     { Local-Symbols fr Turbo-Debugger }
{$F+}                                     { FAR Aufrufe erzwingen }
{$M 1024, 0, 0}                           { Stack und Heap auf Minimum }

uses ml8rtbib;                            { Die ML8 On-Board Bibliothek einbinden }

Const                        { ***** ALLGEMEINE ANGABEN ***** }
   Programm_Number      : Word = $301;    { Programmnummer = 301h }
   Version              : Char = '1';     { Version des Porgramms }
   Revision             : Char = 'D';     { Revision des Programms }

                             { ***** Die verschiedenen Statusmglichkeiten des Programms ***** }
   READY                = 0;              { Bereit }
   RUNNING              = 2;              { Programm luft }
   LED_ERROR            = 3;              { Falsche LED angewhlt }
   STOPPED              = 4;              { Programm abgebrochen }

                             { ***** Zustnde der LED ***** }
   EIN                  = 1;
   AUS                  = 0;

                             { ***** Benutzte LED ***** }
   INTERN               = 1;              { Interne LED }
   EXTERN               = 2;              { Externe LED }

                             { ***** Aufbau der Programm-Descriptor-Tabelle (PDT) ***** }
Type
   PDT_Type             = Record
                             Typ           : Byte;     { Typ der Kopftabelle }
                             Lengt         : Byte;     { Lnge der Kopftabelle incl. Prozedurangaben }
                             Proc_Num      : Word;     { Anzahl der Prozeduren }
                             Prg_Number    : Word;     { Nummer des Programms }
                             Version       : Char;     { Version des Programms. Z.B.: A,B,C ... }
                             Revision      : Char;     { Revision des Programms. Z.B.: 1,2,3 ... }

                             CPU_Type      : Byte;     { Prozessor-Typ }
                             COProz_Type   : Byte;     { Co-Prozessor-Typ }
                             Language      : Byte;     { Programmiersprache }
                             Prg_Type      : Byte;     { Programm-Typ }

                             Flags         : Word;     { Installierungsflags }
                             Interrupt_Num : Word;     { Nummer des benutzten Interrupts }

                             Data_Adr      : LongInt;  { Anfangsadresse des Datenbereichs (physikalische Adresse) }
                             Data_Num      : LongInt;  { Gre des Datenbereichs }
                             Data_Min      : LongInt;  { Datenbereich den das Programm mindestens bentigt }
                             Data_Max      : LongInt;  { Maximaler Datenbereich den das Programm verarbeiten kann }

                             Parameter_Adr : LongInt;  { Adresse des Parameterbereichs (physikalische Adresse) }
                             Parameter_Num : Word;     { Anzahl der Parameter }

                             Hyper_Adr     : LongInt;  { Anfangsadresse des Hypertext (physikalische Adresse) }
                             Res1          : Word;
                             Res2          : LongInt;

                             Main          : Pointer;  { Adresse der Haupt-Prozedur (Task-Prozedur) }
                             Auto_Init     : Pointer;  { Adresse der Auto-Init-Prozedur }
                             Start         : Pointer;  { Adresse der Start-Prozedur }
                             Stop          : Pointer;  { Adresse der Stop-Prozedur }
                          end;


                             { ***** DEKLARATION DER GLOBALEN PROGRAMMDATEN ***** }
VAR
   PDT                  : PDT_Type;                    { Deklaration der Kopftabelle }

                             { ***** Parameterbereich des Programms deklarieren. ***** }
                             { Auf diese Daten kann spter ber das Betriebssystem zugegriffen werden. }
   Parameter            : Record
                             TDT       : TDT_Type;     { Platz fr die TDT reservieren }
                             Status    : Byte;         { Status des Programms }
                             Blink_Rate: Word;         { Blinkrate in Hz }
                             LED_Status: Byte;         { Zustand der LED }
                             LED       : Byte;         { Welche LED 1=Intern, 2=Extern }
                             Soft_Cnt  : Byte;         { Soft-Counter. Erweiterung des Hardware-Counters }
                             Soft_Init : Byte;         { Soft-Counter Initialisierungswert }
                          end;


Procedure Auto_Init;         { ***** AUTO-INITIALISIERUNG }
begin
   ml8rt_entry;                           { Register retten und vorbereiten }
   Parameter.Status := READY;             { Status des Programms auf bereit }
   Parameter.Blink_Rate := 1;             { Blinkrate defaultmig auf 1 Hz einstellen }
   Parameter.LED_Status := AUS;           { Status der LED auf "AUS" }
   Parameter.LED := INTERN;               { Standartmig die externe LED benutzen }
   ml8rt_External_LED_Off;                { LED ausschalten }
   ml8rt_Local_LED_Off;
   ml8rt_exit;                            { Register wieder restaurieren }
end;


Procedure Start;             { ***** Programm starten ***** }
VAR
   n                    : LongInt;
   Timer                : Word;
begin
   ml8rt_entry;
   if (Parameter.LED=EXTERN) OR (Parameter.LED=INTERN) then
   begin                                  { Aus der angegebenen Frequenz den Timer-Wert, und den Soft-Teiler berechnen }
      n := 2500000 DIV Parameter.Blink_Rate;{ Die Timer werden mit 2,5 MHz getaktet. }
      Parameter.Soft_Init := (n DIV 60000); { Die Zeiten fr den Hardware-Timer mglichst gro whlen }
      Parameter.Soft_Cnt := Parameter.Soft_Init;
      Timer := Round (n/(Parameter.Soft_Init+1));
      ml8rt_set_timer (TIMER_A, Timer, INT_MODE); { Timer-A starten }
      Parameter.Status := RUNNING;
      ml8rt_unmask_int (IRQ_TIMER_A);     { Timer-A demaskieren }
   end
   else
      Parameter.Status := LED_ERROR;
   ml8rt_exit;
end;


Procedure Stop;              { ***** Programmlauf abbrechen ***** }
begin
   ml8rt_entry;
   ml8rt_mask_int (IRQ_TIMER_A);          { Timer-A Interrupt maskieren }
   Parameter.Status := STOPPED;           { Parameter setzen }
   ml8rt_exit;
end;


Procedure Main_Proc;         { ***** HAUPTPROZEDUR (DI-Task) ***** }
begin
   ml8rt_entry;
   if Parameter.Soft_Cnt=0 then
   begin
      if Parameter.LED_Status = EIN then
      begin                                  { Falls die LED eingeschaltet ist, ... }
         if Parameter.LED = EXTERN then
            ml8rt_External_LED_Off           { ... LED ausschalten und ... }
         else
            ml8rt_Local_LED_Off;
         Parameter.LED_Status := AUS;        { ... Zustand merken }
      end
      else                                   { Falls die LED ausgeschaltet ist, ... }
      begin
         if Parameter.LED = EXTERN then
            ml8rt_External_LED_On            { ... LED einschalten ... }
         else
            ml8rt_Local_LED_On;
         Parameter.LED_Status := EIN;        { ... Zustand merken }
      end;
      Parameter.Soft_Cnt := Parameter.Soft_Init;
   end
   else
   begin
      Parameter.Soft_Cnt := Parameter.Soft_Cnt - 1;
   end;

   Port[MASTER_INT_CONTR] := $20;         { EOI zum Master-Controller senden }
   Port[SLAVE_INT_CONTR] := $20;          { EOI zum Slave-Controller senden }
   ml8rt_exit_interrupt;
end;


Procedure Init_PDT;
begin                                     { Die Procedur stellt alle Werte in der Kopftabelle ein }
   PDT.Typ := 1;                          { Programm-Desriptor-Tabellen-Typ (im Moment wird nur der Typ 1 untersttzt) }
   PDT.Lengt := _PDT_Length;              { Gre des Vorspanns der Kopftabelle (im Moment immer = _PDT_Length = 48) }
   PDT.Proc_Num := 4;                     { Anzahl der Prozeduren (in diesem Beispiel Auto_Init, Main_Proc, Start, Stop) }
   PDT.Prg_Number := Programm_Number;     { Programm-Nummer }
   PDT.Version := Version;                { Programm-Version }
   PDT.Revision := Revision;              { Programm-Revision }
   PDT.CPU_Type := _486SX;                { CPU-Typ }
   PDT.CoProz_Type := _NOCOPROZ;          { Co-Prozessor-Typ }
   PDT.Language := _TP70;                 { Programmiersprache }
   PDT.Prg_Type := _User_Prg;             { Programm-Typ: Anwenderprogramm }
   PDT.Flags := $0;                       { Flags }
   PDT.Flags := _DI_Task + _Local_Data + _PDT_Info_enable + _Local_Parameter + _Fixed_Datasize;
   PDT.Interrupt_Num := IRQ_TIMER_A;      { Interrupt-Nummer. Hier fest auf Timer-A eingestellt }
   PDT.Data_Adr := 0;                     { Adresse des Datenbereichs }
   PDT.Data_Num := 0;                     { Gre des Datenbereichs }
   PDT.Data_Min := 0;                     { Minimaler Datenbereich }
   PDT.Data_Max := 0;                     { Maximaler Datenbereich }
                                          { Anfangsadresse des Parameterbereichs }
   ml8rt_phys_adr (@Parameter, PDT.Parameter_Adr);
   PDT.Parameter_Adr := PDT.Parameter_Adr + SizeOf(TDT_Type);
                                          { Anzahl der Parameter }
   PDT.Parameter_Num := SizeOf(Parameter) - SizeOf(TDT_Type);
   PDT.Hyper_Adr := 0;                    { Adresse des Hypertextes (wird nicht benutzt) }
   PDT.Res1 := 0;
   PDT.Res2 := 0;
   PDT.Main := @Main_Proc;                { Adresse der Hauptprozedur }
   PDT.Auto_Init := @Auto_Init;           { Adresse der Auto-Init-Prozedur }
   PDT.Start := @Start;                   { Adresse der Start-Prozedur }
   PDT.Stop := @Stop;                     { Adresse der Stop-Prozedur }
end;


begin                        { ***** PREPARE ***** }
   Init_PDT;                              { PDT initialisieren }
   ml8rt_Set_PDT_Adr (PDT);               { Adresse der PDT im Rckgabe-Record dem Betriebssystem bergeben }
end.
