{****************************************************************************}
{**                Blink-LED als NI-Task (MODULAR-4/486)                   **}
{**                                                                        **}
{**  Dieses Programm lt entweder die On-Board- oder die Externe-LED      **}
{**  blinken. Es soll als Beispiel zur Erstellung von Echtzeitprogrammen   **}
{**  dienen. Weitere Hinweise entnehmen Sie bitte dem Benutzerhandbuch     **}
{**  bzw. dem Technischen-Handbuch                                         **}
{**                                                                        **}
{**  Installationshinweise:                                                **}
{**                                                                        **}
{**  Programmnummer  : 300h						   **}
{**  Tasknummer      : beliebig                                            **}
{**  Interruptquelle : 1 (wird nicht verwendet, da NI-Task)                **}
{**  Datenbereich    : 0                                                   **}
{**  Flags           : 0x0980 (nicht aktiviert, Auto-Init aufrufen)        **}
{**                                                                        **}
{**                                                                        **}
{**                    Fr TURBO-PASCAL (Version 7.0)                      **}
{**                                                                        **}
{**                             Version 1.F.                               **}
{**                                                                        **}
{**                   Erstellungsdatum : 24.09.92                          **}
{**                    letzte nderung : 06.06.2000 (HK)                   **}
{**                             PDT auf neustem Stand                      **}
{**                             Status : freigegeben                       **}
{**                              Autor : M.H.                              **}
{**                                                                        **}
{**         (c) 1992, 1993, 1994 SORCUS Computer GmbH Heidelberg           **}
{**                                                                        **}
{****************************************************************************}
{$N+}                                     { Coprozessor aus }
{$E+}                                     { keine Emulation }
{$R-}                                     { kein Range-Check }
{$S-}                                     { kein Stack-Checking }
{$I-}                                     { kein IO-Checking }
{$V-}                                     { Keine strikte Stringbergabe }
{$L+}                                     { Local-Symbols fr Turbo-Debugger }
{$F+}                                     { FAR Aufrufe erzwingen! }
{$M 1024, 0, 0}                           { Stack und Heap auf Minimum }

uses ml8rtbib;                            { Die ML8 On-Board Bibliothek einbinden }

Const                        { ***** ALLGEMEINE ANGABEN ***** }
   Programm_Number      : Word = $300;    { Programmnummer = 300h }
   Version              : Char = '1';     { Version des Programms }
   Revision             : Char = 'F';     { Revision des Programms }

			     { ***** Die verschiedenen Statusmglichkeiten des Programms ***** }
   READY                = 0;              { Bereit }
   RUNNING              = 2;              { Programm luft }
   ERROR                = 3;              { Falsche LED angewhlt }
   STOPPED              = 4;              { Programmlauf angehalten }


			     { ***** Zustnde der LED ***** }
   EIN                  = 1;
   AUS                  = 0;

			     { ***** Benutzte LED ***** }
   INTERN               = 1;              { Interne LED }
   EXTERN               = 2;              { Externe LED }

			     { ***** Aufbau der Programm-Descriptor-Tabelle (PDT) ***** }
Type
   PDT_Type             = Record
			     PDT_HEAD : ML8_PDT_HEAD;  { PDT-Header }

			     Main          : Pointer;  { Adresse der Haupt-Prozedur (Task-Prozedur) }
			     Auto_Init     : Pointer;  { Adresse der Auto-Init-Prozedur }
                             Start         : Pointer;  { Adresse der Start-Prozedur }
                             Stop          : Pointer;  { Adresse der Stop-Prozedur }
			  end;


			     { ***** DEKLARATION DER GLOBALEN PROGRAMMDATEN ***** }
VAR
   PDT                  : PDT_Type;                    { Deklaration der Kopftabelle }

			     { ***** Parameterbereich des Programms deklarieren. ***** }
			     { Auf diese Daten kann spter ber das Betriebssystem zugegriffen werden. }
   Parameter            : Record
			     TDT       : TDT_Type;     { Platz fr die TDT reservieren }
			     Status    : Byte;         { Status des Programms }
			     Blink_Rate: Word;         { Blinkrate }
			     LED_Status: Byte;         { Zustand der LED }
			     LED       : Byte;         { Welche LED 1=Intern, 2=Extern }
			  end;

   Pause                : Word;           { Ein Zhler, der die Durchlufe des Programms zhlt }


Procedure Auto_Init;         { ***** AUTO-INITIALISIERUNG }
begin
   ml8rt_entry;                           { Register retten und vorbereiten }
   Parameter.Status := READY;             { Status des Programms auf bereit }
   Parameter.Blink_Rate := 3000;          { Blinkrate defaultmig einstellen }
   Parameter.LED_Status := AUS;           { Status der LED auf "AUS" }
   Parameter.LED := INTERN;               { Standartmig die interne LED benutzen }
   ml8rt_External_LED_Off;                { Externe LED ausschalten }
   ml8rt_Local_LED_Off;                   { On-Board LED ausschalten }
   Pause := 0;                            { Durchlaufzhler auf Null }
   ml8rt_exit;                            { Register wieder restaurieren }
end;


Procedure Start;             { ***** Start-Prozedur ***** }
begin
   ml8rt_entry;
   if (Parameter.LED<>INTERN) AND (Parameter.LED<>EXTERN) then
   begin                                  { Falls eine falsche LED angewhlt wurde, ... }
      Parameter.Status := ERROR;          { ... Fehlermeldung ausgeben }
   end
   else
   begin                                  { Falls alles OK, dann Task aktivieren }
      Parameter.Status := RUNNING;        { (TDT enthlt die Tasknummer, unter der das Programm installiert wurde) }
      ml8rt_wakeup_task (Parameter.TDT.Task);
   end;
   ml8rt_exit;
end;


Procedure Stop;              { ***** Stop-Prozedur ***** }
begin
   ml8rt_entry;
   ml8rt_sleep_task (Parameter.TDT.Task); { Task deaktivieren }
   Parameter.Status := STOPPED;           { Parameter auf "Programm angehalten". }
   ml8rt_exit;
end;


Procedure Main_Proc;         { ***** HAUPTPROZEDUR (NI-Task) ***** }
begin
   ml8rt_entry;

   Pause := Pause + 1;                    { Durchlaufzhler um 1 erhhen }
   if Pause>=Parameter.Blink_Rate then    { Falls die Pause (Anzahl Aufrufe der Main-Proc) den angegebenen Wert erreicht hat, }
   begin                                  { ... die LED umschalten }
      if Parameter.LED_Status = EIN then
      begin                               { Falls die LED eingeschaltet ist, ... }
	 if Parameter.LED = EXTERN then
	    ml8rt_External_LED_Off        { ... LED ausschalten und ... }
	 else
	    ml8rt_Local_LED_Off;
	 Parameter.LED_Status := AUS;     { ... Zustand merken }
      end
      else                                { Falls die LED ausgeschaltet ist, ... }
      begin
	 if Parameter.LED = EXTERN then
	    ml8rt_External_LED_On         { ... LED einschalten ... }
	 else
	    ml8rt_Local_LED_On;
	 Parameter.LED_Status := EIN;     { ... Zustand merken }
      end;
      Pause := 0;                         { Durchlaufzhler wieder auf Null setzen }
   end;

   ml8rt_Exit;
end;

begin                        { ***** PREPARE ***** }
   PDT.PDT_HEAD.ucType := 1;                    { Programm-Descriptor-Tabellen-Typ (im Moment wird nur der Typ 1 untersttzt) }
   PDT.PDT_HEAD.ucSize := sizeof(ML8_PDT_HEAD); { Gre des Vorspanns der Kopftabelle (im Moment immer = _PDT_Length = 48) }
   PDT.PDT_HEAD.usGlobalProc := 4;              { Anzahl der Prozeduren }
   PDT.PDT_HEAD.usProgNo := Programm_Number;    { Programm-Nummer }
   PDT.PDT_HEAD.cVersion := Version;            { Programm-Version }
   PDT.PDT_HEAD.cRevision := Revision;          { Programm-Revision }
   PDT.PDT_HEAD.ucCPU := _486SX;                { CPU-Typ }
   PDT.PDT_HEAD.ucCoProc := _NOCOPROZ;          { Co-Prozessor-Typ }
   PDT.PDT_HEAD.ucLanguage := _TP70;            { Programmiersprache }
   PDT.PDT_HEAD.ucProgType := _User_Prg;        { Programm-Typ: Anwenderprogramm }
   PDT.PDT_HEAD.usFlags := $0;                  { Flags }
   PDT.PDT_HEAD.usFlags := _NI_Task + _PDT_Info_Enable + _Local_Data + _Local_Parameter + _Fixed_Datasize;
   PDT.PDT_HEAD.usInt := 0;                     { Interrupt-Nummer fr die das Programm gedacht ist (ungltig, da NI-Task) }
   PDT.PDT_HEAD.ulDataAdr := 0;                 { Adresse des Datenbereichs }
   PDT.PDT_HEAD.ulDataSize := 0;                { Gre des Datenbereichs }
   PDT.PDT_HEAD.ulDataSizeMin := 0;             { Minimaler Datenbereich }
   PDT.PDT_HEAD.ulDataSizeMax := 0;             { Maximaler Datenbereich }
					        { Anfangsadresse des Parameterbereichs }
   ml8rt_phys_adr (@Parameter, PDT.PDT_HEAD.ulGlobalParAdr);
   PDT.PDT_HEAD.ulGlobalParAdr:= PDT.PDT_HEAD.ulGlobalParAdr + SizeOf(TDT_Type);
					        { Anzahl der Parameter }
   PDT.PDT_HEAD.usGlobalParSize := SizeOf(Parameter) - SizeOf(TDT_Type);
   PDT.PDT_HEAD.ulHyperAdr := 0;                { Adresse des Hypertextes (wird nicht benutzt) }
   PDT.PDT_HEAD.usRes1 := 0;
   PDT.PDT_HEAD.ulRes2 := 0;
   PDT.Main := @Main_Proc;                      { Adresse der Hauptprozedur }
   PDT.Auto_Init := @Auto_Init;                 { Adresse der Auto-Init-Prozedur }
   PDT.Start := @Start;                         { Adresse der Start-Prozedur }
   PDT.Stop := @Stop;                           { Adresse der Stop-Prozedur }
   ml8rt_Set_PDT_Adr (PDT);                     { Adresse der PDT im Rckgabe-Record dem Betriebssystem bergeben }
end.
