{****************************************************************************
{**                Blink-LED als DI-Task (MODULAR-4/486)
{**
{**  Dieses Programm lt entweder die On-Board- oder die Externe-LED
{**  blinken. Es soll als Beispiel zur Erstellung von Echtzeitprogrammen
{**  dienen. Weitere Hinweise entnehmen Sie bitte dem Benutzerhandbuch
{**  bzw. dem Technischen-Handbuch.
{**
{**  Installationshinweise:
{**
{**  Programmnummer  : 301h
{**  Tasknummer      : beliebig
{**  Interruptquelle : 91h (Timer-A, Slave-Controller)
{**  Datenbereich    : 0
{**  Flags           : 0x0982 (Task nicht aktiviert, Auto-Init aufrufen)
{**
{**
{**                       TURBO-PASCAL (Version 7.0)
{**
{**                             Version 1.G
{**
{**                   Erstellungsdatum : 28.09.92
{**                    letzte nderung : 06.06.2000 (HK)
{**                             PDT auf neustem Stand
{**                             Status : freigegeben
{**                              Autor : M.H.
{**
{**         (c) 1992..1996 SORCUS Computer GmbH Heidelberg
{**
{****************************************************************************}
{$N+}                                     { Coprozessor ein }
{$E+}                                     { keine Emulation }
{$R-}                                     { kein Range-Check }
{$S-}                                     { kein Stack-Checking }
{$I-}                                     { kein IO-Checking }
{$V-}                                     { Keine strikte Stringbergabe }
{$L+}                                     { Local-Symbols fr Turbo-Debugger }
{$F+}                                     { FAR Aufrufe erzwingen }
{$M 1024, 0, 0}                           { Stack und Heap auf Minimum }

uses ml8rtbib;                            { ML8 Echtzeit Bibliothek einbinden}

{ ***** ALLGEMEINE ANGABEN ***** }
Const
   Programm_Number      : Word = $301;    { Programmnummer = 301h }
   Version              : Char = '1';     { Version des Porgramms }
   Revision             : Char = 'G';     { Revision des Programms }

   { ***** Die verschiedenen Statusmglichkeiten des Programms ***** }
   READY                = 0;              { Bereit }
   RUNNING              = 2;              { Programm luft }
   LED_ERROR            = 3;              { Falsche LED angewhlt }
   STOPPED              = 4;              { Programm abgebrochen }

   { ***** Zustnde der LED ***** }
   EIN                  = 1;
   AUS                  = 0;

   { ***** Benutzte LED ***** }
   INTERN               = 1;              { Interne LED }
   EXTERN               = 2;              { Externe LED }

   { ***** Timer Initialisierungen ***** }
   { Timer A 16-bit-> (65535/ 2), da Periodendauer T = t(LED_ON) + t(LED_OFF)}
   TIMER_VAL            : longint = 30000;
   { Timer A Basistakt 2.5MHz (bzw. 10 MHz)}
   TIMER_FREQ           : longint = 2500000;

{ ***** Aufbau der Programm-Descriptor-Tabelle (PDT) ***** }
Type
   PDT_Type = Record
               PDT_HEAD : ML8_PDT_HEAD;  { PDT-Header }

               Main          : Pointer;  { Adresse der Haupt-Prozedur }
               Auto_Init     : Pointer;  { Adresse der Auto-Init-Prozedur }
               Start         : Pointer;  { Adresse der Start-Prozedur }
               Stop          : Pointer;  { Adresse der Stop-Prozedur }
              end;


{ ***** DEKLARATION DER GLOBALEN PROGRAMMDATEN ***** }
VAR
   PDT                  : PDT_Type;            { Deklaration der Kopftabelle }

                             { ***** Parameterbereich des Programms deklarieren. ***** }
                             { Auf diese Daten kann spter ber das Betriebssystem zugegriffen werden. }
   Parameter : Record
                TDT       : TDT_Type;     { Platz fr TDT reservieren }
                Status    : Byte;         { Status des Programms }
                Blink_Rate: Word;         { Blinkrate in Hz }
                LED_Status: Byte;         { Zustand der LED }
                LED       : Byte;         { LED: 1=Intern, 2=Extern }
                Soft_Cnt  : Byte;         { Soft-Counter noetig, um mit Timer}
                                          { auch niedrige Freq. zu erzeugen! }
                Soft_Init : Byte;         { Zaehlerinitialisierungswert }
               end;


{ ***** AUTO-INITIALISIERUNG }
Procedure Auto_Init;
begin
   ml8rt_entry;                           { Register retten und vorbereiten }
   Parameter.Status := READY;             { Status des Programms auf bereit }
   Parameter.Blink_Rate := 1;             { Blinkrate Defaultwert 1 Hz }
   Parameter.LED_Status := AUS;           { Status der LED auf "AUS" }
   Parameter.LED := INTERN;               { Standardmaessig externe LED }
   ml8rt_External_LED_Off;                { LED ausschalten }
   ml8rt_Local_LED_Off;
   ml8rt_exit;                            { Register wieder restaurieren }
end;

{ ***** Programm starten ***** }
Procedure Start;
VAR
   timer : Word;
begin
   ml8rt_entry;
   if (Parameter.LED=EXTERN) OR (Parameter.LED=INTERN) then
   begin
    {* Softzaehler ausrechnen (muss ganzzahlig sein, keine Fliesskommaop.!): *}
	 parameter.soft_init:= TIMER_FREQ div (2* parameter.blink_rate * TIMER_VAL);

	 {* falls Softzaehler auf 0 "abgerundet" wird, setze Zaehler auf 1 ! *}
	 if parameter.soft_init = 0 then
           parameter.soft_init := 1;
	 parameter.soft_cnt := parameter.soft_init;

	 {* Timerwert ausrechnen: Fehler durch Softteiler (ganzzahlig!) wird durch
		 Anpassung des maximalen Timerwerts kompensiert! *}
	 timer := TIMER_FREQ div (2 * parameter.blink_rate * parameter.soft_init);

      ml8rt_set_timer (TIMER_A, timer, INT_MODE); { Timer-A starten }
      Parameter.Status := RUNNING;
      ml8rt_unmask_int (IRQ_TIMER_A);             { Timer-A demaskieren }
   end
   else
      Parameter.Status := LED_ERROR;
   ml8rt_exit;
end;


{ ***** Programmlauf abbrechen ***** }
Procedure Stop;
begin
   ml8rt_entry;
   ml8rt_mask_int (IRQ_TIMER_A);          { Timer-A Interrupt maskieren }
   Parameter.Status := STOPPED;           { Parameter setzen }
   ml8rt_exit;
end;


{ ***** HAUPTPROZEDUR (DI-Task) ***** }
Procedure Main_Proc;
begin
   ml8rt_entry;
   Parameter.Soft_Cnt := Parameter.Soft_Cnt -1;
   if Parameter.Soft_Cnt=0 then
   begin
     if Parameter.LED_Status = EIN then
     begin                                { Falls die LED eingeschaltet ist, }
       if Parameter.LED = EXTERN then
          ml8rt_External_LED_Off          { ... LED ausschalten und ... }
       else
       ml8rt_Local_LED_Off;
       Parameter.LED_Status := AUS;       { ... Zustand merken }
     end
     else                                 { Falls die LED ausgeschaltet ist, }
     begin
       if Parameter.LED = EXTERN then
              ml8rt_External_LED_On       { ... LED einschalten ... }
       else
       ml8rt_Local_LED_On;
       Parameter.LED_Status := EIN;       { ... Zustand merken }
     end;
     Parameter.Soft_Cnt := Parameter.Soft_Init;
   end;

   ml8rt_end_of_int(IRQ_TIMER_A);         { EOI zum Slave-Controller senden }
   ml8rt_exit_interrupt;
end;




begin                                     { ***** PREPARE ***** }
   PDT.PDT_HEAD.ucType := 1;                        { Programm-Descriptor-Tabellen-Typ }
   PDT.PDT_HEAD.ucSize := _PDT_Length;              { Gre des Vorspanns (Standard 48 }
   PDT.PDT_HEAD.usGlobalProc := 4;                  { Anzahl der Prozeduren }
   PDT.PDT_HEAD.usProgNo := Programm_Number;        { Programm-Nummer }
   PDT.PDT_HEAD.cVersion := Version;                { Programm-Version }
   PDT.PDT_HEAD.cRevision := Revision;              { Programm-Revision }
   PDT.PDT_HEAD.ucCPU := _486DX;                    { CPU-Typ }
   PDT.PDT_HEAD.ucCoProc := _NOCOPROZ;              { Co-Prozessor-Typ }
   PDT.PDT_HEAD.ucLanguage := _TP70;                { Programmiersprache }
   PDT.PDT_HEAD.ucProgType := _User_Prg;            { Programm-Typ: Anwenderprogramm }
   PDT.PDT_HEAD.usFlags := $0;                      { Flags }
   PDT.PDT_HEAD.usFlags := _DI_Task + _PDT_Info_enable + _Local_Parameter;
   PDT.PDT_HEAD.usInt := IRQ_TIMER_A;               { Interrupt-Nummer (Timer-A) }
   PDT.PDT_HEAD.ulDataAdr := 0;                     { Adresse des Datenbereichs }
   PDT.PDT_HEAD.ulDataSize := 0;                    { Gre des Datenbereichs }
   PDT.PDT_HEAD.ulDataSizeMin := 0;                 { Minimaler Datenbereich }
   PDT.PDT_HEAD.ulDataSizeMax := 0;                 { Maximaler Datenbereich }
                                                    { Anfangsadresse Parameterbereich }
   ml8rt_phys_adr (@Parameter, PDT.PDT_HEAD.ulGlobalParAdr);
   PDT.PDT_HEAD.ulGlobalParAdr := PDT.PDT_HEAD.ulGlobalParAdr + SizeOf(TDT_Type);
                                                    { Anzahl der Parameter }
   PDT.PDT_HEAD.usGlobalParSize := SizeOf(Parameter) - SizeOf(TDT_Type);
   PDT.PDT_HEAD.ulHyperAdr := 0;                    { Adresse des Hypertextes }
   PDT.PDT_HEAD.usRes1 := 0;
   PDT.PDT_HEAD.ulRes2 := 0;
   PDT.Main := @Main_Proc;                          { Adresse der Hauptprozedur }
   PDT.Auto_Init := @Auto_Init;                     { Adresse der Auto-Init-Prozedur }
   PDT.Start := @Start;                             { Adresse der Start-Prozedur }
   PDT.Stop := @Stop;                               { Adresse der Stop-Prozedur }
   ml8rt_Set_PDT_Adr (PDT);                         { Adresse der PDT im Rckgabe-Record dem Betriebssystem bergeben }
end.
