{****************************************************************************}
{**                Blink-LED als DI-Task (MODULAR-4/486)                   **}
{**                                                                        **}
{**  Dieses Programm lt entweder die On-Board- oder die Externe-LED      **}
{**  blinken. Es soll als Beispiel zur Erstellung von Echtzeitprogrammen   **}
{**  dienen. Es ist zu groen Teil identisch mit dem Programm M8P0301.PAS. **}
{**  Es ist ein Durchlaufzhler in den Parametern hinzugefgt worden, der  **}
{**  die Anzahl der Blinksequenzen mitzhlt. Dieser Parameter soll dann    **}
{**  von einer zweiten Task (M8P0303.PAS) berwacht werden. Ab einer       **}
{**  festgelegten Zahl der Umschaltungen soll die zweite Task dann diese   **}
{**  Task stoppen.                                                         **}
{**  ber eine Taskfunktion lt sich die Anzahl der schon gemachten       **}
{**  Umschaltungen ermitteln. Das Programm M8P0303 benutzt diese Task-     **}
{**  Funktion.                                                             **}
{**  Als weiteres Feature ist dieses Programm unter den Timern A bis C     **}
{**  beliebig installierbar.                                               **}
{**  Weitere Hinweise entnehmen Sie bitte dem Benutzerhandbuch bzw. dem    **}
{**  Technischen-Handbuch.                                                 **}
{**                                                                        **}
{**  Installationshinweise:                                                **}
{**                                                                        **}
{**  Programmnummer  : 302h                                                **}
{**  Tasknummer      : beliebig                                            **}
{**  Interruptquelle : 91h (Timer-A), 92h (Timer-B), 93h (Timer-C)         **}
{**  Datenbereich    : 0                                                   **}
{**  Flags           : 098Ah (Task nicht aktiviert, Auto-Init aufrufen,    **}
{**                    Interruptnummer wird bei Installation festgelegt)   **}
{**                                                                        **}
{**                       TURBO-PASCAL (Version 7.0)                       **}
{**                                                                        **}
{**                             Version 1.G.                               **}
{**                                                                        **}
{**                   Erstellungsdatum : 28.09.92                          **}
{**                    letzte nderung : 19.05.00 (HK)                     **}
{**                        PDT auf neustem Stand                           **}
{**                    ml8rt_end_of_int statt Port-Befehle                 **}
{**                    Statt Pointer Bibliotheksfunktionen                 **}
{**                             Status : freigegeben                       **}
{**                              Autor : M.H.                              **}
{**                                                                        **}
{**       (c) 1992, 1993, 1994 SORCUS Computer GmbH Heidelberg             **}
{**                                                                        **}
{****************************************************************************}
{$N+}                                     { Coprozessor ein }
{$E+}                                     { keine Emulation }
{$R-}                                     { kein Range-Check }
{$S-}                                     { kein Stack-Checking }
{$I-}                                     { kein IO-Checking }
{$V-}                                     { Keine strikte Stringbergabe }
{$L+}                                     { Local-Symbols fr Turbo-Debugger }
{$F+}                                     { FAR Aufrufe erzwingen! }
{$M 1024, 0, 0}                           { Stack und Heap auf Minimum }

uses ml8rtbib;                            { Die ML8 On-Board Bibliothek einbinden }

Const                        { ***** ALLGEMEINE ANGABEN ***** }
   Programm_Number      : Word = $302;    { Programmnummer = 302h }
   Version              : Char = '1';     { Version des Porgramms }
   Revision             : Char = 'G';     { Revision des Programms }

			     { ***** Die verschiedenen Statusmglichkeiten des Programms ***** }
   READY                = 0;              { Bereit }
   RUNNING              = 2;              { Programm luft }
   LED_ERROR            = 3;              { Falsche LED angewhlt }
   STOPPED              = 4;              { Programm abgebrochen }
   SOURCE_ERROR         = 5;              { Programm unter einem falschen Interrupt installiert }

			     { ***** Zustnde der LED ***** }
   EIN                  = 1;
   AUS                  = 0;

                             { ***** Benutzte LED ***** }
   INTERN               = 1;              { Interne LED }
   EXTERN               = 2;              { Externe LED }

                             { ***** Aufbau der Programm-Descriptor-Tabelle (PDT) ***** }
Type
   PDT_Type             = Record
                             PDT_HEAD      : ML8_PDT_HEAD;  { PDT-Header }

                             Main          : Pointer;       { Adresse der Haupt-Prozedur (Task-Prozedur) }
                             Auto_Init     : Pointer;       { Adresse der Auto-Init-Prozedur }
                             Start         : Pointer;       { Adresse der Start-Prozedur }
                             Stop          : Pointer;       { Adresse der Stop-Prozedur }
                             Get_Switches  : Pointer;       { Adresse der Taskfunktion, die die Anzahl Umschaltungen liefert }
                          end;


                             { ***** DEKLARATION DER GLOBALEN PROGRAMMDATEN ***** }
VAR
   PDT                  : PDT_Type;                    { Deklaration der Kopftabelle }

                             { ***** Parameterbereich des Programms deklarieren. ***** }
                             { Auf diese Daten kann spter ber das Betriebssystem zugegriffen werden. }
   Parameter            : Record
                             TDT       : TDT_Type;     { Platz fr die TDT reservieren }
                             Status    : Byte;         { Status des Programms }
                             Blink_Rate: Word;         { Blinkrate in Hz }
                             LED_Status: Byte;         { Zustand der LED }
                             LED       : Byte;         { Welche LED 1=Intern, 2=Extern }
                             Soft_Cnt  : Byte;         { Soft-Counter. Erweiterung des Hardware-Counters }
                             Soft_Init : Byte;         { Soft-Counter Initialisierungswert }
                             Switch_Cnt: Word;         { Enthlt die Anzahl der Umschaltungen der LED }
                          end;


Procedure Auto_Init;         { ***** AUTO-INITIALISIERUNG }
begin
   ml8rt_entry;                           { Register retten und vorbereiten }
   Parameter.Status := READY;             { Status des Programms auf bereit }
   Parameter.Blink_Rate := 1;             { Blinkrate defaultmig auf 1 Hz einstellen }
   Parameter.LED_Status := AUS;           { Status der LED auf "AUS" }
   Parameter.LED := INTERN;               { Standartmig die externe LED benutzen }
   Parameter.Switch_Cnt := 0;             { Anzahl der Umschaltungen auf 0 setzen }
   ml8rt_External_LED_Off;                { LED ausschalten }
   ml8rt_Local_LED_Off;
   ml8rt_exit;                            { Register wieder restaurieren }
end;


Procedure Start;             { ***** Programm starten ***** }
VAR
   n                    : LongInt;
   Timer                : Word;
   Timer_in_use         : Byte;
begin
   ml8rt_entry;
   if (Parameter.LED=EXTERN) OR (Parameter.LED=INTERN) then
   begin                                  { Aus der angegebenen Frequenz den Timer-Wert, und den Soft-Teiler berechnen }
      n := 2500000 DIV Parameter.Blink_Rate;
      Parameter.Soft_Init := (n DIV 60000); { Die Zeiten fr den Hardware-Timer mglichst gro whlen }
      Parameter.Soft_Cnt := Parameter.Soft_Init;
      Timer := Round (n/(Parameter.Soft_Init+1));
      Parameter.Switch_Cnt := 0;          { Blink-Sequenzzhler auf Null }

                                          { Die Nummer des benutzten Timers berechnen (Ergibt sich aus dem Quell-Interrupt) }
                                          { Timer-A bis -C = 0 bis 2, TDT.Inter = Interrupt-Nummer, unter der Prg inst. wurde }
      Timer_in_use := Parameter.TDT.Inter - IRQ_TIMER_A;
      case Timer_in_use of
         TIMER_A,
         TIMER_B,
         TIMER_C :
	    begin                         { Timer setzen, starten und den Interrupt freigeben }
               ml8rt_set_timer (Timer_in_use, Timer, INT_MODE);
               ml8rt_unmask_int (Parameter.TDT.Inter);
               Parameter.Status := RUNNING;
            end;
      else                                { Falls das Prg unter einem anderen Interrupt installiert wurde, ... }
         Parameter.Status := SOURCE_ERROR;
      end;
   end
   else
      Parameter.Status := LED_ERROR;
   ml8rt_exit;
end;


Procedure Stop;              { ***** Programmlauf abbrechen (wird spter von der berwachungstask aufgerufen) ***** }
VAR
   Timer_in_use         : Byte;
begin
   ml8rt_entry;
   Timer_in_use := Parameter.TDT.Inter - IRQ_TIMER_A;
   case Timer_in_use of
      TIMER_A,
      TIMER_B,
      TIMER_C :
         begin                            { Timer setzen, starten und den Interrupt freigeben }
            ml8rt_mask_int (Parameter.TDT.Inter);
            Parameter.Status := STOPPED;  { Status auf "Programm angehalten" }
         end;
   else                                   { Falls das Prg unter einem anderen Interrupt installiert wurde, ... }
      Parameter.Status := SOURCE_ERROR;
   end;
   Parameter.Switch_Cnt := 0;             { Anzahl der Durchlufe wieder auf 0 setzen }
   ml8rt_exit;
end;

                             { ***** Taskfunktion zur Ermittelung der schon gemachten Umschaltungen ***** }
Procedure Get_Switches (task, insize : Word; VAR indata; outsize : Word; VAR outdata : Word);
VAR
   Err                  : Word;
begin
   ml8rt_entry_function;                  { bergabeparameter initialisieren }
   if outsize = SizeOf(outdata) then      { Falls der Datentyp stimmt, die Anzahl der Umschaltungen bergeben. }
   begin
      outdata := Parameter.Switch_Cnt;
      outsize := SizeOf(outdata);         { 2 Byte werden zurckgeliefert. (= Word) }
      Err := 0;
   end
   else
   begin
      outsize := 0;                       { Falls andere Daten erwartet werden, nichts zurckliefern! }
      Err := $01E0;                       { Das Low-Byte der Fehlermeldung mu immer $E0 sein, damit die Fehlermeldung }
   end;                                   { korrekt auf dem PC ankommt (Fehler bei Subroutine)! }
   ml8rt_exit_function (0, outsize, Err);
end;


Procedure Main_Proc;         { ***** HAUPTPROZEDUR (DI-Task) ***** }
begin
   ml8rt_entry;
   if Parameter.Soft_Cnt=0 then
   begin
      if Parameter.LED_Status = EIN then
      begin                                  { Falls die LED eingeschaltet ist, ... }
         INC (Parameter.Switch_Cnt);         { Anzahl der Umschaltungen um 1 erhhen }
         if Parameter.LED = EXTERN then
            ml8rt_External_LED_Off           { ... LED ausschalten und ... }
         else
            ml8rt_Local_LED_Off;
         Parameter.LED_Status := AUS;        { ... Zustand merken }
      end
      else                                   { Falls die LED ausgeschaltet ist, ... }
      begin
         if Parameter.LED = EXTERN then
            ml8rt_External_LED_On            { ... LED einschalten ... }
         else
            ml8rt_Local_LED_On;
         Parameter.LED_Status := EIN;        { ... Zustand merken }
      end;
      Parameter.Soft_Cnt := Parameter.Soft_Init;
   end
   else
   begin
      Parameter.Soft_Cnt := Parameter.Soft_Cnt - 1;
   end;

   ml8rt_end_of_int(Parameter.TDT.Inter);    { EOI zum Interrupt-Controller senden }
   ml8rt_exit_interrupt;
end;




begin                        { ***** PREPARE ***** }
   PDT.PDT_HEAD.ucType := 1;                    { Programm-Descriptor-Tabellen-Typ
                                                  (im Moment wird nur der Typ 1 untersttzt) }
   PDT.PDT_HEAD.ucSize := sizeof(ML8_PDT_HEAD); { Gre des Vorspanns der Kopftabelle
                                                  (im Moment immer = _PDT_Length = 48) }
   PDT.PDT_HEAD.usGlobalProc := 5;              { Anzahl der Prozeduren
                                                  (in diesem Beispiel Auto_Init, Main_Proc, Start, Stop, ...
                                                  ... Get_Switches) }
   PDT.PDT_HEAD.usProgNo := Programm_Number;    { Programm-Nummer }
   PDT.PDT_HEAD.cVersion := Version;            { Programm-Version }
   PDT.PDT_HEAD.cRevision := Revision;          { Programm-Revision }
   PDT.PDT_HEAD.ucCPU := _486SX;                { CPU-Typ }
   PDT.PDT_HEAD.ucCoProc:= _NOCOPROZ;           { Co-Prozessor-Typ }
   PDT.PDT_HEAD.ucLanguage := _TP70;            { Programmiersprache }
   PDT.PDT_HEAD.ucProgType := _User_Prg;        { Programm-Typ: Anwenderprogramm }
   PDT.PDT_HEAD.usFlags := $0;                  { Flags }
   PDT.PDT_HEAD.usFlags := _DI_Task + _Local_Data + _Local_Parameter + _Fixed_Datasize;
   PDT.PDT_HEAD.usInt := 0;                     { Interrupt-Nummer wird bei der Installierung angegeben }
   PDT.PDT_HEAD.ulDataAdr := 0;                 { Adresse des Datenbereichs }
   PDT.PDT_HEAD.ulDataSize := 0;                { Gre des Datenbereichs }
   PDT.PDT_HEAD.ulDataSizeMin := 0;             { Minimaler Datenbereich }
   PDT.PDT_HEAD.ulDataSizeMax := 0;             { Maximaler Datenbereich }
                                                { Anfangsadresse des Parameterbereichs }
   ml8rt_phys_adr (@Parameter, PDT.PDT_HEAD.ulGlobalParAdr);
   PDT.PDT_HEAD.ulGlobalParAdr := PDT.PDT_HEAD.ulGlobalParAdr + SizeOf(TDT_Type);
                                                { Anzahl der Parameter }
   PDT.PDT_HEAD.usGlobalParSize:= SizeOf(Parameter) - SizeOf(TDT_Type);
   PDT.PDT_HEAD.ulHyperAdr := 0;                { Adresse des Hypertextes (wird nicht benutzt) }
   PDT.PDT_HEAD.usRes1 := 0;
   PDT.PDT_HEAD.ulRes2 := 0;
   PDT.Main := @Main_Proc;                      { Adresse der Hauptprozedur }
   PDT.Auto_Init := @Auto_Init;                 { Adresse der Auto-Init-Prozedur }
   PDT.Start := @Start;                         { Adresse der Start-Prozedur }
   PDT.Stop := @Stop;                           { Adresse der Stop-Prozedur }
   PDT.Get_Switches := @Get_Switches;           { Adresse der Taskfunktion }
   ml8rt_Set_PDT_Adr (PDT);                     { Adresse der PDT im Rckgabe-Record dem Betriebssystem bergeben }
end.
