{****************************************************************************}
{**   berwachungs NI-Task (MODULAR-4/486) fr das Programm M8P0302        **}
{**                            (Blink-LED).                                **}
{**                                                                        **}
{**  Dieses Programm berwacht das Blink-LED Programm. Ab einer Anzahl     **}
{**  von Durchlufen wird das Blinken durch Aufruf der Stop-Prozedur       **}
{**  (Prozedur #3 von M8P0302) gestoppt.                                   **}
{**  Es soll als Beispiel zur Intertaskkomunikation dienen.                **}
{**  Weitere Hinweise entnehmen Sie bitte dem Benutzerhandbuch bzw. dem    **}
{**  Technischen-Handbuch.                                                 **}
{**                                                                        **}
{**  Installationshinweise:                                                **}
{**                                                                        **}
{**  Programmnummer  : 303h                                                **}
{**  Tasknummer      : beliebig                                            **}
{**  Interruptquelle : - (wird nicht verwendet, da NI-Task)                **}
{**  Datenbereich    : 0                                                   **}
{**  Flags           : 0x0980 (nicht aktiviert, Auto-Init aufrufen)        **}
{**                                                                        **}
{**                                                                        **}
{**                    Fr TURBO-PASCAL (Version 7.0)                      **}
{**                                                                        **}
{**                             Version 1.F.                               **}
{**                                                                        **}
{**                   Erstellungsdatum : 26.01.93                          **}
{**                    letzte nderung : 06.06.2000(HK)                    **}
{**                             Status : freigegeben                       **}
{**                              Autor : M.H.                              **}
{**                                                                        **}
{**        (c) 1992, 1993, 1994 SORCUS Computer GmbH Heidelberg            **}
{**                                                                        **}
{****************************************************************************}
{$N+}                                     { Coprozessor ein }
{$E+}                                     { keine Emulation }
{$R-}                                     { kein Range-Check }
{$S-}                                     { kein Stack-Checking }
{$I-}                                     { kein IO-Checking }
{$V-}                                     { Keine strikte Stringbergabe }
{$L+}                                     { Local-Symbols fr Turbo-Debugger }
{$F+}                                     { FAR Aufrufe erzwingen }
{$M 1024, 0, 0}                           { Stack und Heap auf Minimum }

uses ml8rtbib;                            { Die ML8 On-Board Bibliothek einbinden }

Const                        { ***** ALLGEMEINE ANGABEN ***** }
   Programm_Number      : Word = $303;    { Programmnummer = 303h }
   Version              : Char = '1';     { Version des Programms }
   Revision             : Char = 'F';     { Revision des Programms }

			     { ***** Die verschiedenen Statusmglichkeiten des Programms ***** }
   READY                = 0;              { Bereit }
   RUNNING              = 2;              { Programm luft }
   ERROR                = 3;              { Fehler aufgetreten (z.B. Blink-Task nicht installiert) }
   STOPPED              = 4;              { Programmlauf wurd abgebrochen }

                             { ***** Informationen ber die Blink-Task ***** }
   SWITCH_FUNC          = 4;              { Taskfunktion der Blink-Task, die die Anzahl der Blink-Sequenzen bergibt }
   STOP_PROC_NR         = 3;              { Nummer der Stop-Prozedur der Blink-Task) }

			     { ***** Aufbau der Programm-Descriptor-Tabelle (PDT) ***** }
Type
   PDT_Type             = Record
			     PDT_HEAD      : ML8_PDT_HEAD;  { PDT-Header }
			     Main          : Pointer;       { Adresse der Haupt-Prozedur (Task-Prozedur) }
			     Auto_Init     : Pointer;       { Adresse der Auto-Init-Prozedur }
			     Start         : Pointer;       { Adresse der Start-Prozedur }
                             Stop          : Pointer;       { Adresse der Stop-Prozedur }
			  end;


			     { ***** DEKLARATION DER GLOBALEN PROGRAMMDATEN ***** }
VAR
   PDT                  : PDT_Type;                    { Deklaration der Kopftabelle }

			     { ***** Parameterbereich des Programms deklarieren. ***** }
			     { Auf diese Daten kann spter ber das Betriebssystem zugegriffen werden. }
   Parameter            : Record
			     TDT       : TDT_Type;     { Platz fr die TDT reservieren }
			     Status    : Byte;         { Status des Programms }
			     Watch_Task: Word;         { Task-Nummer, der Blink-Task }
			     Switches  : Word;         { Anzahl der Blinksequenzen, bis zum Stoppen der Blink-Task }
			  end;


Procedure Auto_Init;         { ***** AUTO-INITIALISIERUNG }
begin
   ml8rt_entry;                           { Register retten und vorbereiten }
   Parameter.Status := READY;             { Status des Programms auf bereit }
   Parameter.Watch_Task := 0;             { Defaultmig Task-0 einstellen }
   Parameter.Switches := 10;              { Defaultmig 10 Blink-Sequenzen einstellen, bis Blinken gestoppt wird }
   ml8rt_exit;                            { Register wieder restaurieren }
end;


Procedure Start;             { ***** Start-Prozedur ***** }
begin
   ml8rt_entry;
   Parameter.Status := RUNNING;           { Status auf "Programm luft" }
   ml8rt_wakeup_task (Parameter.TDT.Task);{ Task aktivieren bzw. starten }
   ml8rt_exit;
end;


Procedure Stop;              { ***** Stop-Prozedur ***** }
begin
   ml8rt_entry;
   ml8rt_sleep_task (Parameter.TDT.Task); { Task deaktivieren (stoppen) }
   Parameter.Status := STOPPED;
   ml8rt_exit;
end;


Procedure Main_Proc;         { ***** HAUPTPROZEDUR (NI-Task) ***** }
VAR                                       { berwachung der Blink-Task }
   Switch_Cnt           : Word;           { Anzahl der schon gemachten Umschaltungen }
   insize, inptr, indata,                 { Normaler Weise Daten, die an die Funktion bergeben werden }
   outsize, error       : Word;
begin
   ml8rt_entry;
                                          { Der folgende Funktionsaufruf liefert die Anzahl der Umschaltungen }
                                          { (siehe M8P0302). Es werden nur Daten von der Funktion zurckgeliefert. }
   ml8rt_call_func (parameter.Watch_Task, SWITCH_FUNC,
            insize, indata, SizeOf(Switch_Cnt), outsize, Switch_cnt, error);

   if (error<>0) OR (outsize<>SizeOf(Switch_Cnt)) then
   begin                                  { Falls ein Fehler auftrat, Programm abbrechen }
      ml8rt_sleep_task (Parameter.TDT.Task); { Task deaktivieren }
      Parameter.Status := ERROR;
   end
   else
   begin
      if Switch_Cnt >= Parameter.Switches then
      begin                                  { Falls die Anzahl erreicht ist, Blinken stoppen }
         ml8rt_call_proc (Parameter.Watch_Task, STOP_PROC_NR);
      end;                                   { Hinweis: Die berwachung der Task luft weiter }
   end;

   ml8rt_Exit;
end;




begin                        { ***** PREPARE ***** }
   PDT.PDT_HEAD.ucType := 1;              { Programm-Descriptor-Tabellen-Typ
                                            (im Moment wird nur der Typ 1 untersttzt) }
   PDT.PDT_HEAD.ucSize := sizeof(ML8_PDT_HEAD);    { Gre des Vorspanns der Kopftabelle
                                          (im Moment immer = _PDT_Length = 48) }
   PDT.PDT_HEAD.usGlobalProc := 4;        { Anzahl der Prozeduren
                                            (in diesem Beispiel nur Auto_Init und Main_Proc) }
   PDT.PDT_HEAD.usProgNo := Programm_Number;{ Programm-Nummer }
   PDT.PDT_HEAD.cVersion := Version;      { Programm-Version }
   PDT.PDT_HEAD.cRevision := Revision;    { Programm-Revision }
   PDT.PDT_HEAD.ucCPU:= _486SX;           { CPU-Typ }
   PDT.PDT_HEAD.ucCoProc:= _NOCOPROZ;     { Co-Prozessor-Typ }
   PDT.PDT_HEAD.ucLanguage := _TP70;      { Programmiersprache }
   PDT.PDT_HEAD.ucProgType := _User_Prg;  { Programm-Typ: Anwenderprogramm }
   PDT.PDT_HEAD.usFlags := $0;            { Flags }
   PDT.PDT_HEAD.usFlags := _NI_Task + _PDT_Info_Enable + _Local_Data + _Local_Parameter + _Fixed_Datasize;
   PDT.PDT_HEAD.usInt := 0;               { Interrupt-Nummer fr die das Programm gedacht ist
                                            (ungltig, da NI-Task) }
   PDT.PDT_HEAD.ulDataAdr := 0;           { Adresse des Datenbereichs }
   PDT.PDT_HEAD.ulDataSize := 0;          { Gre des Datenbereichs }
   PDT.PDT_HEAD.ulDataSizeMin := 0;       { Minimaler Datenbereich }
   PDT.PDT_HEAD.ulDataSizeMax := 0;       { Maximaler Datenbereich }
					  { Anfangsadresse des Parameterbereichs }
   ml8rt_phys_adr (@Parameter, PDT.PDT_HEAD.ulGlobalParAdr);
   PDT.PDT_HEAD.ulGlobalParAdr := PDT.PDT_HEAD.ulGlobalParAdr + SizeOf(TDT_Type);
					  { Anzahl der Parameter }
   PDT.PDT_HEAD.usGlobalParSize := SizeOf(Parameter) - SizeOf(TDT_Type);
   PDT.PDT_HEAD.ulHyperAdr := 0;          { Adresse des Hypertextes (wird nicht benutzt) }
   PDT.PDT_HEAD.usRes1 := 0;
   PDT.PDT_HEAD.ulRes2 := 0;
   PDT.Main := @Main_Proc;                { Adresse der Hauptprozedur }
   PDT.Auto_Init := @Auto_Init;           { Adresse der Auto-Init-Prozedur }
   PDT.Start := @Start;                   { Adresse der Start-Prozedur }
   PDT.Stop := @Stop;                     { Adresse der Stop-Prozedur }
   ml8rt_Set_PDT_Adr (PDT);               { Adresse der PDT im Rckgabe-Record dem Betriebssystem bergeben }
end.
