{****************************************************************************}
{**                Blink-LED als NI-Task (MODULAR-4/486)                   **}
{**                       mit PC-SRQ-Auslsung                             **}
{**                                                                        **}
{**  Dieses Programm lt entweder die On-Board- oder die Externe-LED      **}
{**  blinken. Optional kann bei jeder LED- Einschaltung ein Service-       **}
{**  Request zum PC gesendet werden.                                       **}
{**                                                                        **}
{**  Installationshinweise:                                                **}
{**                                                                        **}
{**  Programmnummer  : $0307						   **}
{**  Tasknummer      : beliebig                                            **}
{**  Interruptquelle : 1 (wird nicht verwendet, da NI-Task)                **}
{**  Datenbereich    : 0                                                   **}
{**  Flags           : 0x1980                                              **}
{**                                                                        **}
{**                                                                        **}
{**                    Fr TURBO-PASCAL (ab Version 6.0)                   **}
{**                                                                        **}
{**                             Version 1.D.                               **}
{**                                                                        **}
{**                   Erstellungsdatum : 03.02.93                          **}
{**                    letzte nderung : 06.06.2000 (HK)                   **}
{**                             PDT auf neustem Stand                      **}
{**                             Status : freigegeben                       **}
{**                              Autor : M.H./B.F.                         **}
{**                                                                        **}
{**            (c) 1992, 1993 SORCUS Computer GmbH Heidelberg              **}
{**                                                                        **}
{****************************************************************************}
{$N+}                                     { Coprozessor ein }
{$E-}                                     { keine Emulation }
{$R-}                                     { kein Range-Check }
{$S-}                                     { kein Stack-Checking }
{$I-}                                     { kein IO-Checking }
{$V-}                                     { Keine strikte Stringbergabe }
{$M 1024, 0, 0}                           { Stack und Heap auf Minimum }

uses ml8rtbib;                            { Die ML8 On-Board Bibliothek einbinden }

Const                        { ***** ALLGEMEINE ANGABEN ***** }
   Programm_Number      : WORD = $0307;   { Programmnummer = 307H }
   Version              : Char = '1';     { Version des Programms }
   Revision             : Char = 'D';     { Revision des Programms }

			     { ***** Die verschiedenen Statusmglichkeiten des Programms ***** }
   Ready                = 0;              { Bereit }
   Start                = 1;              { Programm soll starten }
   Running              = 2;              { Programm luft }
   Error                = 3;              { Falsche LED angewhlt }

			     { ***** Zustnde der LED ***** }
   EIN                  = 1;
   AUS                  = 0;

			     { ***** Benutzte LED ***** }
   INTERN               = 1;              { Interne LED }
   EXTERN               = 2;              { Externe LED }

			     { ***** Aufbau der Programm-Descriptor-Tabelle (PDT) ***** }
Type
   PDT_Type             = Record
			     PDT_HEAD : ML8_PDT_HEAD;  { PDT-Header }

			     Main          : Pointer;  { Adresse der Haupt-Prozedur (Task-Prozedur) }
			     Auto_Init     : Pointer;  { Adresse der Auto-Init-Prozedur }
			  end;


			     { ***** DEKLARATION DER GLOBALEN PROGRAMMDATEN ***** }
VAR
   PDT                  : PDT_Type;                    { Deklaration der Kopftabelle }

			     { ***** Parameterbereich des Programms deklarieren. ***** }
			     { Auf diese Daten kann spter ber das Betriebssystem zugegriffen werden. }
   Parameter            : Record
			     TDT         : TDT_Type;     { Platz fr die TDT reservieren }
			     Status      : Byte;         { Status des Programms }
			     Blink_Rate  : Word;         { Blinkrate }
			     LED_Status  : Byte;         { Zustand der LED }
			     LED         : Byte;         { Welche LED 1=Intern, 2=Extern }
                             Countdown   : WORD;         { Anzahl On-Phasen, 0 = keine, $FFFF = unendlich}
                             With_SRQ    : BOOLEAN;      { PC-Interrupt auslsen, 1=TRUE,0=FALSE}
                             SRQ_Sign    : WORD;         { Request-Zeichen }
                             SRQ_Pending : BOOLEAN       { Request wartet }
			  end;

   Pause                : Word;           { Ein Zhler, der die Durchlufe des Programms zhlt }



Procedure Auto_Init;         { ***** AUTO-INITIALISIERUNG }
begin
   ml8rt_entry;                           { Register retten und vorbereiten }
   Parameter.Status := Ready;             { Status des Programms auf bereit }
   Parameter.Blink_Rate := 3000;          { Blinkrate defaultmig einstellen }
   Parameter.LED_Status := AUS;           { Status der LED auf "AUS" }
   Parameter.LED := EXTERN;               { Standartmig die externe LED benutzen }

   Parameter.Countdown:=$FFFF;            { keine begrenzte Anzahl Blinkphasen }
   Parameter.With_SRQ:=FALSE;             { Betrieb ohne SRQ }
   Parameter.SRQ_Sign:=$0080;             {Voreingestellte SRQ-Zeichen}
                                          {Lowbyte mu zwischen $80 und $C0 liegen }
   Parameter.SRQ_Pending:=FALSE;          { Request wartet }

   ml8rt_External_LED_Off;                { Externe LED ausschalten }
   ml8rt_Local_LED_Off;                   { On-Board LED ausschalten }
   Pause := 0;                            { Durchlaufzhler auf Null }
   ml8rt_exit;                            { Register wieder restaurieren }
end;


Procedure Main_Proc;         { ***** HAUPTPROZEDUR (NI-Task) ***** }
begin
   ml8rt_entry;
   if Parameter.Status=Start then         { Falls Programm gestartet wurde, dann ... }
   begin
      if (Parameter.LED<>INTERN) AND (Parameter.LED<>EXTERN) then
	 Parameter.Status := Error        { Erstmal die angewhlte LED checken }
      else
	 Parameter.Status := Running;
   end;

   if (Parameter.Status=Running)          { Falls Status=Aktiv, dann Programm ablaufen lassen }
      and (Parameter.CountDown>0) then    { und kein Stop durch Count-Down }
   begin
      Pause := Pause + 1;                 { Durchlaufzhler um 1 erhhen }
      if Parameter.SRQ_Pending then begin
        if (ml8rt_send_host_srq(Parameter.srq_Sign)=0) then
          Parameter.SRQ_Pending:=FALSE;
      end;
      if Pause>=Parameter.Blink_Rate then { Falls die Pause (Anzahl Aufrufe der Main-Proc) den angegebenen Wert erreicht hat, }
      begin                               { ... die LED umschalten }
	 if Parameter.LED_Status = EIN then
	 begin                            { Falls die LED eingeschaltet ist, ... }
	    if Parameter.LED = EXTERN then
	       ml8rt_External_LED_Off     { ... LED ausschalten und ... }
	    else
	       ml8rt_Local_LED_Off;
	    Parameter.LED_Status := AUS;  { ... Zustand merken }
	 end
	 else                             { Falls die LED ausgeschaltet ist, ... }
	 begin
	    if Parameter.LED = EXTERN then
	       ml8rt_External_LED_On      { ... LED einschalten ... }
	    else
	       ml8rt_Local_LED_On;
	    Parameter.LED_Status := EIN;  { ... Zustand merken }
            with Parameter do begin
	      if CountDown<>$FFFF then dec(CountDown);
              if WITH_SRQ then begin {Request absetzen}
                SRQ_Pending:=TRUE;
                if (ml8rt_send_host_srq(srq_Sign)=0) then begin
                  SRQ_Pending:=FALSE;
                end;
              end;
            end;
	 end;
	 Pause := 0;                      { Durchlaufzhler wieder auf Null setzen }
      end;
   end;
   ml8rt_Exit;
end;



begin                        { ***** PREPARE ***** }
   PDT.PDT_HEAD.ucType := 1;                    { Programm-Descriptor-Tabellen-Typ (im Moment wird nur der Typ 1 untersttzt) }
   PDT.PDT_HEAD.ucSize := sizeof(ML8_PDT_HEAD); { Gre des Vorspanns der Kopftabelle (im Moment immer = _PDT_Length = 48) }
   PDT.PDT_HEAD.usGlobalProc := 2;              { Anzahl der Prozeduren (in diesem Beispiel nur Auto_Init und Main_Proc) }
   PDT.PDT_HEAD.usProgNo := Programm_Number;    { Programm-Nummer }
   PDT.PDT_HEAD.cVersion := Version;            { Programm-Version }
   PDT.PDT_HEAD.cRevision := Revision;          { Programm-Revision }
   PDT.PDT_HEAD.ucCPU := _486DX;                { CPU-Typ }
   PDT.PDT_HEAD.ucCoProc := _487DX;             { Co-Prozessor-Typ }
   PDT.PDT_HEAD.ucLanguage := _TP60;            { Programmiersprache }
   PDT.PDT_HEAD.ucProgType := _User_Prg;        { Programm-Typ: Anwenderprogramm }
   PDT.PDT_HEAD.usFlags := $0;                  { Flags }
   PDT.PDT_HEAD.usFlags := _NI_Task + _PDT_Info_Enable + _Local_Parameter;
   PDT.PDT_HEAD.usInt := 0;                     { Interrupt-Nummer fr die das Programm gedacht ist (ungltig, da NI-Task) }
   PDT.PDT_HEAD.ulDataAdr := 0;                 { Adresse des Datenbereichs }
   PDT.PDT_HEAD.ulDataSize := $1000;            { Gre des Datenbereichs }
   PDT.PDT_HEAD.ulDataSizeMin := 0;             { Minimaler Datenbereich }
   PDT.PDT_HEAD.ulDataSizeMax := $10000;        { Maximaler Datenbereich }
					        { Anfangsadresse des Parameterbereichs }
   ml8rt_phys_adr (@Parameter, PDT.PDT_HEAD.ulGlobalParAdr);
   PDT.PDT_HEAD.ulGlobalParAdr := PDT.PDT_HEAD.ulGlobalParAdr + SizeOf(TDT_Type);
					        { Anzahl der Parameter }
   PDT.PDT_HEAD.usGlobalParSize := SizeOf(Parameter) - SizeOf(TDT_Type);
   PDT.PDT_HEAD.ulHyperAdr := 0;                { Adresse des Hypertextes (wird nicht benutzt) }
   PDT.PDT_HEAD.usRes1 := 0;
   PDT.PDT_HEAD.ulRes2 := 0;
   PDT.Main := @Main_Proc;                      { Adresse der Hauptprozedur }
   PDT.Auto_Init := @Auto_Init;                 { Adresse der Auto-Init-Prozedur }
   ml8rt_Set_PDT_Adr (PDT);                     { Adresse der PDT im Rckgabe-Record dem Betriebssystem bergeben }
end.
