{****************************************************************************}
{**                  Beispiel-Echtzeitprogramm fr                         **}
{**                Analogdatenerfassung mit M-AD16-8                       **}
{**                           Version 1.D.                                 **}
{**                                                                        **}
{**                 Erstellungsdatum : 22.12.93                            **}
{**                  Letzte nderung : 19.05.00 (HK)                       **}
{**                    ml8rt_end_of_int statt Port-Befehle                 **}
{**                    Statt Pointer Bibliotheksfunktionen                 **}
{**                           PDT auf neustem Stand                        **}
{**                           Status : freigegeben                         **}
{**                            Autor : G. Rudolf                           **}
{**                                    A. Horst                            **}
{**                                                                        **}
{**  Hinweise zur Installierung:                                           **}
{**                                                                        **}
{**  Tasknummer      : 16  (10H)                                           **}
{**  Programmnummer  : 773 (305H)                                          **}
{**  Interruptquelle : Timer-A = 91H                                       **}
{**  Datenbereich    : beliebig (wird beim Installieren festgelegt)        **}
{**  Flags           : 1982H                                               **}
{**                                                                        **}
{**             (c) 1993-96 SORCUS Computer GmbH  Heidelberg               **}
{**                                                                        **}
{****************************************************************************}
{** History : 04.12.1996 : In der main_task wurden die I/O-Zugriffe nicht  **}
{**           gem der Modulbeschreibung des M-AD16-8 verwendet.          **}
{****************************************************************************}


{$N-}                                     { kein Coprozessor erforderlich }
{$E-}                                     { keine Emulation }
{$R-}                                     { kein Range-Check }
{$S-}                                     { kein Stack-Checking }
{$I-}                                     { kein IO-Checking }
{$V-}                                     { Keine strikte Stringbergabe }
{$F+}                                     { FAR-Calls erzeugen }
{$M 1024, 0, 0}                           { Stack und Heap auf Minimum }

uses ml8rtbib;                            { Die ML8 On-Board Bibliothek einbinden }

Const                        { ***** ALLGEMEINE ANGABEN ***** }
   Programm_Number      : word = $30E;    { Programmnummer = 30EH }
   Version              : Char = '1';     { Version des Programms }
   Revision             : Char = 'D';     { Revision des Programms }

                             { ***** Die verschiedenen Statusmglichkeiten des Programms ***** }
   READY                = 0;              { Bereit }
   RUNNING              = 2;              { Messung luft }
   FINISHED             = 3;              { Messung beendet}
   STOPPED              = 4;              { Programm abgebrochen }
   ERROR                = $80;            { Fehler aufgetreten   }
   DATA_ERROR           = $81;            { Fehler: Datenbereich zu klein}
   MODUL_ERROR          = $82;            { Fehler: illegales Modul gewhlt}
   CHANNEL_ERROR        = $83;            { Fehler: illegale Kanalzahl}

   S_H_TIME	        = 10;		  { Wartezeit zwischen Convert = 0 und 1 }

                             { ***** Aufbau der Programm-Descriptor-Tabelle (PDT) ***** }
Type
   PDT_Type             = Record
                             PDT_HEAD      : ML8_PDT_HEAD;  { PDT-Header }

                             Main          : Pointer;       { Adresse der Haupt-Prozedur (Task-Prozedur) }
                             Auto_Init     : Pointer;       { Adresse der Auto-Init-Prozedur }
                             Start         : Pointer;       { Adresse der Start-Prozedur }
                             Stop          : Pointer;       { Adresse der Stop-Prozedur }
                          end;

                             { ***** DEKLARATION DER GLOBALEN PROGRAMMDATEN ***** }
VAR
   PDT                  : PDT_Type;                    { Deklaration der Kopftabelle }

                             { ***** Parameterbereich des Programms deklarieren. ***** }
                             { Auf diese Daten kann spter ber das Betriebssystem zugegriffen werden. }

   Parameter            : Record
                             TDT       : TDT_Type;     { Platz fr die TDT reservieren }
                             Status    : Byte;         { Status des Programms }
                             AbtastRate: WORD;         { * 0.4 s = Zeit zwischen zwei Messwerten}
                             Modul     : BYTE;         { Modulsteckplatz des M-AD16-3}
                             AnzKan    : Byte;         { Anzahl zu messender Kanle}
                             AnzMess   : WORD;         { Anzahl der Messungen pro Kanal}
                          end;

   CntMess     : WORD;            {Zhler fr Messungen}
   mba         : WORD;            {Modulbasisadresse fr I/O-Zugriffe}

   io_kanal      : WORD;          { I/O-Adresse fr die Kanalwahl }
   io_convert_0  : WORD;          { I/O-Adresse fr Start Convert = 0 }
   io_convert_1  : WORD;          { I/O-Adresse fr Start Convert = 1 }
   io_get_status : WORD;          { I/O-Adresse fr Status Wandung anfordern }
   io_ad_status  : WORD;          { I/O-Adresse fr Status des AD-Wandlers }
   io_low_byte   : WORD;          { I/O-Adresse zum Lesen der 8 Bit Low Byte}
   io_high_byte  : WORD;          { I/O-Adresse zum Lesen der 8 Bit high Byte}
   io_power_down : WORD;          { I/O-Adresse fr Power Down }

Procedure Auto_Init;         { ***** AUTO-INITIALISIERUNG }
begin
   ml8rt_entry;                           { Register retten und vorbereiten }
   ml8rt_local_led_off;                   { on-board LED auschalten}
   Parameter.Status := READY;             { Status des Programms auf bereit }
   Parameter.Abtastrate := 2500;          { 2500 x 0,4 s = 1 ms oder 1 kHz}
   Parameter.Modul := 1;                  { Modulsteckplatz 1}
   Parameter.AnzKan := 1;                 { ein Kanal}
   Parameter.AnzMess := 1000;             { tausend Messpunkte}
   ml8rt_exit;                            { Register wieder restaurieren }
end;


Procedure Start;             { ***** Programm starten ***** }
VAR
   datalength : LongInt;
   w          : WORD;

begin
   ml8rt_entry;

   {-------------------- Parameterprfungen ----------------------}

   datalength := parameter.tdt.DataLast - parameter.tdt.DataFirst;   {Gre des reservierten Datenbereichs}
   IF datalength < (parameter.anzkan * 2 * parameter.anzmess) THEN
     Parameter.Status := DATA_ERROR;

   IF NOT(parameter.modul IN [1..10]) THEN        {zulssiger Modulsteckplatz?}
     Parameter.Status := MODUL_ERROR;

   IF NOT(parameter.anzkan IN [1..16]) THEN      {zulssige Kanalzahl?}
     Parameter.Status := CHANNEL_ERROR;

   IF (Parameter.Status AND ERROR)=0 THEN        {Kein Parameterfehler?}
     BEGIN
       ml8rt_local_led_on;                       {LED einschalten; zeigt, da Messung luft}
       ml8rt_reset_w_pointer(parameter.tdt.task);{Datenzeiger auf Anfang des Datenbereichs setzen}

       IF (parameter.modul <= 4) THEN            { Modulbasisadresse berechnen }
	 mba := $100 * parameter.modul + $300    { Auf Basiskarte }
       ELSE
	 mba := $20 * (parameter.modul-4) + $400;{ Auf Extender }

       io_kanal	    	:= mba + $10;	         { I/O-Adresse fr die Kanalwahl }
       io_convert_0	:= mba + $00;            { I/O-Adresse fr Start Convert = 0 }
       io_convert_1	:= mba + $01;            { I/O-Adresse fr Start Convert = 1 }
       io_get_status	:= mba + $07;            { I/O-Adresse fr Status Wandung anfordern }
       io_ad_status	:= mba + $08;            { I/O-Adresse fr Status des AD-Wandlers }
       io_low_byte      := mba + $06;	         { I/O-Adresse zum Lesen der 8 Bit Low Byte}
       io_high_byte	:= mba + $05;            { I/O-Adresse zum Lesen der 8 Bit high Byte}
       io_power_down	:= mba + $17;            { I/O-Adresse fr Power Down }
       cntMess          := 0;                    { Messungenzhler auf Null setzen }
       port[io_kanal]   := 0;                    { Kanal-0 fr erste Messung einstellen }

       {---------------------- Programmieren des Timers ---------------------}
       Parameter.Status := RUNNING;              {Statusparameter einstellen}
       ml8rt_set_timer(TIMER_A,Parameter.AbtastRate,INT_MODE);   {Timer-A einstellen und starten }
       ml8rt_clear_int(IRQ_TIMER_A);             {evtl. gespeicherten Timerinterrupt lschen}
       port[io_kanal] := 0;                      {Anfangskanal einstellen und }
                                                 {Abtasttrigger aktivieren    }
       ml8rt_unmask_int (IRQ_TIMER_A);           {Timer-A-Interrupt demaskieren }
     END;
   ml8rt_exit;
end;


Procedure Stop;              { ***** Programmlauf abbrechen ***** }
begin
   ml8rt_entry;
   ml8rt_local_led_off;                   {LED ausschalten; zeigt, da Messung gestoppt}
   ml8rt_mask_int (IRQ_TIMER_A);          {Timer-Interrupt maskieren }
   Parameter.Status := STOPPED;           {Statusparameter setzen }
   ml8rt_exit;
end;


Procedure Main_Proc;         { ***** HAUPTPROZEDUR (DI-Task) ***** }
VAR kanal      : BYTE;
    b          : BYTE;
    i          : WORD;
    abort_time : WORD;
    low_byte   : BYTE;
    high_byte  : BYTE;

begin
   ml8rt_entry;

   port[io_convert_0] := b;	  	  { Sample/Hold auf Hold }
   FOR i := 0 TO S_H_TIME DO;	  	  { ca. 1 bis 3 us warten, siehe Beschreibung }
   port[io_convert_1] := b;	  	  { AD-Wandlung des ersten Kanals starten }

   kanal := 0;                            { (Kanal-0 !!) }
   port[io_kanal+1] := b;  	          { Nchsten Kanal einstellen }
   REPEAT
      abort_time := 0;
      b := port[io_get_status];	  	  { Status der Wandlung anfordern }
      WHILE ( ((port[io_ad_status] AND $80) = 0) AND
              (abort_time < 100000)) DO
      BEGIN
	 abort_time := abort_time + 1;    { Solange warten, bis Status OK ist. }
	 b := port[io_get_status]; 	  { Status der Wandlung anfordern }
      END;

      low_byte  := port[io_low_byte];     { Ergebnis auf Modul zwischenspeichern }
					  { und Low Byte lesen }
      high_byte := port[io_high_byte];    { High Byte lesen }
      ml8rt_write_data_word(parameter.tdt.task,low_byte OR (high_byte SHL 8)); {Ergebnis abholen}

      FOR i := 0 TO S_H_TIME DO
        BEGIN
          port[$3f] := b;
        END;

      port[io_convert_0] := b;	  	  { Sample/Hold auf Hold }
      FOR i := 0 TO S_H_TIME DO;	  { ca. 1 bis 3 us warten, siehe Beschreibung }
      port[io_convert_1] := b;	  	  { AD-Wandlung starten }

      kanal := kanal + 1;	          { Kanal weiterzhlen }
      IF kanal < 7 THEN
	 port[io_kanal+kanal+1] := b;     { Nchsten Kanal einstellen }
   UNTIL kanal >= parameter.anzkan;       { Schleife, bis alle Kanle gewandelt sind}

   INC(CntMess);                          { Messungenzhler weiterzhlen }
   IF CntMess>Parameter.AnzMess THEN      { wenn alle Messungen durchgefhrt sind }
     BEGIN                                { anhalten }
       ml8rt_local_led_off;               { LED ausschalten }
       ml8rt_mask_int(IRQ_TIMER_A);       { Interrupt Timer-A maskieren }
       Parameter.Status := FINISHED;      { Statusparameter setzen }
     END;

   port[io_power_down] := b;              { Power Down }
   port[io_kanal] := b;                   { Kanal-0 fr die nchste Messung schon mal anwhlen }

   ml8rt_end_of_int(IRQ_TIMER_A);         {EOI zum Interrupt-Controller senden }
   ml8rt_exit_interrupt;
end;



begin                        { ***** PREPARE ***** }
   PDT.PDT_HEAD.ucType := 1;                    {Programm-Descriptor-Tabellen-Typ (z.Zt. nur Typ 1 untersttzt) }
   PDT.PDT_HEAD.ucSize := sizeof(ML8_PDT_HEAD); {Gre des Vorspanns der Kopftabelle (im Moment immer =  = 48) }
   PDT.PDT_HEAD.usGlobalProc := 4;              {Anzahl der Prozeduren (hier: Auto_Init, Main_Proc, Start, Stop) }
   PDT.PDT_HEAD.usProgNo := Programm_Number;    {Programm-Nummer }
   PDT.PDT_HEAD.cVersion := Version;            {Programm-Version }
   PDT.PDT_HEAD.cRevision := Revision;          {Programm-Revision }
   PDT.PDT_HEAD.ucCPU := _486DX;                {CPU-Typ }
   PDT.PDT_HEAD.ucCoProc := _NOCOPROZ;          {Co-Prozessor-Typ }
   PDT.PDT_HEAD.ucLanguage := _TP70;            {Programmiersprache }
   PDT.PDT_HEAD.ucProgType := _User_Prg;        {Programm-Typ: Anwenderprogramm }
   PDT.PDT_HEAD.usFlags := $0;                  {Flags }
   PDT.PDT_HEAD.usFlags := _DI_Task + _PDT_INFO_ENABLE + _Local_Parameter;

   PDT.PDT_HEAD.usInt := IRQ_TIMER_A;           {Interrupt-Nummer fr die das Programm gedacht ist }
   PDT.PDT_HEAD.ulDataAdr := 0;                 {Adresse des Datenbereichs }
   PDT.PDT_HEAD.ulDataSize := 100000;        {Gre des Datenbereichs }
   PDT.PDT_HEAD.ulDataSizeMin := 32;            {Minimaler Datenbereich }
   PDT.PDT_HEAD.ulDataSizeMax := 800000;        {Maximaler Datenbereich }
                                                {Anfangsadresse des Parameterbereichs }
   ml8rt_phys_adr(@Parameter, PDT.PDT_HEAD.ulGlobalParAdr);
   PDT.PDT_HEAD.ulGlobalParAdr := PDT.PDT_HEAD.ulGlobalParAdr + SizeOF(TDT_Type);
                                                {Anzahl der Parameter }
   PDT.PDT_HEAD.usGlobalParSize := SizeOf(Parameter) - SizeOF(TDT_Type);
   PDT.PDT_HEAD.ulHyperAdr := 0;                {Adresse des Hypertextes (wird nicht benutzt) }
   PDT.PDT_HEAD.usRes1 := 0;
   PDT.PDT_HEAD.ulRes2 := 0;
   PDT.Main := @Main_Proc;                      {Adresse der Interruptprozedur }
   PDT.Auto_Init := @Auto_Init;                 {Adresse der Auto-Init-Prozedur }
   PDT.Start := @Start;                         {Adresse der Start-Prozedur }
   PDT.Stop := @Stop;                           {Adresse der Stop-Prozedur }
   ml8rt_Set_PDT_Adr(PDT);                      {Adresse der PDT im Rckgaberecord dem Betriebssystem bergeben }
end.
