{****************************************************************************}
{**           Beispiel fuer die Verwendung eines Funktionsaufrufs          **}
{**                                                                        **}
{** Das Programm enthlt nur eine wichtige Prozedur. Sie dient zur Bildung **}
{** des Mittelwertes aus bis zu 256 Bytes, die vom PC oder von einer       **}
{** anderen Task per Funktionsaufruf uebergeben werden.                    **}
{** Sehen Sie sich dazu auch das PC-Programm "MITT0311.PAS" an.            **}
{**                                                                        **}
{** Das Beispiel dient nur zur Veranschaulichung des Funktionsaufrufs und  **}
{** hat darueber hinaus keine praktische Bedeutung.                        **}
{**                                                                        **}
{**  Installationshinweise:                                                **}
{**                                                                        **}
{**  Programmnummer  : 311h						                              **}
{**  Tasknummer      : beliebig                                            **}
{**  Interruptquelle : 0 (wird nicht verwendet, da NI-Task)                **}
{**  Datenbereich    : 0                                                   **}
{**  Flags           : 0x0180 (nicht aktiviert, kein Auto-Init)            **}
{**                                                                        **}
{**                                                                        **}
{**                             Version 1.D                                **}
{**                                                                        **}
{**                   Erstellungsdatum : 8.6.94                            **}
{**                    letzte nderung : 06.06.2000 (HK)                   **}
{**                             PDT auf neustem Stand                      **}
{**                             Status : freigegeben                       **}
{**                              Autor : H.H.                              **}
{**                                                                        **}
{**            (c) 1994..1996 SORCUS Computer GmbH Heidelberg              **}
{**                                                                        **}
{****************************************************************************}
{$N+}                                     { Coprozessor aus }
{$E+}                                     { keine Emulation }
{$R-}                                     { kein Range-Check }
{$S-}                                     { kein Stack-Checking }
{$I-}                                     { kein IO-Checking }
{$V-}                                     { Keine strikte Stringuebergabe }
{$L+}                                     { Local-Symbols fuer Turbo-Debugger }
{$M 1024, 0, 0}                           { Stack und Heap auf Minimum }

uses ml8rtbib;                            { Die ML8 On-Board Bibliothek einbinden }

{ ***** ALLGEMEINE ANGABEN ***** }
Const
   Programm_Number      : Word = $311;    { Programmnummer = 311h }
   Version              : Char = '1';     { Version des Programms }
   Revision             : Char = 'D';     { Revision des Programms }


{ ***** Aufbau der Programm-Descriptor-Tabelle (PDT) ***** }
Type
   PDT_Type = Record
		    PDT_HEAD : ML8_PDT_HEAD;  { PDT-Header }

                    Main          : Pointer;  { Adresse der Haupt-Prozedur (Task-Prozedur) }
		    Auto_Init     : Pointer;  { Adresse der Auto-Init-Prozedur }
                    Mittel        : Pointer;  { Adresse der Mittelwert Funktion }
                    end;


   Parameter_Type = Record
                          TDT               : TDT_Type;       { Platz fuer die Task-Descriptor-Tabelle }
                          end;

VAR
   PDT                  : PDT_Type;
   Parameter            : Parameter_Type;
   TDT                  : TDT_Type;

(* Die Prozeduren Auto_init und Main_proc werden in diesem Beispiel nicht *)
(* benutzt, sie werden nur deklariert, damit das System nicht abstuerzt,   *)
(* falls Sie versehentlich (z.B. beim Installieren) aufgerufen werden     *)

Procedure Auto_Init;         { ***** AUTO-INITIALISIERUNG }
begin
  ml8rt_entry;
  ml8rt_exit;
end;

Procedure Main_Proc;         { ***** HAUPTPROZEDUR (NI-Task) ***** }
begin
  ml8rt_entry;
  ml8rt_exit;
end;

{* ---- Prozedur Mittel liefert den Mittelwert an PC-Programm zurueck ---- *}
Procedure Mittel(task, insize : Word; VAR inptr; outsize : Word; VAR outptr);

VAR m : LONGINT;             {Mittelwert}
    i : WORD;                {Laufvariable}
    error   : WORD;          {Fehlercode}
    dataptr : ^BYTE;

begin
  ml8rt_entry_function;      {Retten der Register}
  m := 0;                    {Mittelwert initialisieren}
  IF insize>0 THEN           {wurde mindestens ein Byte uebergeben?}
    BEGIN
      dataptr := @inptr;         {Zeiger auf Anfang der ueberg. Daten setzen}

      FOR i := 1 TO insize DO    {Schleife fuer Anzahl der uebergebenen Bytes}
      begin
        m := m + dataptr^;             {Uebergebene Bytes summieren}
        INC (dataptr);                 {Zeiger auf Daten wird erhoeht}
      end;
      m := m DIV insize;               {Mittelwert m bilden}
      move(m, outptr, outsize);        {Mittelwert wird nach outptr kopiert}
      error := 0;                      {Kein Fehler}
    END
  ELSE
    error := 1;                        {Fehler: keine Daten uebergeben}

  ml8rt_exit_function(0,1,error);      {Funktionsaufruf beenden:            }
                                       { Bedeutung der Parameter:           }
                                       { 0 : Alle uebernommenen Bytes genutzt}
                                       { 1 : Ein Byte wird zurueckgegeben    }
                                       { error : Fehlermeldung              }
END;



begin                        { ***** PREPARE ***** }
   PDT.PDT_HEAD.ucType := 1;                    { Programm-Descriptor-Tabellen-Typ
                                                  (im Moment wird nur der Typ 1 unterstuetzt) }
   PDT.PDT_HEAD.ucSize := sizeof(ML8_PDT_HEAD); { Gre des Vorspanns der Kopftabelle (im Moment immer = _PDT_Length = 48) }
   PDT.PDT_HEAD.usGlobalProc := 3;              { Anzahl der Prozeduren }
   PDT.PDT_HEAD.usProgNo := Programm_Number;    { Programm-Nummer }
   PDT.PDT_HEAD.cVersion := Version;            { Programm-Version }
   PDT.PDT_HEAD.cRevision := Revision;          { Programm-Revision }
   PDT.PDT_HEAD.ucCPU := _486DX;                { CPU-Typ }
   PDT.PDT_HEAD.ucCoProc := _487DX;             { Co-Prozessor-Typ }
   PDT.PDT_HEAD.ucLanguage := _TP70;            { Programmiersprache }
   PDT.PDT_HEAD.ucProgType := _User_Prg;        { Programm-Typ: Anwenderprogramm }
   PDT.PDT_HEAD.usFlags := $0;                  { Flags }
   PDT.PDT_HEAD.usFlags := _NI_Task + _PDT_Info_Enable + _LOCAL_PARAMETER;
   PDT.PDT_HEAD.usInt := 0;                     { Interrupt-Nummer fuer die das Programm gedacht ist (ungueltig, da NI-Task) }
   PDT.PDT_HEAD.ulDataAdr := 0;                 { Adresse des Datenbereichs }
   PDT.PDT_HEAD.ulDataSize := 0;                { Gre des Datenbereichs }
   PDT.PDT_HEAD.ulDataSizeMin := 0;             { Minimaler Datenbereich }
   PDT.PDT_HEAD.ulDataSizeMax := 0;             { Maximaler Datenbereich }
					        { Anfangsadresse des Parameterbereichs }
   ml8rt_phys_adr (@Parameter, PDT.PDT_HEAD.ulGlobalParAdr);
   PDT.PDT_HEAD.ulGlobalParAdr := PDT.PDT_HEAD.ulGlobalParAdr + SizeOf(TDT_Type);
					        { Anzahl der Parameter }
   PDT.PDT_HEAD.usGlobalParSize := SizeOf(Parameter) - SizeOf(TDT_Type);


   PDT.PDT_HEAD.ulHyperAdr := 0;                { Adresse des Hypertextes (wird nicht benutzt) }
   PDT.PDT_HEAD.usRes1 := 0;
   PDT.PDT_HEAD.ulRes2 := 0;
   PDT.Main := @Main_Proc;                      { Adresse der Hauptprozedur }
   PDT.Auto_Init := @Auto_Init;                 { Adresse der Auto-Init-Prozedur }
   PDT.Mittel := @Mittel;                       { Adresse der Mittelwertbildung }
   ml8rt_Set_PDT_Adr (PDT);                     { Adresse der PDT im Rueckgabe-Record dem Betriebssystem uebergeben }
end.
